@@ -3,10 +3,10 @@
 _PATH='scripts/packages.txt'
 while IFS= read -r _PACKAGE; do
 	uv pip compile \
-		"--extra=zzz-test-defaults" \
+		"--extra=test" \
 		"--extra=zzz-test-${_PACKAGE}" \
 		--quiet \
-		--prerelease=disallow \
+		--prerelease=explicit \
 		"--output-file=requirements/${_PACKAGE}.txt" \
 		--upgrade \
 		--python-version=3.11 \
