@@ -9,6 +9,8 @@ from zoneinfo import ZoneInfo
 
 from hypothesis import HealthCheck, assume, given, settings
 from hypothesis.strategies import (
+    DataObject,
+    data,
     dates,
     datetimes,
     floats,
@@ -28,6 +30,8 @@ from utilities.datetime import (
     EPOCH_UTC,
     HALF_YEAR,
     HOUR,
+    MICROSECOND,
+    MILLISECOND,
     MINUTE,
     MONTH,
     NOW_HK,
@@ -43,6 +47,7 @@ from utilities.datetime import (
     AddWeekdaysError,
     CheckDateNotDatetimeError,
     CheckZonedDatetimeError,
+    EnsureMonthError,
     MillisecondsSinceEpochError,
     Month,
     MonthError,
@@ -61,6 +66,7 @@ from utilities.datetime import (
     drop_milli_and_microseconds,
     duration_to_float,
     duration_to_timedelta,
+    ensure_month,
     format_datetime_local_and_utc,
     get_half_years,
     get_months,
@@ -73,10 +79,11 @@ from utilities.datetime import (
     get_today_tokyo,
     get_years,
     is_equal_mod_tz,
+    is_instance_date_not_datetime,
     is_local_datetime,
+    is_subclass_date_not_datetime,
     is_weekday,
     is_zoned_datetime,
-    isinstance_date_not_datetime,
     maybe_sub_pct_y,
     microseconds_since_epoch,
     microseconds_since_epoch_to_datetime,
@@ -94,8 +101,8 @@ from utilities.datetime import (
     yield_days,
     yield_weekdays,
 )
-from utilities.hypothesis import assume_does_not_raise, longs, months, text_clean
-from utilities.zoneinfo import HONG_KONG, TOKYO, US_CENTRAL, US_EASTERN, UTC
+from utilities.hypothesis import assume_does_not_raise, int32s, months, text_clean
+from utilities.zoneinfo import UTC, HongKong, Tokyo, USCentral, USEastern
 
 if TYPE_CHECKING:
     from collections.abc import Callable
@@ -151,7 +158,7 @@ class TestCheckDateNotDatetime:
 
 
 class TestCheckZonedDatetime:
-    @given(datetime=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])))
+    @given(datetime=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])))
     def test_date(self, *, datetime: dt.datetime) -> None:
         check_zoned_datetime(datetime)
 
@@ -238,19 +245,19 @@ class TestFormatDatetimeLocalAndUTC:
                 "2000-01-01 02:03:04 (Sat, UTC)",
             ),
             param(
-                dt.datetime(2000, 1, 1, 2, 3, 4, tzinfo=HONG_KONG),
+                dt.datetime(2000, 1, 1, 2, 3, 4, tzinfo=HongKong),
                 "2000-01-01 02:03:04 (Sat, Asia/Hong_Kong, 1999-12-31 18:03:04 UTC)",
             ),
             param(
-                dt.datetime(2000, 2, 1, 2, 3, 4, tzinfo=HONG_KONG),
+                dt.datetime(2000, 2, 1, 2, 3, 4, tzinfo=HongKong),
                 "2000-02-01 02:03:04 (Tue, Asia/Hong_Kong, 01-31 18:03:04 UTC)",
             ),
             param(
-                dt.datetime(2000, 2, 2, 2, 3, 4, tzinfo=HONG_KONG),
+                dt.datetime(2000, 2, 2, 2, 3, 4, tzinfo=HongKong),
                 "2000-02-02 02:03:04 (Wed, Asia/Hong_Kong, 02-01 18:03:04 UTC)",
             ),
             param(
-                dt.datetime(2000, 2, 2, 14, 3, 4, tzinfo=HONG_KONG),
+                dt.datetime(2000, 2, 2, 14, 3, 4, tzinfo=HongKong),
                 "2000-02-02 14:03:04 (Wed, Asia/Hong_Kong, 06:03:04 UTC)",
             ),
         ],
@@ -271,7 +278,7 @@ class TestGetNow:
         now = get_now(time_zone="local")
         assert isinstance(now, dt.datetime)
         ETC = ZoneInfo("Etc/UTC")  # noqa: N806
-        assert now.tzinfo in {ETC, HONG_KONG, TOKYO, UTC}
+        assert now.tzinfo in {ETC, HongKong, Tokyo, UTC}
 
     @mark.parametrize(
         "get_now", [param(get_now), param(get_now_hk), param(get_now_tokyo)]
@@ -327,11 +334,11 @@ class TestGetToday:
 class TestIsInstanceDateNotDatetime:
     @given(date=dates())
     def test_date(self, *, date: dt.date) -> None:
-        assert isinstance_date_not_datetime(date)
+        assert is_instance_date_not_datetime(date)
 
     @given(datetime=datetimes())
     def test_datetime(self, *, datetime: dt.datetime) -> None:
-        assert not isinstance_date_not_datetime(datetime)
+        assert not is_instance_date_not_datetime(datetime)
 
 
 class TestIsEqualModTz:
@@ -366,6 +373,16 @@ class TestIsLocalDateTime:
         assert result is expected
 
 
+class TestIsSubClassDateNotDatetime:
+    @given(date=dates())
+    def test_date(self, *, date: dt.date) -> None:
+        assert is_subclass_date_not_datetime(type(date))
+
+    @given(datetime=datetimes())
+    def test_datetime(self, *, datetime: dt.datetime) -> None:
+        assert not is_subclass_date_not_datetime(type(datetime))
+
+
 class TestIsWeekday:
     @given(date=dates())
     def test_main(self, *, date: dt.date) -> None:
@@ -460,6 +477,22 @@ class TestMonth:
         result = month + n
         assert result == expected
 
+    @mark.parametrize(
+        ("x", "y", "expected"),
+        [
+            param(Month(2000, 1), Month(1999, 11), 2),
+            param(Month(2000, 1), Month(1999, 12), 1),
+            param(Month(2000, 1), Month(2000, 1), 0),
+            param(Month(2000, 1), Month(2000, 2), -1),
+            param(Month(2000, 1), Month(2000, 3), -2),
+            param(Month(2000, 1), Month(2000, 12), -11),
+            param(Month(2000, 1), Month(2001, 1), -12),
+        ],
+    )
+    def test_diff(self, *, x: Month, y: Month, expected: int) -> None:
+        result = x - y
+        assert result == expected
+
     @given(month=months())
     def test_hashable(self, *, month: Month) -> None:
         _ = hash(month)
@@ -508,6 +541,16 @@ class TestParseAndSerializeMonth:
         with raises(ParseMonthError, match="Unable to parse month; got 'invalid'"):
             _ = parse_month("invalid")
 
+    @given(data=data(), month=months())
+    def test_ensure(self, *, data: DataObject, month: Month) -> None:
+        str_or_value = data.draw(sampled_from([month, serialize_month(month)]))
+        result = ensure_month(str_or_value)
+        assert result == month
+
+    def test_error_ensure(self) -> None:
+        with raises(EnsureMonthError, match="Unable to ensure month; got 'invalid'"):
+            _ = ensure_month("invalid")
+
 
 class TestPeriod:
     @given(start=dates(), end=dates())
@@ -516,16 +559,18 @@ class TestPeriod:
         period = Period(start, end)
         assert period.start == start
         assert period.end == end
+        _ = hash(period)
 
     @given(
-        start=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])),
-        end=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])),
+        start=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])),
+        end=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])),
     )
     def test_datetimes(self, *, start: dt.datetime, end: dt.datetime) -> None:
         _ = assume(start <= end)
         period = Period(start, end)
         assert period.start == start
         assert period.end == end
+        _ = hash(period)
 
     @given(start=dates(), days=integers(min_value=0))
     def test_duration(self, *, start: dt.date, days: int) -> None:
@@ -538,7 +583,7 @@ class TestPeriod:
 
     @given(
         start=dates(),
-        end=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC]) | none()),
+        end=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC]) | none()),
     )
     def test_error_date_and_datetime_mix(
         self, *, start: dt.date, end: dt.datetime
@@ -549,8 +594,8 @@ class TestPeriod:
             _ = Period(start, end)
 
     @given(
-        start=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC]) | none()),
-        end=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC]) | none()),
+        start=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC]) | none()),
+        end=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC]) | none()),
     )
     def test_error_naive_datetime(
         self, *, start: dt.datetime, end: dt.datetime
@@ -568,8 +613,8 @@ class TestPeriod:
             _ = Period(start, end)
 
     @given(
-        start=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])),
-        end=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])),
+        start=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])),
+        end=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])),
     )
     def test_error_invalid_datetimes(
         self, *, start: dt.datetime, end: dt.datetime
@@ -606,15 +651,15 @@ class TestRoundToWeekday:
 
 
 class TestTimedeltaSinceEpoch:
-    @given(datetime=datetimes(timezones=sampled_from([HONG_KONG, UTC, dt.UTC])))
+    @given(datetime=datetimes(timezones=sampled_from([HongKong, UTC, dt.UTC])))
     def test_main(self, *, datetime: dt.datetime) -> None:
         result = timedelta_since_epoch(datetime)
         assert isinstance(result, dt.timedelta)
 
     @given(
         datetime=datetimes(timezones=just(UTC)),
-        time_zone1=sampled_from([HONG_KONG, TOKYO, US_CENTRAL, US_EASTERN, UTC]),
-        time_zone2=sampled_from([HONG_KONG, TOKYO, US_CENTRAL, US_EASTERN, UTC]),
+        time_zone1=sampled_from([HongKong, Tokyo, USCentral, USEastern, UTC]),
+        time_zone2=sampled_from([HongKong, Tokyo, USCentral, USEastern, UTC]),
     )
     def test_time_zone(
         self, *, datetime: dt.datetime, time_zone1: ZoneInfo, time_zone2: ZoneInfo
@@ -662,7 +707,7 @@ class TestTimedeltaToMicrosecondsOrMilliseconds:
         ):
             _ = timedelta_to_milliseconds(timedelta, strict=True)
 
-    @given(milliseconds=longs())
+    @given(milliseconds=int32s())
     def test_milliseconds_to_timedelta(self, *, milliseconds: int) -> None:
         with assume_does_not_raise(OverflowError):
             timedelta = milliseconds_to_timedelta(milliseconds)
@@ -673,7 +718,15 @@ class TestTimedeltaToMicrosecondsOrMilliseconds:
 class TestTimedeltas:
     @mark.parametrize(
         "timedelta",
-        [param(SECOND), param(MINUTE), param(HOUR), param(DAY), param(WEEK)],
+        [
+            param(MICROSECOND),
+            param(MILLISECOND),
+            param(SECOND),
+            param(MINUTE),
+            param(HOUR),
+            param(DAY),
+            param(WEEK),
+        ],
     )
     def test_main(self, *, timedelta: dt.timedelta) -> None:
         assert isinstance(timedelta, dt.timedelta)
@@ -687,20 +740,22 @@ class TestTimeZones:
 class TestYieldDays:
     @given(start=dates(), days=integers(0, 365))
     def test_start_and_end(self, *, start: dt.date, days: int) -> None:
-        with assume_does_not_raise(OverflowError):
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
             end = start + dt.timedelta(days=days)
             dates = list(yield_days(start=start, end=end))
         assert all(start <= d <= end for d in dates)
 
     @given(start=dates(), days=integers(0, 10))
     def test_start_and_days(self, *, start: dt.date, days: int) -> None:
-        dates = list(yield_days(start=start, days=days))
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            dates = list(yield_days(start=start, days=days))
         assert len(dates) == days
         assert all(d >= start for d in dates)
 
     @given(end=dates(), days=integers(0, 10))
     def test_end_and_days(self, *, end: dt.date, days: int) -> None:
-        dates = list(yield_days(end=end, days=days))
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            dates = list(yield_days(end=end, days=days))
         assert len(dates) == days
         assert all(d <= end for d in dates)
 
@@ -714,7 +769,7 @@ class TestYieldDays:
 class TestYieldWeekdays:
     @given(start=dates(), days=integers(0, 365))
     def test_start_and_end(self, *, start: dt.date, days: int) -> None:
-        with assume_does_not_raise(OverflowError):
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
             end = start + dt.timedelta(days=days)
         dates = list(yield_weekdays(start=start, end=end))
         assert all(start <= d <= end for d in dates)
@@ -726,14 +781,16 @@ class TestYieldWeekdays:
 
     @given(start=dates(), days=integers(0, 10))
     def test_start_and_days(self, *, start: dt.date, days: int) -> None:
-        dates = list(yield_weekdays(start=start, days=days))
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            dates = list(yield_weekdays(start=start, days=days))
         assert len(dates) == days
         assert all(d >= start for d in dates)
         assert all(map(is_weekday, dates))
 
     @given(end=dates(), days=integers(0, 10))
     def test_end_and_days(self, *, end: dt.date, days: int) -> None:
-        dates = list(yield_weekdays(end=end, days=days))
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            dates = list(yield_weekdays(end=end, days=days))
         assert len(dates) == days
         assert all(d <= end for d in dates)
         assert all(map(is_weekday, dates))
