@@ -1,12 +1,12 @@
 from __future__ import annotations
 
+import datetime as dt
 from itertools import pairwise
 from pathlib import Path
 from re import search
 from subprocess import PIPE, check_output
 from typing import TYPE_CHECKING, Any, cast
 
-import redis.asyncio
 from hypothesis import HealthCheck, Phase, assume, given, settings
 from hypothesis.errors import InvalidArgument
 from hypothesis.extra.numpy import array_shapes
@@ -16,116 +16,91 @@ from hypothesis.strategies import (
     booleans,
     composite,
     data,
-    dates,
     datetimes,
     floats,
     integers,
     just,
     none,
+    sampled_from,
     sets,
     timedeltas,
+    timezones,
+    uuids,
 )
-from luigi import Task
-from numpy import (
-    iinfo,
-    inf,
-    int32,
-    int64,
-    isfinite,
-    isinf,
-    isnan,
-    ravel,
-    rint,
-    uint32,
-    uint64,
-    zeros,
-)
-from pandas import Timestamp
-from pandas.testing import assert_index_equal
+from numpy import inf, int64, isfinite, isinf, isnan, ravel, rint
 from pytest import mark, param, raises
-from semver import Version
-from sqlalchemy import Column, Engine, Integer, MetaData, Select, Table, select
+from sqlalchemy import Column, Integer, MetaData, Table, insert, select
 from sqlalchemy.ext.asyncio import AsyncEngine
-from sqlalchemy.orm import DeclarativeBase, MappedAsDataclass
 
-from tests.conftest import FLAKY, SKIPIF_CI_AND_NOT_LINUX
+from tests.conftest import FLAKY, SKIPIF_CI_AND_NOT_LINUX, SKIPIF_CI_AND_WINDOWS
+from utilities.datetime import duration_to_float, is_local_datetime, is_zoned_datetime
 from utilities.git import _GET_BRANCH_NAME
 from utilities.hypothesis import (
+    _SQLALCHEMY_ENGINE_DIALECTS,
+    _ZONED_DATETIMES_LEFT_MOST,
+    _ZONED_DATETIMES_RIGHT_MOST,
     Shape,
-    _merge_into_dict_of_indexes,
-    aiosqlite_engines,
     assume_does_not_raise,
     bool_arrays,
-    bool_data_arrays,
-    concatenated_arrays,
-    dates_pd,
-    datetimes_pd,
-    datetimes_utc,
-    dicts_of_indexes,
+    durations,
     float_arrays,
-    float_data_arrays,
     floats_extra,
     git_repos,
     hashables,
-    indexes,
     int32s,
     int64s,
     int_arrays,
-    int_data_arrays,
-    int_indexes,
+    lift_data,
+    lift_draw,
     lists_fixed_length,
-    longs,
     months,
-    namespace_mixins,
-    redis_clients,
-    redis_time_series,
+    random_states,
+    sets_fixed_length,
     settings_with_reduced_examples,
     setup_hypothesis_profiles,
     slices,
-    sqlite_engines,
+    sqlalchemy_engines,
     str_arrays,
-    str_data_arrays,
-    str_indexes,
     temp_dirs,
     temp_paths,
     text_ascii,
     text_clean,
+    text_digits,
     text_printable,
     timedeltas_2w,
-    timestamps,
     uint32s,
     uint64s,
-    versions,
+    yield_test_redis,
+    zoned_datetimes,
+)
+from utilities.math import (
+    MAX_INT32,
+    MAX_INT64,
+    MAX_UINT32,
+    MAX_UINT64,
+    MIN_INT32,
+    MIN_INT64,
+    MIN_UINT32,
+    MIN_UINT64,
 )
-from utilities.math import MAX_LONG, MIN_LONG
 from utilities.os import temp_environ
-from utilities.pandas import (
-    TIMESTAMP_MAX_AS_DATE,
-    TIMESTAMP_MAX_AS_DATETIME,
-    TIMESTAMP_MIN_AS_DATE,
-    TIMESTAMP_MIN_AS_DATETIME,
-    IndexA,
-    string,
-)
-from utilities.pathvalidate import valid_path
 from utilities.platform import maybe_yield_lower_case
-from utilities.sqlalchemy import get_table, insert_items, insert_items_async
+from utilities.sqlalchemy import Dialect, _get_dialect
+from utilities.types import Duration, Number, make_isinstance
 from utilities.whenever import (
     MAX_TWO_WAY_TIMEDELTA,
     MIN_TWO_WAY_TIMEDELTA,
+    check_valid_zoned_datetime,
+    parse_duration,
     parse_timedelta,
+    serialize_duration,
     serialize_timedelta,
 )
-from utilities.zoneinfo import UTC
 
 if TYPE_CHECKING:
-    import datetime as dt
-    from collections.abc import Hashable, Mapping, Sequence
     from collections.abc import Set as AbstractSet
     from uuid import UUID
-
-    import redis
-    from redis.commands.timeseries import TimeSeries
+    from zoneinfo import ZoneInfo
 
     from utilities.datetime import Month
     from utilities.tempfile import TemporaryDirectory
@@ -174,120 +149,70 @@ class TestBoolArrays:
         assert array.shape == shape
 
 
-class TestBoolDataArrays:
-    @given(data=data(), indexes=dicts_of_indexes(), name=text_ascii() | none())
-    def test_main(
-        self, *, data: DataObject, indexes: Mapping[str, IndexA], name: str | None
-    ) -> None:
-        array = data.draw(bool_data_arrays(indexes, name=name))
-        assert set(array.coords) == set(indexes)
-        assert array.dims == tuple(indexes)
-        assert array.dtype == bool
-        assert array.name == name
-        for arr, exp in zip(array.indexes.values(), indexes.values(), strict=True):
-            assert_index_equal(arr, exp, check_names=False)
-
-
-class TestConcatenatedArrays:
-    @given(data=data(), m=integers(0, 10), n=integers(0, 10))
-    def test_1d(self, *, data: DataObject, m: int, n: int) -> None:
-        arrays = just(zeros(n, dtype=float))
-        array = data.draw(concatenated_arrays(arrays, m, n))
-        assert array.shape == (m, n)
-
-    @given(data=data(), m=integers(0, 10), n=integers(0, 10), p=integers(0, 10))
-    def test_2d(self, *, data: DataObject, m: int, n: int, p: int) -> None:
-        arrays = just(zeros((n, p), dtype=float))
-        array = data.draw(concatenated_arrays(arrays, m, (n, p)))
-        assert array.shape == (m, n, p)
-
-
-class TestDatesPd:
+class TestDurations:
     @given(
         data=data(),
-        min_value=dates(
-            min_value=TIMESTAMP_MIN_AS_DATE, max_value=TIMESTAMP_MAX_AS_DATE
-        ),
-        max_value=dates(
-            min_value=TIMESTAMP_MIN_AS_DATE, max_value=TIMESTAMP_MAX_AS_DATE
-        ),
+        min_number=integers() | floats() | none(),
+        max_number=integers() | floats() | none(),
+        min_timedelta=timedeltas(),
+        max_timedelta=timedeltas(),
     )
     @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_main(
-        self, *, data: DataObject, min_value: dt.date, max_value: dt.date
-    ) -> None:
-        _ = assume(min_value <= max_value)
-        date = data.draw(dates_pd(min_value=min_value, max_value=max_value))
-        _ = Timestamp(date)
-        assert min_value <= date <= max_value
-
-
-class TestDatetimesPd:
-    @given(
-        data=data(),
-        min_value=datetimes_utc(
-            min_value=TIMESTAMP_MIN_AS_DATETIME, max_value=TIMESTAMP_MAX_AS_DATETIME
-        ),
-        max_value=datetimes_utc(
-            min_value=TIMESTAMP_MIN_AS_DATETIME, max_value=TIMESTAMP_MAX_AS_DATETIME
-        ),
-    )
-    @settings(suppress_health_check={HealthCheck.filter_too_much})
-    def test_main(
-        self, *, data: DataObject, min_value: dt.datetime, max_value: dt.datetime
-    ) -> None:
-        _ = assume(min_value <= max_value)
-        datetime = data.draw(datetimes_pd(min_value=min_value, max_value=max_value))
-        _ = Timestamp(datetime)
-        assert min_value <= datetime <= max_value
-
-
-class TestDatetimesUTC:
-    @given(data=data(), min_value=datetimes(), max_value=datetimes())
-    def test_main(
-        self, *, data: DataObject, min_value: dt.datetime, max_value: dt.datetime
-    ) -> None:
-        min_value, max_value = (v.replace(tzinfo=UTC) for v in [min_value, max_value])
-        _ = assume(min_value <= max_value)
-        datetime = data.draw(datetimes_utc(min_value=min_value, max_value=max_value))
-        assert min_value <= datetime <= max_value
-
-
-class TestDictsOfIndexes:
-    @given(
-        data=data(),
-        min_dims=integers(1, 3),
-        max_dims=integers(1, 3) | none(),
-        min_side=integers(1, 10),
-        max_side=integers(1, 10) | none(),
-    )
     def test_main(
         self,
         *,
         data: DataObject,
-        min_dims: int,
-        max_dims: int | None,
-        min_side: int,
-        max_side: int | None,
+        min_number: Number | None,
+        max_number: Number | None,
+        min_timedelta: dt.timedelta,
+        max_timedelta: dt.timedelta,
     ) -> None:
         with assume_does_not_raise(InvalidArgument):
-            indexes = data.draw(
-                dicts_of_indexes(
-                    min_dims=min_dims,
-                    max_dims=max_dims,
-                    min_side=min_side,
-                    max_side=max_side,
+            x = data.draw(
+                durations(
+                    min_number=min_number,
+                    max_number=max_number,
+                    min_timedelta=min_timedelta,
+                    max_timedelta=max_timedelta,
                 )
             )
-        ndims = len(indexes)
-        assert ndims >= min_dims
-        if max_dims is not None:
-            assert ndims <= max_dims
-        for index in indexes.values():
-            length = len(index)
-            assert length >= min_side
-            if max_side is not None:
-                assert length <= max_side
+        assert isinstance(x, Duration)
+        if isinstance(x, int):
+            if isinstance(min_number, int):
+                assert x >= min_number
+            if isinstance(max_number, int):
+                assert x <= max_number
+        elif isinstance(x, float):
+            if min_number is not None:
+                assert x >= min_number
+            if max_number is not None:
+                assert x <= max_number
+        else:
+            assert min_timedelta <= x <= max_timedelta
+
+    @given(data=data())
+    @settings(suppress_health_check={HealthCheck.filter_too_much})
+    def test_int_and_float_bounds(self, *, data: DataObject) -> None:
+        min_number = data.draw(integers(-10, 0))
+        max_number = data.draw(floats(0.0, 10.0))
+        duration = data.draw(durations(min_number=min_number, max_number=max_number))
+        _ = assume(isinstance(duration, int | float))
+        as_float = duration_to_float(duration)
+        assert min_number <= as_float <= max_number
+
+    @given(
+        data=data(),
+        min_value=durations(two_way=True).filter(make_isinstance(dt.timedelta)),
+        max_value=durations(two_way=True).filter(make_isinstance(dt.timedelta)),
+    )
+    @settings(suppress_health_check={HealthCheck.filter_too_much})
+    def test_two_way(
+        self, *, data: DataObject, min_value: dt.timedelta, max_value: dt.timedelta
+    ) -> None:
+        _ = assume(min_value <= max_value)
+        duration = data.draw(durations(two_way=True))
+        ser = serialize_duration(duration)
+        _ = parse_duration(ser)
 
 
 class TestFloatArrays:
@@ -353,58 +278,6 @@ class TestFloatArrays:
             assert len(set(flat)) == len(flat)
 
 
-class TestFloatDataArrays:
-    @given(
-        data=data(),
-        indexes=dicts_of_indexes(),
-        min_value=floats() | none(),
-        max_value=floats() | none(),
-        allow_nan=booleans(),
-        allow_inf=booleans(),
-        allow_pos_inf=booleans(),
-        allow_neg_inf=booleans(),
-        integral=booleans(),
-        unique=booleans(),
-        name=text_ascii() | none(),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        indexes: Mapping[str, IndexA],
-        min_value: float | None,
-        max_value: float | None,
-        allow_nan: bool,
-        allow_inf: bool,
-        allow_pos_inf: bool,
-        allow_neg_inf: bool,
-        integral: bool,
-        unique: bool,
-        name: str | None,
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            array = data.draw(
-                float_data_arrays(
-                    indexes,
-                    min_value=min_value,
-                    max_value=max_value,
-                    allow_nan=allow_nan,
-                    allow_inf=allow_inf,
-                    allow_pos_inf=allow_pos_inf,
-                    allow_neg_inf=allow_neg_inf,
-                    integral=integral,
-                    unique=unique,
-                    name=name,
-                )
-            )
-        assert set(array.coords) == set(indexes)
-        assert array.dims == tuple(indexes)
-        assert array.dtype == float
-        assert array.name == name
-        for arr, exp in zip(array.indexes.values(), indexes.values(), strict=True):
-            assert_index_equal(arr, exp, check_names=False)
-
-
 class TestFloatsExtra:
     @given(
         data=data(),
@@ -499,74 +372,12 @@ class TestHashables:
         _ = hash(x)
 
 
-class TestIndexes:
-    @given(
-        data=data(),
-        n=integers(0, 10),
-        unique=booleans(),
-        name=hashables(),
-        sort=booleans(),
-    )
-    def test_generic(
-        self, *, data: DataObject, n: int, unique: bool, name: Hashable, sort: bool
-    ) -> None:
-        index = data.draw(
-            indexes(
-                elements=int64s(), dtype=int64, n=n, unique=unique, name=name, sort=sort
-            )
-        )
-        assert len(index) == n
-        if unique:
-            assert not index.duplicated().any()
-        assert index.name == name
-        if sort:
-            assert_index_equal(index, index.sort_values())
-
-    @given(
-        data=data(),
-        n=integers(0, 10),
-        unique=booleans(),
-        name=hashables(),
-        sort=booleans(),
-    )
-    def test_int(
-        self, *, data: DataObject, n: int, unique: bool, name: Hashable, sort: bool
-    ) -> None:
-        index = data.draw(int_indexes(n=n, unique=unique, name=name, sort=sort))
-        assert index.dtype == int64
-        assert len(index) == n
-        if unique:
-            assert not index.duplicated().any()
-        assert index.name == name
-        if sort:
-            assert_index_equal(index, index.sort_values())
-
-    @given(
-        data=data(),
-        n=integers(0, 10),
-        unique=booleans(),
-        name=hashables(),
-        sort=booleans(),
-    )
-    def test_str(
-        self, *, data: DataObject, n: int, unique: bool, name: Hashable, sort: bool
-    ) -> None:
-        index = data.draw(str_indexes(n=n, unique=unique, name=name, sort=sort))
-        assert index.dtype == string
-        assert len(index) == n
-        if unique:
-            assert not index.duplicated().any()
-        assert index.name == name
-        if sort:
-            assert_index_equal(index, index.sort_values())
-
-
 class TestIntArrays:
     @given(
         data=data(),
         shape=array_shapes(),
-        min_value=int64s() | none(),
-        max_value=int64s() | none(),
+        min_value=int64s(),
+        max_value=int64s(),
         unique=booleans(),
     )
     def test_main(
@@ -574,8 +385,8 @@ class TestIntArrays:
         *,
         data: DataObject,
         shape: Shape,
-        min_value: int | None,
-        max_value: int | None,
+        min_value: int,
+        max_value: int,
         unique: bool,
     ) -> None:
         with assume_does_not_raise(InvalidArgument):
@@ -592,91 +403,61 @@ class TestIntArrays:
 
 
 class TestInt32s:
-    @given(data=data(), min_value=int32s() | none(), max_value=int32s() | none())
-    def test_main(
-        self, *, data: DataObject, min_value: int | None, max_value: int | None
-    ) -> None:
+    @given(data=data(), min_value=int32s(), max_value=int32s())
+    def test_main(self, *, data: DataObject, min_value: int, max_value: int) -> None:
         with assume_does_not_raise(InvalidArgument):
             x = data.draw(int32s(min_value=min_value, max_value=max_value))
-        info = iinfo(int32)
-        assert info.min <= x <= info.max
-        if min_value is not None:
-            assert x >= min_value
-        if max_value is not None:
-            assert x <= max_value
+        assert max(min_value, MIN_INT32) <= x <= min(max_value, MAX_INT32)
 
 
 class TestInt64s:
-    @given(data=data(), min_value=int64s() | none(), max_value=int64s() | none())
-    def test_main(
-        self, *, data: DataObject, min_value: int | None, max_value: int | None
-    ) -> None:
+    @given(data=data(), min_value=int64s(), max_value=int64s())
+    def test_main(self, *, data: DataObject, min_value: int, max_value: int) -> None:
         with assume_does_not_raise(InvalidArgument):
             x = data.draw(int64s(min_value=min_value, max_value=max_value))
-        info = iinfo(int64)
-        assert info.min <= x <= info.max
-        if min_value is not None:
-            assert x >= min_value
-        if max_value is not None:
-            assert x <= max_value
+        assert max(min_value, MIN_INT64) <= x <= min(max_value, MAX_INT64)
 
 
-class TestIntDataArrays:
-    @given(
-        data=data(),
-        indexes=dicts_of_indexes(),
-        min_value=int64s() | none(),
-        max_value=int64s() | none(),
-        unique=booleans(),
-        name=text_ascii() | none(),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        indexes: Mapping[str, IndexA],
-        min_value: int | None,
-        max_value: int | None,
-        unique: bool,
-        name: str | None,
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            array = data.draw(
-                int_data_arrays(
-                    indexes,
-                    min_value=min_value,
-                    max_value=max_value,
-                    unique=unique,
-                    name=name,
-                )
-            )
-        assert set(array.coords) == set(indexes)
-        assert array.dims == tuple(indexes)
-        assert array.dtype == int64
-        assert array.name == name
-        for arr, exp in zip(array.indexes.values(), indexes.values(), strict=True):
-            assert_index_equal(arr, exp, check_names=False)
+class TestLiftDataDraw:
+    @given(data=data(), value=booleans())
+    def test_fixed(self, *, data: DataObject, value: bool) -> None:
+        def strategy(_data: DataObject, /) -> bool:
+            draw = lift_data(_data)
+            return draw(value)
+
+        result = strategy(data)
+        assert result is value
+
+    @given(data=data(), value=booleans())
+    def test_strategy(self, *, data: DataObject, value: bool) -> None:
+        def strategy(_data: DataObject, /) -> bool:
+            draw = lift_data(_data)
+            return draw(just(value))
+
+        result = strategy(data)
+        assert result is value
 
 
 class TestLiftDraw:
-    @given(data=data(), x=booleans())
-    def test_fixed(self, *, data: DataObject, x: bool) -> None:
+    @given(data=data(), value=booleans())
+    def test_fixed(self, *, data: DataObject, value: bool) -> None:
         @composite
-        def func(_draw: DrawFn, /) -> bool:
-            _ = _draw(booleans())
-            return x
+        def strategy(_draw: DrawFn, /) -> bool:
+            draw = lift_draw(_draw)
+            return draw(value)
 
-        result = data.draw(func())
-        assert result is x
+        result = data.draw(strategy())
+        assert result is value
 
-    @given(data=data())
-    def test_strategy(self, *, data: DataObject) -> None:
+    @given(data=data(), value=booleans())
+    def test_strategy(self, *, data: DataObject, value: bool) -> None:
         @composite
-        def func(_draw: DrawFn, /) -> bool:
-            return _draw(booleans())
+        def strategy(_draw: DrawFn, /) -> bool:
+            draw = lift_draw(_draw)
+            return draw(just(value))
 
-        result = data.draw(func())
-        assert isinstance(result, bool)
+        result = data.draw(strategy())
+        assert result is value
 
 
 class TestListsFixedLength:
@@ -701,20 +482,6 @@ class TestListsFixedLength:
             assert sorted(result) == result
 
 
-class TestLongs:
-    @given(data=data(), min_value=longs() | none(), max_value=longs() | none())
-    def test_main(
-        self, *, data: DataObject, min_value: int | None, max_value: int | None
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            x = data.draw(longs(min_value=min_value, max_value=max_value))
-        assert MIN_LONG <= x <= MAX_LONG
-        if min_value is not None:
-            assert x >= min_value
-        if max_value is not None:
-            assert x <= max_value
-
-
 class TestMonths:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
@@ -730,49 +497,10 @@ class TestMonths:
         assert min_value <= month <= max_value
 
 
-class TestNamespaceMixins:
+class TestRandomStates:
     @given(data=data())
     def test_main(self, *, data: DataObject) -> None:
-        _ = data.draw(namespace_mixins())
-
-    @given(namespace_mixin=namespace_mixins())
-    def test_first(self, *, namespace_mixin: Any) -> None:
-        class Example(namespace_mixin, Task): ...
-
-        _ = Example()
-
-    @given(namespace_mixin=namespace_mixins())
-    def test_second(self, *, namespace_mixin: Any) -> None:
-        class Example(namespace_mixin, Task): ...
-
-        _ = Example()
-
-
-@SKIPIF_CI_AND_NOT_LINUX
-class TestRedisClients:
-    @given(client_pair=redis_clients(), key=text_ascii(), value=integers())
-    def test_main(
-        self, *, client_pair: tuple[redis.Redis, UUID], key: str, value: int
-    ) -> None:
-        client, uuid = client_pair
-        full_key = f"{uuid}_{key}"
-        _ = client.set(full_key, value)
-        result = int(cast(str, client.get(full_key)))
-        assert result == value
-
-
-@SKIPIF_CI_AND_NOT_LINUX
-class TestRedisTimeSeries:
-    @given(ts_pair=redis_time_series(), key=text_ascii(), value=longs())
-    def test_main(
-        self, *, ts_pair: tuple[TimeSeries, UUID], key: str, value: int
-    ) -> None:
-        ts, uuid = ts_pair
-        full_key = f"{uuid}_{key}"
-        _ = ts.add(full_key, "*", value)
-        res_timestamp, res_value = ts.get(full_key)
-        assert isinstance(res_timestamp, int)
-        assert res_value == value
+        _ = data.draw(random_states())
 
 
 class TestReducedExamples:
@@ -787,42 +515,12 @@ class TestReducedExamples:
         assert result == expected
 
 
-class TestSlices:
-    @given(data=data(), iter_len=integers(0, 10))
-    def test_main(self, *, data: DataObject, iter_len: int) -> None:
-        slice_len = data.draw(integers(0, iter_len) | none())
-        slice_ = data.draw(slices(iter_len, slice_len=slice_len))
-        range_slice = range(iter_len)[slice_]
-        assert all(i + 1 == j for i, j in pairwise(range_slice))
-        if slice_len is not None:
-            assert len(range_slice) == slice_len
-
-    @given(data=data(), iter_len=integers(0, 10))
-    def test_error(self, *, data: DataObject, iter_len: int) -> None:
-        with raises(
-            InvalidArgument, match=r"Slice length \d+ exceeds iterable length \d+"
-        ):
-            _ = data.draw(slices(iter_len, slice_len=iter_len + 1))
-
-
-class TestMergeIntoDictOfIndexes:
-    @given(data=data())
-    def test_empty(self, *, data: DataObject) -> None:
-        _ = data.draw(_merge_into_dict_of_indexes())
-
-    @given(
-        data=data(), indexes1=dicts_of_indexes() | none(), indexes2=dicts_of_indexes()
-    )
-    def test_non_empty(
-        self,
-        *,
-        data: DataObject,
-        indexes1: Mapping[str, IndexA] | None,
-        indexes2: Mapping[str, IndexA],
-    ) -> None:
-        indexes_ = data.draw(_merge_into_dict_of_indexes(indexes1, **indexes2))
-        expected = (set() if indexes1 is None else set(indexes1)) | set(indexes2)
-        assert set(indexes_) == expected
+class TestSetsFixedLength:
+    @given(data=data(), size=integers(1, 10))
+    def test_main(self, *, data: DataObject, size: int) -> None:
+        result = data.draw(sets_fixed_length(integers(), size))
+        assert isinstance(result, set)
+        assert len(result) == size
 
 
 class TestSetupHypothesisProfiles:
@@ -844,89 +542,54 @@ class TestSetupHypothesisProfiles:
         assert settings().max_examples == max_examples
 
 
-class TestSQLiteEngines:
-    @given(engine=sqlite_engines())
-    def test_sync(self, *, engine: Engine) -> None:
-        assert isinstance(engine, Engine)
-        database = engine.url.database
-        assert database is not None
-        assert not valid_path(database).exists()
+class TestSlices:
+    @given(data=data(), iter_len=integers(0, 10))
+    def test_main(self, *, data: DataObject, iter_len: int) -> None:
+        slice_len = data.draw(integers(0, iter_len) | none())
+        slice_ = data.draw(slices(iter_len, slice_len=slice_len))
+        range_slice = range(iter_len)[slice_]
+        assert all(i + 1 == j for i, j in pairwise(range_slice))
+        if slice_len is not None:
+            assert len(range_slice) == slice_len
 
-    @given(data=data(), ids=sets(integers(0, 10)))
-    def test_sync_table(self, *, data: DataObject, ids: set[int]) -> None:
-        metadata, table = self._metadata_and_table()
-        engine = data.draw(sqlite_engines(metadata=metadata))
-        self._run_test_sync(engine, table, ids)
+    @given(data=data(), iter_len=integers(0, 10))
+    def test_error(self, *, data: DataObject, iter_len: int) -> None:
+        with raises(
+            InvalidArgument, match=r"Slice length \d+ exceeds iterable length \d+"
+        ):
+            _ = data.draw(slices(iter_len, slice_len=iter_len + 1))
 
-    @given(data=data(), ids=sets(integers(0, 10)))
-    def test_sync_mapped_class(self, *, data: DataObject, ids: set[int]) -> None:
-        base, mapped_class = self._base_and_mapped_class()
-        engine = data.draw(sqlite_engines(base=base))
-        self._run_test_sync(engine, mapped_class, ids)
 
-    @given(data=data())
-    async def test_async(self, *, data: DataObject) -> None:
-        engine = await aiosqlite_engines(data)
-        assert isinstance(engine, AsyncEngine)
-        database = engine.url.database
-        assert database is not None
-        assert not valid_path(database).exists()
-
-    @given(data=data(), ids=sets(integers(0, 10)))
-    async def test_async_table(self, *, data: DataObject, ids: set[int]) -> None:
-        metadata, table = self._metadata_and_table()
-        engine = await aiosqlite_engines(data, metadata=metadata)
-        await self._run_test_async(engine, table, ids)
-
-    @given(data=data(), ids=sets(integers(0, 10)))
-    async def test_async_mapped_class(self, *, data: DataObject, ids: set[int]) -> None:
-        base, mapped_class = self._base_and_mapped_class()
-        engine = await aiosqlite_engines(data, base=base)
-        await self._run_test_async(engine, mapped_class, ids)
-
-    def _metadata_and_table(self) -> tuple[MetaData, Table]:
-        metadata = MetaData()
-        table = Table("example", metadata, Column("id_", Integer, primary_key=True))
-        return metadata, table
-
-    def _base_and_mapped_class(self) -> tuple[type[Any], type[Any]]:
-        class Base(DeclarativeBase, MappedAsDataclass): ...  # pyright: ignore[reportUnsafeMultipleInheritance]
-
-        class Example(Base):
-            __tablename__ = "example"
-
-            id_ = Column(Integer, primary_key=True)
-
-        return Base, Example
-
-    def _run_test_sync(
-        self, engine: Engine, table_or_mapped_class: Table | type[Any], ids: set[int], /
+class TestSQLAlchemyEngines:
+    @FLAKY
+    @given(
+        data=data(),
+        name=uuids(),
+        dialect=_SQLALCHEMY_ENGINE_DIALECTS,
+        ids=sets(integers(0, 10), min_size=1),
+    )
+    @settings(phases={Phase.generate})
+    async def test_main(
+        self, *, data: DataObject, name: UUID, dialect: Dialect, ids: set[int]
     ) -> None:
-        insert_items(engine, ([(id_,) for id_ in ids], table_or_mapped_class))
-        sel = self._get_select(table_or_mapped_class)
-        with engine.begin() as conn:
-            res = conn.execute(sel).scalars().all()
-        self._assert_results(res, ids)
-
-    async def _run_test_async(
-        self,
-        engine: AsyncEngine,
-        table_or_mapped_class: Table | type[Any],
-        ids: set[int],
-        /,
-    ) -> None:
-        await insert_items_async(
-            engine, ([(id_,) for id_ in ids], table_or_mapped_class)
+        table = Table(
+            f"test_{name}", MetaData(), Column("id_", Integer, primary_key=True)
         )
-        sel = self._get_select(table_or_mapped_class)
+        engine = await sqlalchemy_engines(data, table, dialect=dialect)
+        assert isinstance(engine, AsyncEngine)
+        assert _get_dialect(engine) == dialect
+        if dialect == "sqlite":
+            database = engine.url.database
+            assert database is not None
+            assert not Path(database).exists()
         async with engine.begin() as conn:
-            res = (await conn.execute(sel)).scalars().all()
-        self._assert_results(res, ids)
-
-    def _get_select(self, table_or_mapped_class: Table | type[Any], /) -> Select[Any]:
-        return select(get_table(table_or_mapped_class).c["id_"])
-
-    def _assert_results(self, results: Sequence[Any], ids: set[int], /) -> None:
+            await conn.run_sync(table.metadata.create_all)
+        ins = insert(table).values([(id_,) for id_ in ids])
+        async with engine.begin() as conn:
+            _ = await conn.execute(ins)
+        sel = select(table.c["id_"])
+        async with engine.begin() as conn:
+            results = (await conn.execute(sel)).scalars().all()
         assert set(results) == ids
 
 
@@ -973,46 +636,6 @@ class TestStrArrays:
             assert len(set(flat)) == len(flat)
 
 
-class TestStrDataArrays:
-    @given(
-        data=data(),
-        indexes=dicts_of_indexes(),
-        min_size=integers(0, 100),
-        max_size=integers(0, 100) | none(),
-        allow_none=booleans(),
-        unique=booleans(),
-        name=text_ascii() | none(),
-    )
-    def test_main(
-        self,
-        *,
-        data: DataObject,
-        indexes: Mapping[str, IndexA],
-        min_size: int,
-        max_size: int | None,
-        allow_none: bool,
-        unique: bool,
-        name: str | None,
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            array = data.draw(
-                str_data_arrays(
-                    indexes,
-                    min_size=min_size,
-                    max_size=max_size,
-                    allow_none=allow_none,
-                    unique=unique,
-                    name=name,
-                )
-            )
-        assert set(array.coords) == set(indexes)
-        assert array.dims == tuple(indexes)
-        assert array.dtype == object
-        assert array.name == name
-        for arr, exp in zip(array.indexes.values(), indexes.values(), strict=True):
-            assert_index_equal(arr, exp, check_names=False)
-
-
 class TestTempDirs:
     @given(temp_dir=temp_dirs())
     def test_main(self, *, temp_dir: TemporaryDirectory) -> None:
@@ -1107,6 +730,35 @@ class TestTextClean:
             assert text != "NA"
 
 
+class TestTextDigits:
+    @given(
+        data=data(),
+        min_size=integers(0, 100),
+        max_size=integers(0, 100) | none(),
+        disallow_na=booleans(),
+    )
+    def test_main(
+        self,
+        *,
+        data: DataObject,
+        min_size: int,
+        max_size: int | None,
+        disallow_na: bool,
+    ) -> None:
+        with assume_does_not_raise(InvalidArgument, AssertionError):
+            text = data.draw(
+                text_digits(
+                    min_size=min_size, max_size=max_size, disallow_na=disallow_na
+                )
+            )
+        assert search("^[0-9]*$", text)
+        assert len(text) >= min_size
+        if max_size is not None:
+            assert len(text) <= max_size
+        if disallow_na:
+            assert text != "NA"
+
+
 class TestTextPrintable:
     @given(
         data=data(),
@@ -1157,16 +809,41 @@ class TestTimeDeltas2W:
         assert min_value <= timedelta <= max_value
 
 
-class TestTimestamps:
+class TestUInt32s:
+    @given(data=data(), min_value=uint32s(), max_value=uint32s())
+    def test_main(self, *, data: DataObject, min_value: int, max_value: int) -> None:
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint32s(min_value=min_value, max_value=max_value))
+        assert max(min_value, MIN_UINT32) <= x <= min(max_value, MAX_UINT32)
+
+
+class TestUInt64s:
+    @given(data=data(), min_value=uint64s(), max_value=uint64s())
+    def test_main(self, *, data: DataObject, min_value: int, max_value: int) -> None:
+        with assume_does_not_raise(InvalidArgument):
+            x = data.draw(uint64s(min_value=min_value, max_value=max_value))
+        assert max(min_value, MIN_UINT64) <= x <= min(max_value, MAX_UINT64)
+
+
+@SKIPIF_CI_AND_NOT_LINUX
+class TestYieldTestRedis:
+    @given(data=data(), value=int32s())
+    @settings_with_reduced_examples()
+    async def test_core(self, *, data: DataObject, value: int) -> None:
+        async with yield_test_redis(data) as test:
+            assert not await test.redis.exists(test.key)
+            _ = await test.redis.set(test.key, value)
+            result = int(cast(str, await test.redis.get(test.key)))
+            assert result == value
+
+
+class TestZonedDatetimes:
     @given(
         data=data(),
-        min_value=datetimes_utc(
-            min_value=TIMESTAMP_MIN_AS_DATETIME, max_value=TIMESTAMP_MAX_AS_DATETIME
-        ),
-        max_value=datetimes_utc(
-            min_value=TIMESTAMP_MIN_AS_DATETIME, max_value=TIMESTAMP_MAX_AS_DATETIME
-        ),
-        allow_nanoseconds=booleans(),
+        min_value=datetimes(timezones=timezones() | just(dt.UTC) | none()),
+        max_value=timezones() | datetimes(timezones=just(dt.UTC) | none()),
+        time_zone1=timezones() | just(dt.UTC),
+        time_zone2=timezones() | just(dt.UTC),
     )
     @settings(suppress_health_check={HealthCheck.filter_too_much})
     def test_main(
@@ -1175,85 +852,58 @@ class TestTimestamps:
         data: DataObject,
         min_value: dt.datetime,
         max_value: dt.datetime,
-        allow_nanoseconds: bool,
+        time_zone1: ZoneInfo,
+        time_zone2: ZoneInfo,
     ) -> None:
+        _ = assume(
+            (is_local_datetime(min_value) and is_local_datetime(max_value))
+            or (is_zoned_datetime(min_value) and is_zoned_datetime(max_value))
+        )
         _ = assume(min_value <= max_value)
-        timestamp = data.draw(
-            timestamps(
-                min_value=min_value,
-                max_value=max_value,
-                allow_nanoseconds=allow_nanoseconds,
+        datetime = data.draw(
+            zoned_datetimes(
+                min_value=min_value, max_value=max_value, time_zone=time_zone1
             )
         )
-        assert min_value <= timestamp <= max_value
-        if not allow_nanoseconds:
-            assert cast(Any, timestamp).nanosecond == 0
+        assert datetime.tzinfo is time_zone1
+        if min_value.tzinfo is None:
+            min_value_use = min_value.replace(tzinfo=time_zone1)
+        else:
+            min_value_use = min_value.astimezone(time_zone1)
+        if max_value.tzinfo is None:
+            max_value_use = max_value.replace(tzinfo=time_zone1)
+        else:
+            max_value_use = max_value.astimezone(time_zone1)
+        assert min_value_use <= datetime <= max_value_use
+        _ = datetime.astimezone(time_zone2)
 
+    @given(
+        time_zone=timezones()
+        | sampled_from([_ZONED_DATETIMES_LEFT_MOST, _ZONED_DATETIMES_RIGHT_MOST])
+        | just(dt.UTC)
+    )
+    def test_min(self, *, time_zone: ZoneInfo) -> None:
+        datetime = dt.datetime.min.replace(tzinfo=_ZONED_DATETIMES_LEFT_MOST)
+        _ = datetime.astimezone(time_zone)
 
-class TestUInt32s:
-    @given(data=data(), min_value=uint32s() | none(), max_value=uint32s() | none())
-    def test_main(
-        self, *, data: DataObject, min_value: int | None, max_value: int | None
+    @given(
+        time_zone=timezones()
+        | sampled_from([_ZONED_DATETIMES_LEFT_MOST, _ZONED_DATETIMES_RIGHT_MOST])
+        | just(dt.UTC)
+    )
+    def test_max(self, *, time_zone: ZoneInfo) -> None:
+        datetime = dt.datetime.max.replace(tzinfo=_ZONED_DATETIMES_RIGHT_MOST)
+        _ = datetime.astimezone(time_zone)
+
+    @given(
+        data=data(),
+        min_value=zoned_datetimes(valid=True),
+        max_value=zoned_datetimes(valid=True),
+    )
+    @SKIPIF_CI_AND_WINDOWS
+    def test_valid(
+        self, *, data: DataObject, min_value: dt.datetime, max_value: dt.datetime
     ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            x = data.draw(uint32s(min_value=min_value, max_value=max_value))
-        info = iinfo(uint32)
-        assert info.min <= x <= info.max
-        if min_value is not None:
-            assert x >= min_value
-        if max_value is not None:
-            assert x <= max_value
-
-
-class TestUInt64s:
-    @given(data=data(), min_value=uint64s() | none(), max_value=uint64s() | none())
-    def test_main(
-        self, *, data: DataObject, min_value: int | None, max_value: int | None
-    ) -> None:
-        with assume_does_not_raise(InvalidArgument):
-            x = data.draw(uint64s(min_value=min_value, max_value=max_value))
-        info = iinfo(uint64)
-        assert info.min <= x <= info.max
-        if min_value is not None:
-            assert x >= min_value
-        if max_value is not None:
-            assert x <= max_value
-
-
-class TestVersions:
-    @given(data=data())
-    def test_main(self, data: DataObject) -> None:
-        version = data.draw(versions())
-        assert isinstance(version, Version)
-
-    @given(data=data())
-    def test_min_version(self, data: DataObject) -> None:
-        min_version = data.draw(versions())
-        version = data.draw(versions(min_version=min_version))
-        assert version >= min_version
-
-    @given(data=data())
-    def test_max_version(self, data: DataObject) -> None:
-        max_version = data.draw(versions())
-        version = data.draw(versions(max_version=max_version))
-        assert version <= max_version
-
-    @given(data=data())
-    def test_min_and_max_version(self, data: DataObject) -> None:
-        version1, version2 = data.draw(lists_fixed_length(versions(), 2))
-        min_version = min(version1, version2)
-        max_version = max(version1, version2)
-        version = data.draw(versions(min_version=min_version, max_version=max_version))
-        assert min_version <= version <= max_version
-
-    @given(data=data())
-    def test_error(self, data: DataObject) -> None:
-        version1, version2 = data.draw(lists_fixed_length(versions(), 2))
-        _ = assume(version1 != version2)
-        with raises(InvalidArgument):
-            _ = data.draw(
-                versions(
-                    min_version=max(version1, version2),
-                    max_version=min(version1, version2),
-                )
-            )
+        _ = assume(min_value <= max_value)
+        datetime = data.draw(zoned_datetimes(valid=True))
+        check_valid_zoned_datetime(datetime)
