@@ -1,12 +1,33 @@
 from __future__ import annotations
 
+from dataclasses import dataclass, replace
+from enum import Enum, auto
 from itertools import repeat
+from math import isfinite, isinf, isnan
 from typing import TYPE_CHECKING, Any, ClassVar, Literal
 
 from hypothesis import given
-from hypothesis.strategies import DataObject, data, integers, sampled_from, sets
+from hypothesis.strategies import (
+    DataObject,
+    binary,
+    data,
+    datetimes,
+    dictionaries,
+    floats,
+    frozensets,
+    integers,
+    lists,
+    none,
+    permutations,
+    sampled_from,
+    sets,
+    text,
+)
 from pytest import mark, param, raises
+from typing_extensions import override
 
+from tests.test_operator import make_objects
+from utilities.hypothesis import sets_fixed_length, text_ascii, zoned_datetimes
 from utilities.iterables import (
     CheckBijectionError,
     CheckDuplicatesError,
@@ -19,11 +40,16 @@ from utilities.iterables import (
     CheckSubSetError,
     CheckSuperMappingError,
     CheckSuperSetError,
+    Collection,
     EnsureIterableError,
     EnsureIterableNotStrError,
+    MaybeIterable,
     OneEmptyError,
     OneNonUniqueError,
     OneStrError,
+    ResolveIncludeAndExcludeError,
+    SortIterableError,
+    always_iterable,
     check_bijection,
     check_duplicates,
     check_iterables_equal,
@@ -36,37 +62,81 @@ from utilities.iterables import (
     check_supermapping,
     check_superset,
     chunked,
-    describe_mapping,
     ensure_hashables,
     ensure_iterable,
     ensure_iterable_not_str,
     expanding_window,
+    filter_include_and_exclude,
     groupby_lists,
+    hashable_to_iterable,
     is_iterable,
+    is_iterable_not_enum,
     is_iterable_not_str,
     one,
     one_str,
+    pairwise_tail,
     product_dicts,
+    resolve_include_and_exclude,
+    sort_iterable,
     take,
     transpose,
 )
+from utilities.sentinel import sentinel
 
 if TYPE_CHECKING:
-    from collections.abc import Iterable, Sequence
+    import datetime as dt
+    from collections.abc import Iterable, Iterator, Sequence
+
+
+class TestAlwaysIterable:
+    @given(x=binary())
+    def test_bytes(self, *, x: bytes) -> None:
+        assert list(always_iterable(x)) == [x]
+
+    @given(x=integers())
+    def test_integer(self, *, x: int) -> None:
+        assert list(always_iterable(x)) == [x]
+
+    @given(x=lists(binary()))
+    def test_list_of_bytes(self, *, x: list[bytes]) -> None:
+        assert list(always_iterable(x)) == x
+
+    @given(x=text())
+    def test_string(self, *, x: str) -> None:
+        assert list(always_iterable(x)) == [x]
+
+    @given(x=lists(text()))
+    def test_list_of_strings(self, *, x: list[str]) -> None:
+        assert list(always_iterable(x)) == x
+
+    @given(x=dictionaries(text(), integers()))
+    def test_dict(self, *, x: dict[str, int]) -> None:
+        assert list(always_iterable(x)) == list(x)
+
+    @given(x=lists(integers()))
+    def test_lists(self, *, x: list[int]) -> None:
+        assert list(always_iterable(x)) == x
+
+    def test_generator(self) -> None:
+        def yield_ints() -> Iterator[int]:
+            yield 0
+            yield 1
+
+        assert list(always_iterable(yield_ints())) == [0, 1]
 
 
 class TestCheckBijection:
     @given(data=data(), n=integers(0, 10))
     def test_main(self, *, data: DataObject, n: int) -> None:
-        keys = data.draw(sets(integers(0, 100), min_size=n, max_size=n))
-        values = data.draw(sets(integers(0, 100), min_size=n, max_size=n))
+        keys = data.draw(sets_fixed_length(integers(0, 100), n))
+        values = data.draw(sets_fixed_length(integers(0, 100), n))
         mapping = dict(zip(keys, values, strict=True))
         check_bijection(mapping)
 
     def test_error(self) -> None:
         with raises(
             CheckBijectionError,
-            match=r"Mapping .* must be a bijection; got duplicates \(.*, n=2\)\.",
+            match="Mapping .* must be a bijection; got duplicates {None: 2}",
         ):
             check_bijection({True: None, False: None})
 
@@ -79,7 +149,7 @@ class TestCheckDuplicates:
     def test_error(self) -> None:
         with raises(
             CheckDuplicatesError,
-            match=r"Iterable .* must not contain duplicates; got {None: 2}\.",
+            match="Iterable .* must not contain duplicates; got {None: 2}",
         ):
             check_duplicates([None, None])
 
@@ -91,35 +161,35 @@ class TestCheckIterablesEqual:
     def test_error_differing_items_and_left_longer(self) -> None:
         with raises(
             CheckIterablesEqualError,
-            match=r"Iterables .* and .* must be equal; differing items were \(.*, .*, i=.*\) and left was longer\.",
+            match="Iterables .* and .* must be equal; differing items were .* and left was longer",
         ):
             check_iterables_equal([1, 2, 3], [9])
 
     def test_error_differing_items_and_right_longer(self) -> None:
         with raises(
             CheckIterablesEqualError,
-            match=r"Iterables .* and .* must be equal; differing items were \(.*, .*, i=.*\) and right was longer\.",
+            match="Iterables .* and .* must be equal; differing items were .* and right was longer",
         ):
             check_iterables_equal([9], [1, 2, 3])
 
     def test_error_differing_items_and_same_length(self) -> None:
         with raises(
             CheckIterablesEqualError,
-            match=r"Iterables .* and .* must be equal; differing items were \(.*, .*, i=.*\)\.",
+            match="Iterables .* and .* must be equal; differing items were .*",
         ):
             check_iterables_equal([1, 2, 3], [1, 2, 9])
 
     def test_error_no_differing_items_just_left_longer(self) -> None:
         with raises(
             CheckIterablesEqualError,
-            match=r"Iterables .* and .* must be equal; left was longer\.",
+            match="Iterables .* and .* must be equal; left was longer",
         ):
             check_iterables_equal([1, 2, 3], [1])
 
     def test_error_no_differing_items_just_right_longer(self) -> None:
         with raises(
             CheckIterablesEqualError,
-            match=r"Iterables .* and .* must be equal; right was longer\.",
+            match="Iterables .* and .* must be equal; right was longer",
         ):
             check_iterables_equal([1], [1, 2, 3])
 
@@ -129,7 +199,7 @@ class TestCheckLength:
         check_length([], equal=0)
 
     def test_equal_fail(self) -> None:
-        with raises(CheckLengthError, match=r"Object .* must have length .*; got .*\."):
+        with raises(CheckLengthError, match=r"Object .* must have length .*; got .*"):
             check_length([], equal=1)
 
     @mark.parametrize("equal_or_approx", [param(10), param((11, 0.1))])
@@ -141,10 +211,10 @@ class TestCheckLength:
     @mark.parametrize(
         ("equal_or_approx", "match"),
         [
-            param(10, r"Object .* must have length .*; got .*\."),
+            param(10, "Object .* must have length .*; got .*"),
             param(
                 (11, 0.1),
-                r"Object .* must have approximate length .* \(error .*\); got .*\.",
+                r"Object .* must have approximate length .* \(error .*\); got .*",
             ),
         ],
     )
@@ -159,7 +229,7 @@ class TestCheckLength:
 
     def test_min_error(self) -> None:
         with raises(
-            CheckLengthError, match=r"Object .* must have minimum length .*; got .*\."
+            CheckLengthError, match="Object .* must have minimum length .*; got .*"
         ):
             check_length([], min=1)
 
@@ -168,7 +238,7 @@ class TestCheckLength:
 
     def test_max_error(self) -> None:
         with raises(
-            CheckLengthError, match=r"Object .* must have maximum length .*; got .*\."
+            CheckLengthError, match="Object .* must have maximum length .*; got .*"
         ):
             check_length([1], max=0)
 
@@ -180,7 +250,7 @@ class TestCheckLengthsEqual:
     def test_error(self) -> None:
         with raises(
             CheckLengthsEqualError,
-            match=r"Sized objects .* and .* must have the same length; got .* and .*\.",
+            match="Sized objects .* and .* must have the same length; got .* and .*",
         ):
             check_lengths_equal([], [1, 2, 3])
 
@@ -192,49 +262,49 @@ class TestCheckMappingsEqual:
     def test_error_extra_and_missing_and_differing_values(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; left had extra keys .*, right had extra keys .* and differing values were \(.*, .*, k=.*\)\.",
+            match="Mappings .* and .* must be equal; left had extra keys .*, right had extra keys .* and differing values were .*",
         ):
             check_mappings_equal({"a": 1, "b": 2, "c": 3}, {"b": 2, "c": 9, "d": 4})
 
     def test_error_extra_and_missing(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; left had extra keys .* and right had extra keys .*\.",
+            match="Mappings .* and .* must be equal; left had extra keys .* and right had extra keys .*",
         ):
             check_mappings_equal({"a": 1, "b": 2, "c": 3}, {"b": 2, "c": 3, "d": 4})
 
     def test_error_extra_and_differing_values(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; left had extra keys .* and differing values were \(.*, .*, k=.*\)\.",
+            match="Mappings .* and .* must be equal; left had extra keys .* and differing values were .*",
         ):
             check_mappings_equal({"a": 1, "b": 2, "c": 3}, {"a": 9})
 
     def test_error_missing_and_differing_values(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; right had extra keys .* and differing values were \(.*, .*, k=.*\)\.",
+            match="Mappings .* and .* must be equal; right had extra keys .* and differing values were .*",
         ):
             check_mappings_equal({"a": 1}, {"a": 9, "b": 2, "c": 3})
 
     def test_error_extra_only(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; left had extra keys .*\.",
+            match="Mappings .* and .* must be equal; left had extra keys .*",
         ):
             check_mappings_equal({"a": 1, "b": 2, "c": 3}, {"a": 1})
 
     def test_error_missing_only(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; right had extra keys .*\.",
+            match="Mappings .* and .* must be equal; right had extra keys .*",
         ):
             check_mappings_equal({"a": 1}, {"a": 1, "b": 2, "c": 3})
 
     def test_error_differing_values_only(self) -> None:
         with raises(
             CheckMappingsEqualError,
-            match=r"Mappings .* and .* must be equal; differing values were \(.*, .*, k=.*\)\.",
+            match="Mappings .* and .* must be equal; differing values were .*",
         ):
             check_mappings_equal({"a": 1, "b": 2, "c": 3}, {"a": 1, "b": 2, "c": 9})
 
@@ -249,21 +319,21 @@ class TestCheckSetsEqual:
     def test_error_extra_and_missing(self) -> None:
         with raises(
             CheckSetsEqualError,
-            match=r"Sets .* and .* must be equal; left had extra items .* and right had extra items .*\.",
+            match="Sets .* and .* must be equal; left had extra items .* and right had extra items .*",
         ):
             check_sets_equal({1, 2, 3}, {2, 3, 4})
 
     def test_error_extra(self) -> None:
         with raises(
             CheckSetsEqualError,
-            match=r"Sets .* and .* must be equal; left had extra items .*\.",
+            match="Sets .* and .* must be equal; left had extra items .*",
         ):
             check_sets_equal({1, 2, 3}, set())
 
     def test_error_missing(self) -> None:
         with raises(
             CheckSetsEqualError,
-            match=r"Sets .* and .* must be equal; right had extra items .*\.",
+            match="Sets .* and .* must be equal; right had extra items .*",
         ):
             check_sets_equal(set(), {1, 2, 3})
 
@@ -275,21 +345,21 @@ class TestCheckSubMapping:
     def test_error_extra_and_differing_values(self) -> None:
         with raises(
             CheckSubMappingError,
-            match=r"Mapping .* must be a submapping of .*; left had extra keys .* and differing values were \(.*, .*., k=.*\)\.",
+            match="Mapping .* must be a submapping of .*; left had extra keys .* and differing values were .*",
         ):
             check_submapping({"a": 1, "b": 2, "c": 3}, {"a": 9})
 
     def test_error_extra_only(self) -> None:
         with raises(
             CheckSubMappingError,
-            match=r"Mapping .* must be a submapping of .*; left had extra keys .*\.",
+            match="Mapping .* must be a submapping of .*; left had extra keys .*",
         ):
             check_submapping({"a": 1, "b": 2, "c": 3}, {"a": 1})
 
     def test_error_differing_values_only(self) -> None:
         with raises(
             CheckSubMappingError,
-            match=r"Mapping .* must be a submapping of .*; differing values were \(.*, .*, k=.*\)\.",
+            match="Mapping .* must be a submapping of .*; differing values were .*",
         ):
             check_submapping({"a": 1, "b": 2, "c": 3}, {"a": 1, "b": 2, "c": 9})
 
@@ -304,7 +374,7 @@ class TestCheckSubSet:
     def test_error(self) -> None:
         with raises(
             CheckSubSetError,
-            match=r"Set .* must be a subset of .*; left had extra items .*\.",
+            match="Set .* must be a subset of .*; left had extra items .*",
         ):
             check_subset({1, 2, 3}, {1})
 
@@ -316,21 +386,21 @@ class TestCheckSuperMapping:
     def test_error_missing_and_differing_values(self) -> None:
         with raises(
             CheckSuperMappingError,
-            match=r"Mapping .* must be a supermapping of .*; right had extra keys .* and differing values were \(.*, .*, k=.*\)\.",
+            match="Mapping .* must be a supermapping of .*; right had extra keys .* and differing values were .*",
         ):
             check_supermapping({"a": 1}, {"a": 9, "b": 2, "c": 3})
 
     def test_error_extra_only(self) -> None:
         with raises(
             CheckSuperMappingError,
-            match=r"Mapping .* must be a supermapping of .*; right had extra keys .*\.",
+            match="Mapping .* must be a supermapping of .*; right had extra keys .*",
         ):
             check_supermapping({"a": 1}, {"a": 1, "b": 2, "c": 3})
 
     def test_error_differing_values_only(self) -> None:
         with raises(
             CheckSuperMappingError,
-            match=r"Mapping .* must be a supermapping of .*; differing values were \(.*, .*, k=.*\)\.",
+            match="Mapping .* must be a supermapping of .*; differing values were .*",
         ):
             check_supermapping({"a": 1, "b": 2, "c": 3}, {"a": 1, "b": 2, "c": 9})
 
@@ -370,41 +440,165 @@ class TestChunked:
         assert result == expected
 
 
-class TestDescribeMapping:
-    @mark.parametrize(
-        ("include_underscore", "include_none", "expected"),
-        [
-            param(False, False, "a=1, c=3"),
-            param(False, True, "a=1, b=None, c=3"),
-            param(True, False, "a=1, c=3, _underscore=4"),
-            param(True, True, "a=1, b=None, c=3, _underscore=4"),
-        ],
-    )
-    def test_main(
-        self, *, include_underscore: bool, include_none: bool, expected: str
-    ) -> None:
-        mapping = {"a": 1, "b": None, "c": 3, "_underscore": 4}
-        result = describe_mapping(
-            mapping, include_underscore=include_underscore, include_none=include_none
-        )
+@dataclass(unsafe_hash=True, slots=True)
+class _Item:
+    n: int
+
+
+class TestCollection:
+    def test_and_singleton(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection & _Item(1)
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(1))
         assert result == expected
 
-    @mark.parametrize(
-        ("b", "include_none", "expected"),
-        [
-            param(2, False, "a=1, b=2, total=3"),
-            param(2, True, "a=1, b=2, total=3"),
-            param(None, False, "a=1, total=1"),
-            param(None, True, "a=1, b=None, total=1"),
-        ],
-    )
-    def test_func(self, *, b: int | None, include_none: bool, expected: str) -> None:
-        def func(a: int, /, *, b: int | None = None) -> str:
-            init = describe_mapping(locals(), func=func, include_none=include_none)
-            total = a if b is None else (a + b)
-            return f"{init}, total={total}"
+    def test_and_collection(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection & Collection(_Item(1))
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(1))
+        assert result == expected
 
-        result = func(1, b=b)
+    def test_and_iterable(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection & [_Item(1)]
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(1))
+        assert result == expected
+
+    def test_filter(self) -> None:
+        collection = Collection(map(_Item, range(4)))
+        result = collection.filter(lambda item: item.n % 2 == 0)
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(0), _Item(2))
+        assert result == expected
+
+    def test_hash(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        _ = hash(collection)
+
+    def test_init(self) -> None:
+        class SubCollection(Collection[_Item]):
+            @override
+            def __init__(self, *item_or_items: MaybeIterable[_Item]) -> None:
+                super().__init__(*item_or_items)
+                if any(item.n >= 1 for item in self):
+                    msg = "n >= 1 is not permitted"
+                    raise ValueError(msg)
+
+        with raises(ValueError, match="n >= 1 is not permitted"):
+            _ = SubCollection(map(_Item, range(3)))
+
+    def test_map_return_same_type(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection.map(lambda item: replace(item, n=item.n + 1))
+        assert isinstance(result, Collection)
+        expected = Collection(map(_Item, range(1, 4)))
+        assert result == expected
+
+    def test_map_return_different_type(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection.map(lambda item: item.n)
+        assert isinstance(result, Collection)
+        expected = Collection(range(3))
+        assert result == expected
+
+    def test_new_one_singleton(self) -> None:
+        collection = Collection(_Item(1))
+        assert isinstance(collection, Collection)
+        assert len(collection) == 1
+        assert one(collection) == _Item(1)
+
+    def test_new_one_iterable(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        assert isinstance(collection, Collection)
+        assert len(collection) == 3
+
+    def test_new_many_singletons(self) -> None:
+        collection = Collection(_Item(1), _Item(2), _Item(3))
+        assert isinstance(collection, Collection)
+        assert len(collection) == 3
+
+    def test_new_many_iterables(self) -> None:
+        collection = Collection(map(_Item, range(3)), map(_Item, range(3)))
+        assert isinstance(collection, Collection)
+        assert len(collection) == 3
+
+    def test_new_check_items(self) -> None:
+        class SubCollection(Collection[_Item]):
+            @classmethod
+            @override
+            def check_items(cls, items: Iterable[_Item]) -> None:
+                if any(item.n >= 1 for item in items):
+                    msg = "n >= 1 is not permitted"
+                    raise ValueError(msg)
+
+        with raises(ValueError, match="n >= 1 is not permitted"):
+            _ = SubCollection(map(_Item, range(3)))
+
+    def test_or_singleton(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection | _Item(3)
+        assert isinstance(result, Collection)
+        expected = Collection(map(_Item, range(4)))
+        assert result == expected
+
+    def test_or_collection(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection | Collection(map(_Item, range(1, 4)))
+        assert isinstance(result, Collection)
+        expected = Collection(map(_Item, range(4)))
+        assert result == expected
+
+    def test_or_iterable(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection | map(_Item, range(1, 4))
+        assert isinstance(result, Collection)
+        expected = Collection(map(_Item, range(4)))
+        assert result == expected
+
+    def test_partition(self) -> None:
+        collection = Collection(map(_Item, range(4)))
+        result_false, result_true = collection.partition(lambda item: item.n % 2 == 0)
+        assert isinstance(result_false, Collection)
+        expected_false = Collection(_Item(1), _Item(3))
+        assert result_false == expected_false
+        assert isinstance(result_true, Collection)
+        expected_true = Collection(_Item(0), _Item(2))
+        assert result_true == expected_true
+
+    def test_repr(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = repr(collection)
+        expected = "Collection({_Item(n=0), _Item(n=1), _Item(n=2)})"
+        assert result == expected
+
+    def test_str(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = str(collection)
+        expected = "Collection({_Item(n=0), _Item(n=1), _Item(n=2)})"
+        assert result == expected
+
+    def test_sub_single_item(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection - _Item(1)
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(0), _Item(2))
+        assert result == expected
+
+    def test_sub_collection(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection - Collection(_Item(1))
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(0), _Item(2))
+        assert result == expected
+
+    def test_sub_iterable(self) -> None:
+        collection = Collection(map(_Item, range(3)))
+        result = collection - [_Item(1)]
+        assert isinstance(result, Collection)
+        expected = Collection(_Item(0), _Item(2))
         assert result == expected
 
 
@@ -419,7 +613,7 @@ class TestEnsureIterable:
         _ = ensure_iterable(obj)
 
     def test_error(self) -> None:
-        with raises(EnsureIterableError, match=r"Object .* must be iterable\."):
+        with raises(EnsureIterableError, match="Object .* must be iterable"):
             _ = ensure_iterable(None)
 
 
@@ -432,7 +626,7 @@ class TestEnsureIterableNotStr:
     def test_error(self, *, obj: Any) -> None:
         with raises(
             EnsureIterableNotStrError,
-            match=r"Object .* must be iterable, but not a string\.",
+            match="Object .* must be iterable, but not a string",
         ):
             _ = ensure_iterable_not_str(obj)
 
@@ -452,6 +646,79 @@ class TestExpandingWindow:
         assert result == expected
 
 
+class TestHashableToIterable:
+    def test_none(self) -> None:
+        result = hashable_to_iterable(None)
+        expected = None
+        assert result is expected
+
+    @given(x=lists(integers()))
+    def test_integers(self, *, x: int) -> None:
+        result = hashable_to_iterable(x)
+        expected = (x,)
+        assert result == expected
+
+
+class TestFilterIncludeAndExclude:
+    def test_none(self) -> None:
+        rng = list(range(5))
+        result = list(filter_include_and_exclude(rng))
+        assert result == rng
+
+    def test_include_singleton(self) -> None:
+        result = list(filter_include_and_exclude(range(5), include=0))
+        expected = [0]
+        assert result == expected
+
+    def test_include_iterable(self) -> None:
+        result = list(filter_include_and_exclude(range(5), include=[0, 1, 2]))
+        expected = [0, 1, 2]
+        assert result == expected
+
+    def test_exclude_singleton(self) -> None:
+        result = list(filter_include_and_exclude(range(5), exclude=0))
+        expected = [1, 2, 3, 4]
+        assert result == expected
+
+    def test_exclude_iterable(self) -> None:
+        result = list(filter_include_and_exclude(range(5), exclude=[0, 1, 2]))
+        expected = [3, 4]
+        assert result == expected
+
+    def test_both(self) -> None:
+        result = list(
+            filter_include_and_exclude(range(5), include=[0, 1], exclude=[3, 4])
+        )
+        expected = [0, 1]
+        assert result == expected
+
+    def test_include_key(self) -> None:
+        @dataclass(kw_only=True, slots=True)
+        class Example:
+            n: int
+
+        result = list(
+            filter_include_and_exclude(
+                [Example(n=n) for n in range(5)], include=[0, 1, 2], key=lambda x: x.n
+            )
+        )
+        expected = [Example(n=n) for n in [0, 1, 2]]
+        assert result == expected
+
+    def test_exclude_key(self) -> None:
+        @dataclass(kw_only=True, slots=True)
+        class Example:
+            n: int
+
+        result = list(
+            filter_include_and_exclude(
+                [Example(n=n) for n in range(5)], exclude=[0, 1, 2], key=lambda x: x.n
+            )
+        )
+        expected = [Example(n=n) for n in [3, 4]]
+        assert result == expected
+
+
 class TestGroupbyLists:
     iterable: ClassVar[str] = "AAAABBBCCDAABB"
 
@@ -489,6 +756,33 @@ class TestIsIterable:
         assert is_iterable(obj) is expected
 
 
+class TestIsIterableNotEnum:
+    def test_single(self) -> None:
+        class Truth(Enum):
+            true = auto()
+            false = auto()
+
+        assert not is_iterable_not_enum(Truth)
+
+    def test_union(self) -> None:
+        class Truth1(Enum):
+            true = auto()
+            false = auto()
+
+        class Truth2(Enum):
+            true = auto()
+            false = auto()
+
+        assert is_iterable_not_enum((Truth1, Truth2))
+
+    @mark.parametrize(
+        ("obj", "expected"),
+        [param(None, False), param([], True), param((), True), param("", True)],
+    )
+    def test_others(self, *, obj: Any, expected: bool) -> None:
+        assert is_iterable_not_enum(obj) is expected
+
+
 class TestIsIterableNotStr:
     @mark.parametrize(
         ("obj", "expected"),
@@ -503,13 +797,13 @@ class TestOne:
         assert one([None]) is None
 
     def test_error_empty(self) -> None:
-        with raises(OneEmptyError, match=r"Iterable .* must not be empty\."):
+        with raises(OneEmptyError, match="Iterable .* must not be empty"):
             _ = one([])
 
     def test_error_non_unique(self) -> None:
         with raises(
             OneNonUniqueError,
-            match=r"Iterable .* must contain exactly one item; got .*, .* and perhaps more\.",
+            match="Iterable .* must contain exactly one item; got .*, .* and perhaps more",
         ):
             _ = one([1, 2])
 
@@ -543,7 +837,7 @@ class TestOneStr:
     def test_error_bijection_error(self) -> None:
         with raises(
             OneStrError,
-            match=r"Iterable .* must not contain duplicates \(case insensitive\); got {'a': 2}",
+            match=r"Iterable .* must not contain duplicates \(case insensitive\); got .*",
         ):
             _ = one_str(["a", "A"], "a", case_sensitive=False)
 
@@ -554,6 +848,14 @@ class TestOneStr:
             _ = one_str(["a", "b", "c"], "d", case_sensitive=False)
 
 
+class TestPairwiseTail:
+    def test_main(self) -> None:
+        iterable = range(5)
+        result = list(pairwise_tail(iterable))
+        expected = [(0, 1), (1, 2), (2, 3), (3, 4), (4, sentinel)]
+        assert result == expected
+
+
 class TestProductDicts:
     def test_main(self) -> None:
         mapping = {"x": [1, 2], "y": [7, 8, 9]}
@@ -569,6 +871,96 @@ class TestProductDicts:
         assert result == expected
 
 
+class TestResolveIncludeAndExclude:
+    def test_none(self) -> None:
+        include, exclude = resolve_include_and_exclude()
+        assert include is None
+        assert exclude is None
+
+    def test_include_singleton(self) -> None:
+        include, exclude = resolve_include_and_exclude(include=1)
+        assert include == {1}
+        assert exclude is None
+
+    def test_include_iterable(self) -> None:
+        include, exclude = resolve_include_and_exclude(include=[1, 2, 3])
+        assert include == {1, 2, 3}
+        assert exclude is None
+
+    def test_exclude_singleton(self) -> None:
+        include, exclude = resolve_include_and_exclude(exclude=1)
+        assert include is None
+        assert exclude == {1}
+
+    def test_exclude_iterable(self) -> None:
+        include, exclude = resolve_include_and_exclude(exclude=[1, 2, 3])
+        assert include is None
+        assert exclude == {1, 2, 3}
+
+    def test_both(self) -> None:
+        include, exclude = resolve_include_and_exclude(
+            include=[1, 2, 3], exclude=[4, 5, 6]
+        )
+        assert include == {1, 2, 3}
+        assert exclude == {4, 5, 6}
+
+    def test_error(self) -> None:
+        with raises(
+            ResolveIncludeAndExcludeError,
+            match="Iterables .* and .* must not overlap; got .*",
+        ):
+            _ = resolve_include_and_exclude(include=[1, 2, 3], exclude=[3, 4, 5])
+
+
+class TestSortIterables:
+    @given(
+        x=make_objects(floats_allow_nan=False), y=make_objects(floats_allow_nan=False)
+    )
+    def test_main(self, *, x: Any, y: Any) -> None:
+        result1 = sort_iterable([x, y])
+        result2 = sort_iterable([y, x])
+        assert result1 == result2
+
+    @given(x=datetimes() | zoned_datetimes(), y=datetimes() | zoned_datetimes())
+    def test_datetimes(self, *, x: dt.datetime, y: dt.datetime) -> None:
+        result1 = sort_iterable([x, y])
+        result2 = sort_iterable([y, x])
+        assert result1 == result2
+
+    @given(x=floats(), y=floats())
+    def test_floats(self, *, x: float, y: float) -> None:
+        result1 = sort_iterable([x, y])
+        result2 = sort_iterable([y, x])
+        for i, j in zip(result1, result2, strict=True):
+            assert isfinite(i) is isfinite(j)
+            assert isinf(i) is isinf(j)
+            assert isnan(i) is isnan(j)
+
+    @given(x=text_ascii(), y=text_ascii())
+    def test_strings(self, *, x: str, y: str) -> None:
+        result1 = sort_iterable([x, y])
+        result2 = sort_iterable([y, x])
+        assert result1 == result2
+
+    @given(x=frozensets(frozensets(integers())), y=frozensets(frozensets(integers())))
+    def test_nested_frozensets(
+        self, *, x: frozenset[frozenset[int]], y: frozenset[frozenset[int]]
+    ) -> None:
+        result1 = sort_iterable([x, y])
+        result2 = sort_iterable([y, x])
+        assert result1 == result2
+
+    @given(data=data(), x=lists(none()))
+    def test_nones(self, *, data: DataObject, x: list[None]) -> None:
+        result1 = sort_iterable(x)
+        result2 = sort_iterable(data.draw(permutations(result1)))
+        assert result1 == result2
+
+    def test_error(self) -> None:
+        with raises(SortIterableError, match="Unable to sort .* and .*"):
+            _ = sort_iterable([sentinel, sentinel])
+
+
 class TestTake:
     def test_simple(self) -> None:
         result = take(5, range(10))
