@@ -7,19 +7,29 @@ from typing import TYPE_CHECKING, Any
 
 from pytest import mark, param
 
-from tests.modules import package_with, package_without, standalone
+from tests.modules import package_with, package_without, standalone, with_imports
+from utilities.functions import get_class_name
 from utilities.modules import (
+    is_installed,
     yield_module_contents,
     yield_module_subclasses,
     yield_modules,
 )
-from utilities.types import get_class_name
 
 if TYPE_CHECKING:
     from collections.abc import Callable
     from types import ModuleType
 
 
+class TestIsInstalled:
+    @mark.parametrize(
+        ("module", "expected"), [param("importlib", True), param("invalid", False)]
+    )
+    def test_main(self, *, module: str, expected: int) -> None:
+        result = is_installed(module)
+        assert result is expected
+
+
 class TestYieldModules:
     @mark.parametrize(
         ("module", "recursive", "expected"),
@@ -118,3 +128,8 @@ class TestYieldModuleSubclasses:
             module, type_, recursive=recursive, predicate=predicate
         )
         assert len(list(it)) == (factor * expected)
+
+    @mark.parametrize(("is_module", "expected"), [param(True, 1), param(False, 2)])
+    def test_is_module(self, *, is_module: bool, expected: int) -> None:
+        it = yield_module_subclasses(with_imports, object, is_module=is_module)
+        assert len(list(it)) == expected
