@@ -1,73 +1,40 @@
 from __future__ import annotations
 
-import datetime as dt
 from re import escape
 from typing import TYPE_CHECKING, Any, Literal
 
 from hypothesis import assume, given
-from hypothesis.strategies import (
-    DataObject,
-    data,
-    dates,
-    datetimes,
-    floats,
-    integers,
-    just,
-    none,
-)
+from hypothesis.strategies import DataObject, data, floats, integers, none
 from numpy import (
     arange,
     array,
-    bool_,
-    concatenate,
-    datetime64,
     eye,
-    float64,
     full,
     inf,
-    int64,
     isclose,
     median,
     nan,
     ndarray,
-    object_,
     ones,
     zeros,
     zeros_like,
 )
 from numpy.random import Generator
 from numpy.testing import assert_allclose, assert_equal
-from pandas import DatetimeTZDtype, Series
 from pytest import mark, param, raises
 
-from utilities.hypothesis import assume_does_not_raise, datetimes_utc, float_arrays
+from utilities.hypothesis import assume_does_not_raise, float_arrays
 from utilities.numpy import (
     DEFAULT_RNG,
     AsIntError,
-    DateTime64ToDateError,
-    DateTime64ToDateTimeError,
-    DatetimeToDatetime64Error,
-    EmptyNumpyConcatenateError,
     FlatN0EmptyError,
     FlatN0MultipleError,
-    GetFillValueError,
     NDArrayF,
-    NDArrayF1,
-    NDArrayF2,
-    NDArrayI2,
+    NDArrayI,
     PctChangeError,
     ShiftError,
     array_indexer,
     as_int,
-    date_to_datetime64,
-    datetime64_to_date,
-    datetime64_to_datetime,
-    datetime64_to_int,
-    datetime64D,
-    datetime64ns,
-    datetime64us,
-    datetime64Y,
-    datetime_to_datetime64,
     discretize,
     ewma,
     exp_moving_sum,
@@ -75,7 +42,6 @@ from utilities.numpy import (
     ffill_non_nan_slices,
     fillna,
     flatn0,
-    get_fill_value,
     has_dtype,
     is_at_least,
     is_at_least_or_nan,
@@ -126,16 +92,12 @@ from utilities.numpy import (
     maximum,
     minimum,
     pct_change,
-    redirect_empty_numpy_concatenate,
     shift,
     shift_bool,
-    year,
 )
-from utilities.zoneinfo import HONG_KONG, UTC
 
 if TYPE_CHECKING:
     from collections.abc import Sequence
-    from zoneinfo import ZoneInfo
 
 
 class TestArrayIndexer:
@@ -210,96 +172,6 @@ class TestAsInt:
             _ = as_int(arr)
 
 
-class TestDateToDatetime64ns:
-    def test_example(self) -> None:
-        result = date_to_datetime64(dt.date(2000, 1, 1))
-        assert result == datetime64("2000-01-01", "D")
-        assert result.dtype == datetime64D
-
-    @given(date=dates())
-    def test_main(self, *, date: dt.date) -> None:
-        result = date_to_datetime64(date)
-        assert result.dtype == datetime64D
-
-
-class TestDatetimeToDatetime64:
-    @mark.parametrize("time_zone", [param(UTC), param(None)])
-    def test_example(self, *, time_zone: ZoneInfo | None) -> None:
-        result = datetime_to_datetime64(
-            dt.datetime(2000, 1, 1, 0, 0, 0, 123456, tzinfo=time_zone)
-        )
-        assert result == datetime64("2000-01-01 00:00:00.123456", "us")
-        assert result.dtype == datetime64us
-
-    @given(datetime=datetimes() | datetimes_utc())
-    def test_main(self, *, datetime: dt.datetime) -> None:
-        result = datetime_to_datetime64(datetime)
-        assert result.dtype == datetime64us
-
-    @given(datetime=datetimes(timezones=just(HONG_KONG)))
-    def test_error(self, *, datetime: dt.datetime) -> None:
-        with raises(
-            DatetimeToDatetime64Error, match=r"Timezone must be None or UTC; got .*\."
-        ):
-            _ = datetime_to_datetime64(datetime)
-
-
-class TestDatetime64ToDate:
-    def test_example(self) -> None:
-        assert datetime64_to_date(datetime64("2000-01-01", "D")) == dt.date(2000, 1, 1)
-
-    @given(date=dates())
-    def test_round_trip(self, *, date: dt.date) -> None:
-        assert datetime64_to_date(date_to_datetime64(date)) == date
-
-    @mark.parametrize(
-        ("datetime", "dtype", "error"),
-        [
-            param("10000-01-01", "D", DateTime64ToDateError),
-            param("2000-01-01", "ns", NotImplementedError),
-        ],
-    )
-    def test_error(self, *, datetime: str, dtype: str, error: type[Exception]) -> None:
-        with raises(error):
-            _ = datetime64_to_date(datetime64(datetime, dtype))
-
-
-class TestDatetime64ToInt:
-    def test_example(self) -> None:
-        expected = 10957
-        assert datetime64_to_int(datetime64("2000-01-01", "D")) == expected
-
-
-class TestDatetime64ToDatetime:
-    def test_example_ms(self) -> None:
-        assert datetime64_to_datetime(
-            datetime64("2000-01-01 00:00:00.123", "ms")
-        ) == dt.datetime(2000, 1, 1, 0, 0, 0, 123000, tzinfo=UTC)
-
-    @mark.parametrize("dtype", [param("us"), param("ns")])
-    def test_examples_us_ns(self, *, dtype: str) -> None:
-        assert datetime64_to_datetime(
-            datetime64("2000-01-01 00:00:00.123456", dtype)
-        ) == dt.datetime(2000, 1, 1, 0, 0, 0, 123456, tzinfo=UTC)
-
-    @given(datetime=datetimes_utc())
-    def test_round_trip(self, *, datetime: dt.datetime) -> None:
-        assert datetime64_to_datetime(datetime_to_datetime64(datetime)) == datetime
-
-    @mark.parametrize(
-        ("datetime", "dtype", "error"),
-        [
-            param("0000-12-31", "ms", DateTime64ToDateTimeError),
-            param("10000-01-01", "ms", DateTime64ToDateTimeError),
-            param("1970-01-01 00:00:00.000000001", "ns", DateTime64ToDateTimeError),
-            param("2000-01-01", "D", NotImplementedError),
-        ],
-    )
-    def test_error(self, *, datetime: str, dtype: str, error: type[Exception]) -> None:
-        with raises(error):
-            _ = datetime64_to_datetime(datetime64(datetime, dtype))
-
-
 class TestDefaultRng:
     def test_main(self) -> None:
         assert isinstance(DEFAULT_RNG, Generator)
@@ -307,7 +179,7 @@ class TestDefaultRng:
 
 class TestDiscretize:
     @given(arr=float_arrays(shape=integers(0, 10), min_value=-1.0, max_value=1.0))
-    def test_1_bin(self, *, arr: NDArrayF1) -> None:
+    def test_1_bin(self, *, arr: NDArrayF) -> None:
         result = discretize(arr, 1)
         expected = zeros_like(arr, dtype=float)
         assert_equal(result, expected)
@@ -317,7 +189,7 @@ class TestDiscretize:
             shape=integers(1, 10), min_value=-1.0, max_value=1.0, unique=True
         )
     )
-    def test_2_bins(self, *, arr: NDArrayF1) -> None:
+    def test_2_bins(self, *, arr: NDArrayF) -> None:
         _ = assume(len(arr) % 2 == 0)
         result = discretize(arr, 2)
         med = median(arr)
@@ -489,33 +361,6 @@ class TestFlatN0:
             _ = flatn0(ones(2, dtype=bool))
 
 
-class TestGetFillValue:
-    @mark.parametrize(
-        "dtype",
-        [
-            param(bool),
-            param(bool_),
-            param(datetime64D),
-            param(datetime64Y),
-            param(datetime64ns),
-            param(float),
-            param(float64),
-            param(int),
-            param(int64),
-            param(object),
-            param(object_),
-        ],
-    )
-    def test_main(self, *, dtype: Any) -> None:
-        fill_value = get_fill_value(dtype)
-        array = full(0, fill_value, dtype=dtype)
-        assert has_dtype(array, dtype)
-
-    def test_error(self) -> None:
-        with raises(GetFillValueError, match="Invalid data type; got 'invalid'"):
-            _ = get_fill_value("invalid")
-
-
 class TestHasDtype:
     @mark.parametrize(("dtype", "expected"), [param(float, True), param(int, False)])
     @mark.parametrize("is_tuple", [param(True), param(False)])
@@ -524,22 +369,6 @@ class TestHasDtype:
         result = has_dtype(array([], dtype=float), against)
         assert result is expected
 
-    @mark.parametrize(
-        ("dtype", "against", "expected"),
-        [
-            param("Int64", "Int64", True),
-            param("Int64", ("Int64",), True),
-            param("Int64", int, False),
-            param(DatetimeTZDtype(tz="UTC"), DatetimeTZDtype(tz="UTC"), True),
-            param(
-                DatetimeTZDtype(tz="UTC"), DatetimeTZDtype(tz="Asia/Hong_Kong"), False
-            ),
-        ],
-    )
-    def test_pandas(self, *, dtype: Any, against: Any, expected: bool) -> None:
-        result = has_dtype(Series([], dtype=dtype), against)
-        assert result is expected
-
 
 class TestIsAtLeast:
     @mark.parametrize(
@@ -1008,7 +837,7 @@ class TestIsNonSingular:
         ("array", "expected"), [param(eye(2), True), param(ones((2, 2)), False)]
     )
     @mark.parametrize("dtype", [param(float), param(int)])
-    def test_main(self, *, array: NDArrayF2, dtype: Any, expected: bool) -> None:
+    def test_main(self, *, array: NDArrayF, dtype: Any, expected: bool) -> None:
         assert is_non_singular(array.astype(dtype)) is expected
 
     def test_overflow(self) -> None:
@@ -1077,12 +906,12 @@ class TestIsPositiveSemiDefinite:
     )
     @mark.parametrize("dtype", [param(float), param(int)])
     def test_main(
-        self, *, array: NDArrayF2 | NDArrayI2, dtype: Any, expected: bool
+        self, *, array: NDArrayF | NDArrayI, dtype: Any, expected: bool
     ) -> None:
         assert is_positive_semidefinite(array.astype(dtype)) is expected
 
     @given(array=float_arrays(shape=(2, 2), min_value=-1.0, max_value=1.0))
-    def test_overflow(self, *, array: NDArrayF2) -> None:
+    def test_overflow(self, *, array: NDArrayF) -> None:
         _ = is_positive_semidefinite(array)
 
 
@@ -1097,7 +926,7 @@ class TestIsSymmetric:
     )
     @mark.parametrize("dtype", [param(float), param(int)])
     def test_main(
-        self, *, array: NDArrayF2 | NDArrayI2, dtype: Any, expected: bool
+        self, *, array: NDArrayF | NDArrayI, dtype: Any, expected: bool
     ) -> None:
         assert is_symmetric(array.astype(dtype)) is expected
 
@@ -1301,12 +1130,6 @@ class TestPctChange:
             _ = pct_change(arr, n=0)
 
 
-class TestRedirectEmptyNumpyConcatenate:
-    def test_main(self) -> None:
-        with raises(EmptyNumpyConcatenateError), redirect_empty_numpy_concatenate():
-            _ = concatenate([])
-
-
 class TestShift:
     @mark.parametrize(
         ("n", "expected_v"),
@@ -1403,17 +1226,3 @@ class TestShiftBool:
             [fill_value if e is None else e for e in expected_v], dtype=bool
         )
         assert_equal(result, expected)
-
-
-class TestYear:
-    @given(date=dates())
-    def test_scalar(self, *, date: dt.date) -> None:
-        date64 = datetime64(date, "D")
-        yr = year(date64)
-        assert yr == date.year
-
-    @given(date=dates())
-    def test_array(self, *, date: dt.date) -> None:
-        dates = array([date], dtype=datetime64D)
-        years = year(dates)
-        assert years.item() == date.year
