@@ -1,14 +1,14 @@
 from __future__ import annotations
 
 from functools import partial
-from operator import neg, pow
+from operator import neg
 from typing import TYPE_CHECKING, Any, Literal
 
 from pytest import mark, param
 
+from utilities.functions import get_class_name
 from utilities.pqdm import _get_desc, pmap, pstarmap
 from utilities.sentinel import Sentinel, sentinel
-from utilities.types import get_class_name
 
 if TYPE_CHECKING:
     from collections.abc import Callable, Mapping
