@@ -1,20 +1,66 @@
 from __future__ import annotations
 
 from inspect import signature
+from pathlib import Path
 from time import sleep
 from typing import TYPE_CHECKING
 
-from pytest import mark, param
+from pytest import mark, param, raises
 
 from tests.conftest import FLAKY
-from utilities.pytest import throttle
+from utilities.pytest import NodeIdToPathError, is_pytest, node_id_to_path, throttle
 
 if TYPE_CHECKING:
-    from pathlib import Path
+    from collections.abc import Sequence
 
     from _pytest.legacypath import Testdir
 
-    from utilities.types import IterableStrs
+
+class TestIsPytest:
+    def test_main(self) -> None:
+        assert is_pytest()
+
+
+class TestNodeIdToPath:
+    @mark.parametrize(
+        ("node_id", "expected"),
+        [
+            param(
+                "src/tests/module/test_funcs.py::TestClass::test_main",
+                Path("src.tests.module.test_funcs", "TestClass__test_main.csv"),
+            ),
+            param(
+                "src/tests/module/test_funcs.py::TestClass::test_main.csv",
+                Path("src.tests.module.test_funcs", "TestClass__test_main.csv"),
+            ),
+            param(
+                "src/tests/module/test_funcs.py::TestClass::test_main[param1, param2]",
+                Path(
+                    "src.tests.module.test_funcs",
+                    "TestClass__test_main[param1, param2].csv",
+                ),
+            ),
+            param(
+                "src/tests/module/test_funcs.py::TestClass::test_main[EUR.USD]",
+                Path(
+                    "src.tests.module.test_funcs", "TestClass__test_main[EUR.USD].csv"
+                ),
+            ),
+        ],
+    )
+    def test_main(self, *, node_id: str, expected: Path) -> None:
+        result = node_id_to_path(node_id)
+        assert result == expected
+
+    def test_head(self) -> None:
+        node_id = "src/tests/module/test_funcs.py::TestClass::test_main"
+        result = node_id_to_path(node_id, head=Path("src/tests"))
+        expected = Path("module.test_funcs", "TestClass__test_main.csv")
+        assert result == expected
+
+    def test_error_file_suffix(self) -> None:
+        with raises(NodeIdToPathError, match="Node ID must be a Python file; got .*"):
+            _ = node_id_to_path("src/tests/module/test_funcs.csv::TestClass::test_main")
 
 
 class TestPytestOptions:
@@ -60,10 +106,10 @@ class TestPytestOptions:
         self,
         *,
         testdir: Testdir,
-        case: IterableStrs,
+        case: Sequence[str],
         passed: int,
         skipped: int,
-        matches: IterableStrs,
+        matches: Sequence[str],
     ) -> None:
         _ = testdir.makeconftest(
             """
@@ -126,10 +172,10 @@ class TestPytestOptions:
         self,
         *,
         testdir: Testdir,
-        case: IterableStrs,
+        case: Sequence[str],
         passed: int,
         skipped: int,
-        matches: IterableStrs,
+        matches: Sequence[str],
     ) -> None:
         _ = testdir.makeconftest(
             """
