@@ -1,10 +1,13 @@
 from __future__ import annotations
 
-from collections.abc import Callable
+from typing import TYPE_CHECKING
 
 from pytest import mark, param
 
-from utilities.sentinel import _REPR, Sentinel, sentinel
+from utilities.sentinel import SENTINEL_REPR, Sentinel, sentinel
+
+if TYPE_CHECKING:
+    from collections.abc import Callable
 
 
 class TestSentinel:
@@ -13,7 +16,7 @@ class TestSentinel:
 
     @mark.parametrize("method", [param(repr), param(str)])
     def test_repr_and_str(self, method: Callable[..., str]) -> None:
-        assert method(sentinel) == _REPR
+        assert method(sentinel) == SENTINEL_REPR
 
     def test_singletone(self) -> None:
         assert Sentinel() is sentinel
