@@ -1,11 +1,12 @@
 from __future__ import annotations
 
+import datetime as dt
 from operator import eq
-from typing import TYPE_CHECKING, Any, ClassVar, TypeVar
+from typing import TYPE_CHECKING, Any
 
 import polars as pl
 import sqlalchemy
-from hypothesis import given
+from hypothesis import Phase, given, settings
 from hypothesis.strategies import (
     DataObject,
     SearchStrategy,
@@ -30,7 +31,11 @@ from polars import (
     Float64,
     Int32,
     Int64,
+    UInt32,
+    UInt64,
     Utf8,
+    col,
+    when,
 )
 from polars.testing import assert_frame_equal
 from pytest import mark, param, raises
@@ -61,10 +66,8 @@ from sqlalchemy import (
     VARCHAR,
     BigInteger,
     Column,
-    Connection,
     DateTime,
     Double,
-    Engine,
     Float,
     Integer,
     Interval,
@@ -81,19 +84,18 @@ from sqlalchemy import (
     Uuid,
     select,
 )
-from sqlalchemy.exc import DuplicateColumnError
+from sqlalchemy.exc import DuplicateColumnError, OperationalError, ProgrammingError
 
+from tests.conftest import FLAKY
+from tests.test_sqlalchemy import _table_names, _upsert_lists
 from utilities.datetime import is_equal_mod_tz
-from utilities.hypothesis import aiosqlite_engines, sqlite_engines, text_ascii
+from utilities.hypothesis import sqlalchemy_engines, text_ascii
 from utilities.math import is_equal
 from utilities.polars import DatetimeUTC, check_polars_dataframe
-from utilities.sqlalchemy import (
-    ensure_tables_created,
-    ensure_tables_created_async,
-    yield_connection,
-)
+from utilities.sqlalchemy import ensure_tables_created
 from utilities.sqlalchemy_polars import (
     InsertDataFrameError,
+    _insert_dataframe_check_df_and_db_types,
     _insert_dataframe_map_df_column_to_table_column_and_type,
     _insert_dataframe_map_df_column_to_table_schema,
     _insert_dataframe_map_df_schema_to_table,
@@ -105,212 +107,187 @@ from utilities.sqlalchemy_polars import (
     _select_to_dataframe_map_table_column_type_to_dtype,
     _select_to_dataframe_yield_selects_with_in_clauses,
     insert_dataframe,
-    insert_dataframe_async,
     select_to_dataframe,
-    select_to_dataframe_async,
 )
 from utilities.zoneinfo import UTC
 
 if TYPE_CHECKING:
     from collections.abc import Callable, Iterable, Sequence
 
-    from _pytest.mark import ParameterSet
     from polars._typing import PolarsDataType
     from polars.datatypes import DataTypeClass
+    from sqlalchemy.ext.asyncio import AsyncEngine
 
 
-_T = TypeVar("_T")
+_CASES_SELECT: list[
+    tuple[SearchStrategy[Any], PolarsDataType, Any, Callable[[Any, Any], bool]]
+] = [
+    (booleans() | none(), pl.Boolean, sqlalchemy.Boolean, eq),
+    (dates() | none(), pl.Date, sqlalchemy.Date, eq),
+    (datetimes() | none(), Datetime, DateTime, eq),
+    (
+        datetimes(timezones=just(UTC)) | none(),
+        DatetimeUTC,
+        DateTime(timezone=True),
+        is_equal_mod_tz,
+    ),
+    (floats(allow_nan=False) | none(), Float64, Float, is_equal),
+    (integers(-10, 10) | none(), Int64, Integer, eq),
+    (text_ascii() | none(), Utf8, String, eq),
+]
+_CASES_INSERT: list[
+    tuple[SearchStrategy[Any], PolarsDataType, Any, Callable[[Any, Any], bool]]
+] = [*_CASES_SELECT, (integers(-10, 10) | none(), Int32, Integer, eq)]
 
 
 class TestInsertDataFrame:
-    cases: ClassVar[list[ParameterSet]] = [
-        param(booleans() | none(), pl.Boolean, sqlalchemy.Boolean, eq),
-        param(dates() | none(), pl.Date, sqlalchemy.Date, eq),
-        param(datetimes() | none(), Datetime, DateTime, eq),
-        param(
-            datetimes(timezones=just(UTC)) | none(),
-            DatetimeUTC,
-            DateTime(timezone=True),
-            is_equal_mod_tz,
-        ),
-        param(floats(allow_nan=False) | none(), Float64, Float, is_equal),
-        param(integers(-10, 10) | none(), Int32, Integer, eq),
-        param(integers(-10, 10) | none(), Int64, Integer, eq),
-        param(text_ascii() | none(), Utf8, String, eq),
-    ]
-
-    @given(data=data(), engine=sqlite_engines())
-    @mark.parametrize(("strategy", "pl_dtype", "col_type", "check"), cases)
-    @mark.parametrize("use_conn", [param(True), param(False)])
-    def test_sync(
+    @FLAKY
+    @given(data=data(), name=_table_names(), case=sampled_from(_CASES_INSERT))
+    @settings(phases={Phase.generate})
+    async def test_main(
         self,
         *,
         data: DataObject,
-        engine: Engine,
-        strategy: SearchStrategy[Any],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
-        check: Callable[[Any, Any], bool],
-        use_conn: bool,
+        name: str,
+        case: tuple[
+            SearchStrategy[Any], PolarsDataType, Any, Callable[[Any, Any], bool]
+        ],
     ) -> None:
-        values, df, table, sel = self._prepare_main_test(
-            data, strategy, pl_dtype, col_type
-        )
-        self._run_test_sync(df, table, engine, sel, values, check, use_conn=use_conn)
-
-    @given(engine=sqlite_engines(), values=lists(booleans() | none(), max_size=100))
-    @mark.parametrize("sr_name", [param("Value"), param("value")])
-    @mark.parametrize("use_conn", [param(True), param(False)])
-    def test_sync_snake(
-        self, *, engine: Engine, values: list[bool | None], sr_name: str, use_conn: bool
-    ) -> None:
-        df, table, sel = self._prepare_snake_test(sr_name, values)
-        self._run_test_sync(
-            df, table, engine, sel, values, eq, snake=True, use_conn=use_conn
-        )
-
-    @given(
-        values=lists(booleans() | none(), min_size=1, max_size=100),
-        engine=sqlite_engines(),
-    )
-    @mark.parametrize("use_conn", [param(True), param(False)])
-    def test_sync_error(
-        self, *, values: list[bool | None], engine: Engine, use_conn: bool
-    ) -> None:
-        df, table = self._prepare_empty_test(values)
-        self._run_test_sync_error(df, table, engine, use_conn=use_conn)
-
-    @given(data=data())
-    @mark.parametrize(("strategy", "pl_dtype", "col_type", "check"), cases)
-    async def test_async(
-        self,
-        *,
-        data: DataObject,
-        strategy: SearchStrategy[Any],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
-        check: Callable[[Any, Any], bool],
-    ) -> None:
-        values, df, table, sel = self._prepare_main_test(
-            data, strategy, pl_dtype, col_type
-        )
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine)
-        async with engine.begin() as conn:
-            res = (await conn.execute(sel)).scalars().all()
-        self._assert_results(res, values, check)
-
-    @given(data=data(), values=lists(booleans() | none(), max_size=100))
-    @mark.parametrize("sr_name", [param("Value"), param("value")])
-    async def test_async_snake(
-        self, *, data: DataObject, values: list[bool | None], sr_name: str
-    ) -> None:
-        df, table, sel = self._prepare_snake_test(sr_name, values)
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine, snake=True)
-        async with engine.begin() as conn:
-            res = (await conn.execute(sel)).scalars().all()
-        assert res == values
-
-    @given(data=data(), values=lists(booleans() | none(), min_size=1, max_size=100))
-    async def test_async_error(
-        self, *, data: DataObject, values: list[bool | None]
-    ) -> None:
-        df, table = self._prepare_empty_test(values)
-        engine = await aiosqlite_engines(data)
-        with raises(
-            InsertDataFrameError,
-            match="Non-empty DataFrame must resolve to at least 1 item",
-        ):
-            _ = await insert_dataframe_async(df, table, engine)
-
-    def _prepare_main_test(
-        self,
-        data: DataObject,
-        strategy: SearchStrategy[_T],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
-        /,
-    ) -> tuple[Sequence[_T], DataFrame, Table, Select[Any]]:
+        strategy, pl_dtype, col_type, check = case
         values = data.draw(lists(strategy, max_size=100))
         df = DataFrame({"value": values}, schema={"value": pl_dtype})
-        table = self._make_table(col_type)
+        table = self._make_table(name, col_type)
+        engine = await sqlalchemy_engines(data, table)
+        await insert_dataframe(df, table, engine)
         sel = select(table.c["value"])
-        return values, df, table, sel
-
-    def _make_table(self, type_: Any, /, *, title: bool = False) -> Table:
-        return Table(
-            "example",
-            MetaData(),
-            Column("Id" if title else "id", Integer, primary_key=True),
-            Column("Value" if title else "value", type_),
-        )
-
-    def _run_test_sync(
-        self,
-        df: DataFrame,
-        table: Table,
-        engine_or_conn: Engine | Connection,
-        sel: Select[Any],
-        values: Sequence[Any],
-        check: Callable[[Any, Any], bool],
-        /,
-        *,
-        snake: bool = False,
-        use_conn: bool = False,
-    ) -> None:
-        if use_conn:
-            with yield_connection(engine_or_conn) as conn:
-                self._run_test_sync(df, table, conn, sel, values, check, snake=snake)
-            return
-        insert_dataframe(df, table, engine_or_conn, snake=snake)
-        with yield_connection(engine_or_conn) as conn:
-            res = conn.execute(sel).scalars().all()
-        self._assert_results(res, values, check)
-
-    def _assert_results(
-        self,
-        results: Sequence[Any],
-        values: Sequence[Any],
-        check: Callable[[Any, Any], bool],
-        /,
-    ) -> None:
+        async with engine.begin() as conn:
+            results = (await conn.execute(sel)).scalars().all()
         for r, v in zip(results, values, strict=True):
             assert ((r is None) == (v is None)) or check(r, v)
 
-    def _prepare_snake_test(
-        self, sr_name: str, values: Sequence[bool | None], /
-    ) -> tuple[DataFrame, Table, Select[Any]]:
-        df = DataFrame({sr_name: values}, schema={sr_name: pl.Boolean})
-        table = self._make_table(sqlalchemy.Boolean, title=True)
-        sel = select(table.c["Value"])
-        return df, table, sel
-
-    def _prepare_empty_test(
-        self, values: Sequence[bool | None], /
-    ) -> tuple[DataFrame, Table]:
-        df = DataFrame({"other": values}, schema={"other": pl.Boolean})
-        table = self._make_table(sqlalchemy.Boolean)
-        return df, table
-
-    def _run_test_sync_error(
-        self,
-        df: DataFrame,
-        table: Table,
-        engine_or_conn: Engine | Connection,
-        /,
-        *,
-        use_conn: bool = False,
+    @FLAKY
+    @given(data=data(), name=_table_names())
+    @settings(phases={Phase.generate})
+    async def test_assume_exists_with_empty(
+        self, *, data: DataObject, name: str
     ) -> None:
-        if use_conn:
-            with yield_connection(engine_or_conn) as conn:
-                self._run_test_sync_error(df, table, conn)
-            return
+        df = DataFrame(schema={"value": pl.Boolean})
+        table = self._make_table(name, sqlalchemy.Boolean)
+        engine = await sqlalchemy_engines(data, table)
+        await ensure_tables_created(engine, table)
+        await insert_dataframe(df, table, engine, assume_tables_exist=True)
+
+    @FLAKY
+    @given(data=data(), name=_table_names(), value=booleans())
+    @settings(phases={Phase.generate})
+    async def test_assume_exists_with_non_empty(
+        self, *, data: DataObject, name: str, value: bool
+    ) -> None:
+        df = DataFrame([(value,)], schema={"value": pl.Boolean})
+        table = self._make_table(name, sqlalchemy.Boolean)
+        engine = await sqlalchemy_engines(data, table)
+        with raises(
+            (OperationalError, ProgrammingError), match="(no such table|does not exist)"
+        ):
+            await insert_dataframe(df, table, engine, assume_tables_exist=True)
+
+    @FLAKY
+    @given(data=data(), name=_table_names())
+    @settings(phases={Phase.generate})
+    async def test_empty(self, *, data: DataObject, name: str) -> None:
+        df = DataFrame(schema={"value": pl.Boolean})
+        table = self._make_table(name, sqlalchemy.Boolean)
+        engine = await sqlalchemy_engines(data, table)
+        await insert_dataframe(df, table, engine, assume_tables_exist=False)
+        sel = select(table.c["value"])
+        async with engine.begin() as conn:
+            results = (await conn.execute(sel)).scalars().all()
+        assert results == []
+
+    @FLAKY
+    @given(data=data(), name=_table_names())
+    @settings(phases={Phase.generate})
+    async def test_upsert(self, *, data: DataObject, name: str) -> None:
+        values = data.draw(_upsert_lists())
+        df = DataFrame(
+            values,
+            schema={"id_": Int64, "init": pl.Boolean, "post": pl.Boolean},
+            orient="row",
+        )
+        table = self._make_table(name, sqlalchemy.Boolean)
+        engine = await sqlalchemy_engines(data, table)
+        await insert_dataframe(
+            df.select("id_", col("init").alias("value")),
+            table,
+            engine,
+            upsert="selected",
+        )
+        sel = select(table)
+        async with engine.begin() as conn:
+            results = (await conn.execute(sel)).all()
+        assert set(results) == set(df.select("id_", "init").rows())
+        await insert_dataframe(
+            df.select("id_", col("post").alias("value")).drop_nulls(),
+            table,
+            engine,
+            upsert="selected",
+        )
+        async with engine.begin() as conn:
+            results = (await conn.execute(sel)).all()
+        expected = df.select(
+            "id_", when(col("post").is_null()).then("init").otherwise("post")
+        )
+        assert set(results) == set(expected.rows())
+
+    @FLAKY
+    @given(data=data(), name=_table_names(), value=booleans())
+    @settings(phases={Phase.generate})
+    async def test_error(self, *, data: DataObject, name: str, value: bool) -> None:
+        df = DataFrame([(value,)], schema={"other": pl.Boolean})
+        table = self._make_table(name, sqlalchemy.Boolean)
+        engine = await sqlalchemy_engines(data, table)
         with raises(
             InsertDataFrameError,
             match="Non-empty DataFrame must resolve to at least 1 item",
         ):
-            insert_dataframe(df, table, engine_or_conn)
+            _ = await insert_dataframe(df, table, engine)
+
+    def _make_table(self, name: str, type_: Any, /) -> Table:
+        return Table(
+            name,
+            MetaData(),
+            Column("id_", Integer, primary_key=True),
+            Column("value", type_),
+        )
+
+
+class TestInsertDataFrameCheckDFAndDBTypes:
+    @mark.parametrize(
+        ("dtype", "db_col_type", "expected"),
+        [
+            param(pl.Boolean, bool, True),
+            param(pl.Date, dt.date, True),
+            param(pl.Date, dt.datetime, False),
+            param(Datetime, dt.date, False),
+            param(Datetime, dt.datetime, True),
+            param(Float64, float, True),
+            param(Float64, int, False),
+            param(Int32, int, True),
+            param(Int32, float, False),
+            param(Int64, int, True),
+            param(Int64, float, False),
+            param(UInt32, int, True),
+            param(UInt32, float, False),
+            param(UInt64, int, True),
+            param(UInt64, float, False),
+            param(Utf8, str, True),
+        ],
+    )
+    def test_main(
+        self, *, dtype: PolarsDataType, db_col_type: type, expected: bool
+    ) -> None:
+        result = _insert_dataframe_check_df_and_db_types(dtype, db_col_type)
+        assert result is expected
 
 
 class TestInsertDataFrameMapDFColumnToTableColumnAndType:
@@ -332,14 +309,20 @@ class TestInsertDataFrameMapDFColumnToTableColumnAndType:
     @mark.parametrize("snake", [param(True), param(False)])
     def test_error_empty(self, *, snake: bool) -> None:
         schema = {"a": int, "b": float, "c": str}
-        with raises(_InsertDataFrameMapDFColumnToTableColumnAndTypeError):
+        with raises(
+            _InsertDataFrameMapDFColumnToTableColumnAndTypeError,
+            match=r"Unable to map DataFrame column 'value' into table schema \{.*\} with snake=[True|False]",
+        ):
             _ = _insert_dataframe_map_df_column_to_table_column_and_type(
                 "value", schema, snake=snake
             )
 
     def test_error_non_unique(self) -> None:
         schema = {"a": int, "b": float, "B": float, "c": str}
-        with raises(_InsertDataFrameMapDFColumnToTableColumnAndTypeError):
+        with raises(
+            _InsertDataFrameMapDFColumnToTableColumnAndTypeError,
+            match=r"Unable to map DataFrame column 'b' into table schema \{.*\} with snake=True",
+        ):
             _ = _insert_dataframe_map_df_column_to_table_column_and_type(
                 "b", schema, snake=True
             )
@@ -417,80 +400,89 @@ class TestInsertDataFrameMapDFSchemaToTable:
 
 
 class TestSelectToDataFrame:
-    cases: ClassVar[list[ParameterSet]] = [
-        param(booleans() | none(), pl.Boolean, sqlalchemy.Boolean),
-        param(dates() | none(), pl.Date, sqlalchemy.Date),
-        param(datetimes() | none(), Datetime, DateTime),
-        param(
-            datetimes(timezones=just(UTC)) | none(),
-            DatetimeUTC,
-            DateTime(timezone=True),
-        ),
-        param(floats(allow_nan=False) | none(), Float64, Float),
-        param(integers(-10, 10) | none(), Int64, Integer),
-        param(text_ascii() | none(), Utf8, String),
-    ]
-
-    @given(data=data(), engine=sqlite_engines())
-    @mark.parametrize(("strategy", "pl_dtype", "col_type"), cases)
-    def test_sync(
+    @FLAKY
+    @given(
+        data=data(),
+        name=_table_names(),
+        case=sampled_from([
+            (strategy, pl_dtype, col_type)
+            for (strategy, pl_dtype, col_type, _) in _CASES_SELECT
+        ]),
+    )
+    @settings(phases={Phase.generate})
+    async def test_main(
         self,
         *,
         data: DataObject,
-        engine: Engine,
-        strategy: SearchStrategy[Any],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
+        name: str,
+        case: tuple[SearchStrategy[Any], PolarsDataType, Any],
     ) -> None:
-        df, table, sel = self._prepare_main_test(data, strategy, pl_dtype, col_type)
-        insert_dataframe(df, table, engine)
-        result = select_to_dataframe(sel, engine)
+        strategy, pl_dtype, col_type = case
+        values = data.draw(lists(strategy, max_size=100))
+        df = DataFrame({"value": values}, schema={"value": pl_dtype})
+        table = self._make_table(name, col_type)
+        engine = await sqlalchemy_engines(data, table)
+        await insert_dataframe(df, table, engine)
+        sel = select(table.c["value"])
+        result = await select_to_dataframe(sel, engine)
         assert_frame_equal(result, df)
 
+    @FLAKY
     @given(
-        engine=sqlite_engines(),
+        data=data(),
+        name=_table_names(),
         values=lists(booleans() | none(), min_size=1, max_size=100),
+        sr_name=sampled_from(["Value", "value"]),
     )
-    @mark.parametrize("sr_name", [param("Value"), param("value")])
-    def test_sync_snake(
-        self, *, engine: Engine, values: list[bool | None], sr_name: str
+    @settings(phases={Phase.generate})
+    async def test_snake(
+        self, *, data: DataObject, name: str, values: list[bool], sr_name: str
     ) -> None:
-        df, table, sel, expected = self._prepare_snake_test(values, sr_name)
-        insert_dataframe(df, table, engine, snake=True)
-        result = select_to_dataframe(sel, engine, snake=True)
+        df = DataFrame({sr_name: values}, schema={sr_name: pl.Boolean})
+        table = self._make_table(name, sqlalchemy.Boolean, title=True)
+        engine = await sqlalchemy_engines(data, table)
+        await insert_dataframe(df, table, engine, snake=True)
+        sel = select(table.c["Value"])
+        result = await select_to_dataframe(sel, engine, snake=True)
+        expected = DataFrame({"value": values}, schema={"value": pl.Boolean})
         assert_frame_equal(result, expected)
 
-    @given(
-        engine=sqlite_engines(),
-        values=lists(integers(0, 100), max_size=100),
-        batch_size=integers(1, 10),
-    )
-    def test_sync_batch(
-        self, *, engine: Engine, values: list[int], batch_size: int
-    ) -> None:
-        df, table, sel = self._prepare_feature_test(values)
-        insert_dataframe(df, table, engine)
-        dfs = select_to_dataframe(sel, engine, batch_size=batch_size)
-        self._assert_batch_results(dfs, batch_size, values)
-
+    @FLAKY
     @given(
         data=data(),
-        engine=sqlite_engines(),
+        name=_table_names(),
+        values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
+        batch_size=integers(1, 10),
+    )
+    @settings(phases={Phase.generate})
+    async def test_batch(
+        self, *, data: DataObject, name: str, values: list[int], batch_size: int
+    ) -> None:
+        df, table, engine, sel = await self._prepare_feature_test(data, name, values)
+        await insert_dataframe(df, table, engine)
+        dfs = await select_to_dataframe(sel, engine, batch_size=batch_size)
+        self._assert_batch_results(dfs, batch_size, values)
+
+    @FLAKY
+    @given(
+        data=data(),
+        name=_table_names(),
         values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
         in_clauses_chunk_size=integers(1, 10),
     )
-    def test_sync_in_clauses_non_empty(
+    @settings(phases={Phase.generate})
+    async def test_in_clauses_non_empty(
         self,
         *,
         data: DataObject,
-        engine: Engine,
+        name: str,
         values: list[int],
         in_clauses_chunk_size: int,
     ) -> None:
-        df, table, sel = self._prepare_feature_test(values)
-        insert_dataframe(df, table, engine)
+        df, table, engine, sel = await self._prepare_feature_test(data, name, values)
+        await insert_dataframe(df, table, engine)
         in_values = data.draw(sets(sampled_from(values)))
-        df = select_to_dataframe(
+        df = await select_to_dataframe(
             sel,
             engine,
             in_clauses=(table.c["value"], in_values),
@@ -499,135 +491,39 @@ class TestSelectToDataFrame:
         check_polars_dataframe(df, height=len(in_values), schema_list={"value": Int64})
         assert set(df["value"].to_list()) == in_values
 
-    @given(engine=sqlite_engines())
-    def test_sync_in_clauses_empty(self, *, engine: Engine) -> None:
-        table, sel = self._prepare_empty_test()
-        ensure_tables_created(engine, table)
-        df = select_to_dataframe(sel, engine, in_clauses=(table.c["value"], []))
+    @FLAKY
+    @given(data=data(), name=_table_names())
+    @settings(phases={Phase.generate})
+    async def test_in_clauses_empty(self, *, data: DataObject, name: str) -> None:
+        table = self._make_table(name, Integer)
+        engine = await sqlalchemy_engines(data, table)
+        await ensure_tables_created(engine, table)
+        sel = select(table.c["value"])
+        df = await select_to_dataframe(sel, engine, in_clauses=(table.c["value"], []))
         check_polars_dataframe(df, height=0, schema_list={"value": Int64})
 
+    @FLAKY
     @given(
         data=data(),
-        engine=sqlite_engines(),
-        batch_size=integers(1, 10),
+        name=_table_names(),
         values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
+        batch_size=integers(1, 10),
         in_clauses_chunk_size=integers(1, 10),
     )
-    def test_sync_batch_and_in_clauses(
+    @settings(phases={Phase.generate})
+    async def test_batch_and_in_clauses(
         self,
         *,
         data: DataObject,
-        batch_size: int,
-        engine: Engine,
+        name: str,
         values: list[int],
+        batch_size: int,
         in_clauses_chunk_size: int,
     ) -> None:
-        df, table, sel, in_values = self._prepare_batch_and_in_clauses_test(
-            data, values
-        )
-        insert_dataframe(df, table, engine)
-        dfs = select_to_dataframe(
-            sel,
-            engine,
-            batch_size=batch_size,
-            in_clauses=(table.c["value"], in_values),
-            in_clauses_chunk_size=in_clauses_chunk_size,
-        )
-        max_height = batch_size * in_clauses_chunk_size
-        self._assert_batch_results(dfs, max_height, in_values)
-
-    @given(data=data())
-    @mark.parametrize(("strategy", "pl_dtype", "col_type"), cases)
-    async def test_async(
-        self,
-        *,
-        data: DataObject,
-        strategy: SearchStrategy[Any],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
-    ) -> None:
-        df, table, sel = self._prepare_main_test(data, strategy, pl_dtype, col_type)
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine)
-        result = await select_to_dataframe_async(sel, engine)
-        assert_frame_equal(result, df)
-
-    @given(data=data(), values=lists(booleans() | none(), min_size=1, max_size=100))
-    @mark.parametrize("sr_name", [param("Value"), param("value")])
-    async def test_async_snake(
-        self, *, data: DataObject, values: list[bool], sr_name: str
-    ) -> None:
-        df, table, sel, expected = self._prepare_snake_test(values, sr_name)
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine, snake=True)
-        result = await select_to_dataframe_async(sel, engine, snake=True)
-        assert_frame_equal(result, expected)
-
-    @given(
-        data=data(),
-        values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
-        batch_size=integers(1, 10),
-    )
-    async def test_async_batch(
-        self, *, data: DataObject, values: list[int], batch_size: int
-    ) -> None:
-        df, table, sel = self._prepare_feature_test(values)
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine)
-        dfs = await select_to_dataframe_async(sel, engine, batch_size=batch_size)
-        self._assert_batch_results(dfs, batch_size, values)
-
-    @given(
-        data=data(),
-        values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
-        in_clauses_chunk_size=integers(1, 10),
-    )
-    async def test_async_in_clauses_non_empty(
-        self, *, data: DataObject, values: list[int], in_clauses_chunk_size: int
-    ) -> None:
-        df, table, sel = self._prepare_feature_test(values)
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine)
+        df, table, engine, sel = await self._prepare_feature_test(data, name, values)
+        await insert_dataframe(df, table, engine)
         in_values = data.draw(sets(sampled_from(values)))
-        df = await select_to_dataframe_async(
-            sel,
-            engine,
-            in_clauses=(table.c["value"], in_values),
-            in_clauses_chunk_size=in_clauses_chunk_size,
-        )
-        check_polars_dataframe(df, height=len(in_values), schema_list={"value": Int64})
-        assert set(df["value"].to_list()) == in_values
-
-    @given(data=data())
-    async def test_async_in_clauses_empty(self, *, data: DataObject) -> None:
-        table, sel = self._prepare_empty_test()
-        engine = await aiosqlite_engines(data)
-        await ensure_tables_created_async(engine, table)
-        df = await select_to_dataframe_async(
-            sel, engine, in_clauses=(table.c["value"], [])
-        )
-        check_polars_dataframe(df, height=0, schema_list={"value": Int64})
-
-    @given(
-        data=data(),
-        values=lists(integers(0, 100), min_size=1, max_size=100, unique=True),
-        batch_size=integers(1, 10),
-        in_clauses_chunk_size=integers(1, 10),
-    )
-    async def test_async_batch_and_in_clauses(
-        self,
-        *,
-        data: DataObject,
-        values: list[int],
-        batch_size: int,
-        in_clauses_chunk_size: int,
-    ) -> None:
-        df, table, sel, in_values = self._prepare_batch_and_in_clauses_test(
-            data, values
-        )
-        engine = await aiosqlite_engines(data)
-        await insert_dataframe_async(df, table, engine)
-        async_dfs = await select_to_dataframe_async(
+        async_dfs = await select_to_dataframe(
             sel,
             engine,
             batch_size=batch_size,
@@ -638,45 +534,6 @@ class TestSelectToDataFrame:
         max_height = batch_size * in_clauses_chunk_size
         self._assert_batch_results(sync_dfs, max_height, in_values)
 
-    def _prepare_main_test(
-        self,
-        data: DataObject,
-        strategy: SearchStrategy[Any],
-        pl_dtype: PolarsDataType,
-        col_type: Any,
-        /,
-    ) -> tuple[DataFrame, Table, Select[Any]]:
-        values = data.draw(lists(strategy, max_size=100))
-        df = DataFrame({"value": values}, schema={"value": pl_dtype})
-        table = self._make_table(col_type)
-        sel = select(table.c["value"])
-        return df, table, sel
-
-    def _make_table(self, type_: Any, /, *, title: bool = False) -> Table:
-        return Table(
-            "example",
-            MetaData(),
-            Column("Id" if title else "id", Integer, primary_key=True),
-            Column("Value" if title else "value", type_),
-        )
-
-    def _prepare_snake_test(
-        self, values: Sequence[bool | None], sr_name: str, /
-    ) -> tuple[DataFrame, Table, Select[Any], DataFrame]:
-        df = DataFrame({sr_name: values}, schema={sr_name: pl.Boolean})
-        table = self._make_table(sqlalchemy.Boolean, title=True)
-        sel = select(table.c["Value"])
-        expected = DataFrame({"value": values}, schema={"value": pl.Boolean})
-        return df, table, sel, expected
-
-    def _prepare_feature_test(
-        self, values: Sequence[int], /
-    ) -> tuple[DataFrame, Table, Select[Any]]:
-        df = DataFrame({"value": values}, schema={"value": Int64})
-        table = self._make_table(Integer)
-        sel = select(table.c["value"])
-        return df, table, sel
-
     def _assert_batch_results(
         self, dfs: Iterable[DataFrame], max_height: int, values: Iterable[int], /
     ) -> None:
@@ -690,19 +547,22 @@ class TestSelectToDataFrame:
             seen.update(df_i["value"].to_list())
         assert seen == values
 
-    def _prepare_empty_test(self, /) -> tuple[Table, Select[Any]]:
-        table = self._make_table(Integer)
-        sel = select(table.c["value"])
-        return table, sel
+    def _make_table(self, name: str, type_: Any, /, *, title: bool = False) -> Table:
+        return Table(
+            name,
+            MetaData(),
+            Column("Id" if title else "id", Integer, primary_key=True),
+            Column("Value" if title else "value", type_),
+        )
 
-    def _prepare_batch_and_in_clauses_test(
-        self, data: DataObject, values: Sequence[int], /
-    ) -> tuple[DataFrame, Table, Select[Any], set[int]]:
-        df, table, sel = self._prepare_feature_test(values)
-        table = self._make_table(Integer)
+    async def _prepare_feature_test(
+        self, data: DataObject, name: str, values: Sequence[int], /
+    ) -> tuple[DataFrame, Table, AsyncEngine, Select[Any]]:
+        df = DataFrame({"value": values}, schema={"value": Int64})
+        table = self._make_table(name, Integer)
+        engine = await sqlalchemy_engines(data, table)
         sel = select(table.c["value"])
-        in_values = data.draw(sets(sampled_from(values)))
-        return df, table, sel, in_values
+        return df, table, engine, sel
 
 
 class TestSelectToDataFrameApplySnake:
@@ -716,15 +576,18 @@ class TestSelectToDataFrameApplySnake:
         sel = select(table)
         res = _select_to_dataframe_apply_snake(sel)
         expected = ["id", "value"]
-        for col, exp in zip(res.selected_columns, expected, strict=True):
-            assert col.name == exp
+        for column, exp in zip(res.selected_columns, expected, strict=True):
+            assert column.name == exp
 
 
 class TestSelectToDataFrameCheckDuplicates:
     def test_error(self) -> None:
         table = Table("example", MetaData(), Column("id", Integer, primary_key=True))
         sel = select(table.c["id"], table.c["id"])
-        with raises(DuplicateColumnError):
+        with raises(
+            DuplicateColumnError,
+            match=r"Columns must not contain duplicates; got \{'id': 2\}",
+        ):
             _select_to_dataframe_check_duplicates(sel.selected_columns)
 
 
@@ -800,29 +663,30 @@ class TestSelectToDataFrameMapTableColumnTypeToDType:
 
 class TestSelectToDataFrameYieldSelectsWithInClauses:
     @given(
-        engine=sqlite_engines(),
+        data=data(),
+        name=_table_names(),
         values=sets(integers(), max_size=100),
         in_clauses_chunk_size=integers(1, 10) | none(),
         chunk_size_frac=floats(0.1, 10.0),
     )
-    def test_main(
+    async def test_main(
         self,
         *,
-        engine: Engine,
+        data: DataObject,
+        name: str,
         values: set[int],
         in_clauses_chunk_size: int | None,
         chunk_size_frac: float,
     ) -> None:
-        table = Table("example", MetaData(), Column("id", Integer, primary_key=True))
+        table = Table(name, MetaData(), Column("id", Integer, primary_key=True))
+        engine = await sqlalchemy_engines(data, table)
         sel = select(table.c["id"])
-        with engine.begin() as conn:
-            iterator = _select_to_dataframe_yield_selects_with_in_clauses(
-                sel,
-                conn,
-                (table.c["id"], values),
-                in_clauses_chunk_size=in_clauses_chunk_size,
-                chunk_size_frac=chunk_size_frac,
-            )
-            sels = list(iterator)
+        sels = _select_to_dataframe_yield_selects_with_in_clauses(
+            sel,
+            engine,
+            (table.c["id"], values),
+            in_clauses_chunk_size=in_clauses_chunk_size,
+            chunk_size_frac=chunk_size_frac,
+        )
         for sel in sels:
             assert isinstance(sel, Select)
