@@ -1,7 +1,7 @@
 from __future__ import annotations
 
 from hypothesis import given
-from hypothesis.strategies import lists
+from hypothesis.strategies import integers, lists
 from pytest import mark, param, raises
 
 from utilities.hypothesis import text_ascii
@@ -12,7 +12,9 @@ from utilities.text import (
     ensure_bytes,
     ensure_str,
     join_strs,
+    repr_encode,
     split_str,
+    str_encode,
     strip_and_dedent,
 )
 
@@ -55,6 +57,14 @@ class TestEnsureStr:
             _ = ensure_str(sentinel, nullable=nullable)
 
 
+class TestReprEncode:
+    @given(n=integers())
+    def test_main(self, *, n: int) -> None:
+        result = repr_encode(n)
+        expected = repr(n).encode()
+        assert result == expected
+
+
 class TestSplitStrAndJoinStr:
     @mark.parametrize(
         ("text", "texts"),
@@ -75,12 +85,21 @@ class TestSplitStrAndJoinStr:
         assert split_str(join_strs(texts)) == texts
 
 
+class TestStrEncode:
+    @given(n=integers())
+    def test_main(self, *, n: int) -> None:
+        result = str_encode(n)
+        expected = str(n).encode()
+        assert result == expected
+
+
 class TestStripAndDedent:
-    def test_main(self) -> None:
+    @mark.parametrize("trailing", [param(True), param(False)])
+    def test_main(self, *, trailing: bool) -> None:
         text = """
                This is line 1.
                This is line 2.
                """
-        result = strip_and_dedent(text)
-        expected = "This is line 1.\nThis is line 2."
+        result = strip_and_dedent(text, trailing=trailing)
+        expected = "This is line 1.\nThis is line 2." + ("\n" if trailing else "")
         assert result == expected
