@@ -17,12 +17,12 @@ from typing import (
 
 from typing_extensions import override
 
-from utilities.platform import SYSTEM, System
-from utilities.types import ensure_not_none
+from utilities.functions import ensure_not_none
+from utilities.platform import SYSTEM
 from utilities.zoneinfo import (
-    HONG_KONG,
-    TOKYO,
     UTC,
+    HongKong,
+    Tokyo,
     ensure_time_zone,
     get_time_zone_name,
 )
@@ -33,11 +33,14 @@ if TYPE_CHECKING:
 
     from utilities.types import Duration
 
+
 _DAYS_PER_YEAR = 365.25
 _MICROSECONDS_PER_MILLISECOND = int(1e3)
 _MICROSECONDS_PER_SECOND = int(1e6)
 _SECONDS_PER_DAY = 24 * 60 * 60
 _MICROSECONDS_PER_DAY = _MICROSECONDS_PER_SECOND * _SECONDS_PER_DAY
+MICROSECOND = dt.timedelta(microseconds=1)
+MILLISECOND = dt.timedelta(milliseconds=1)
 SECOND = dt.timedelta(seconds=1)
 MINUTE = dt.timedelta(minutes=1)
 HOUR = dt.timedelta(hours=1)
@@ -58,10 +61,10 @@ def add_weekdays(date: dt.date, /, *, n: int = 1) -> dt.date:
         raise AddWeekdaysError(date)
     if n >= 1:
         for _ in range(n):
-            date = round_to_next_weekday(date + dt.timedelta(days=1))
+            date = round_to_next_weekday(date + DAY)
     elif n <= -1:
         for _ in range(-n):
-            date = round_to_prev_weekday(date - dt.timedelta(days=1))
+            date = round_to_prev_weekday(date - DAY)
     return date
 
 
@@ -70,11 +73,11 @@ class AddWeekdaysError(Exception): ...
 
 def check_date_not_datetime(date: dt.date, /) -> None:
     """Check if a date is not a datetime."""
-    if isinstance(date, dt.datetime):
+    if not is_instance_date_not_datetime(date):
         raise CheckDateNotDatetimeError(date=date)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckDateNotDatetimeError(Exception):
     date: dt.date
 
@@ -89,7 +92,7 @@ def check_zoned_datetime(datetime: dt.datetime, /) -> None:
         raise CheckZonedDatetimeError(datetime=datetime)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckZonedDatetimeError(Exception):
     datetime: dt.datetime
 
@@ -144,13 +147,29 @@ def duration_to_timedelta(duration: Duration, /) -> dt.timedelta:
 
 def ensure_month(month: Month | str, /) -> Month:
     """Ensure the object is a month."""
-    return month if isinstance(month, Month) else parse_month(month)
+    if isinstance(month, Month):
+        return month
+    try:
+        return parse_month(month)
+    except ParseMonthError as error:
+        raise EnsureMonthError(month=error.month) from None
+
+
+@dataclass(kw_only=True, slots=True)
+class EnsureMonthError(Exception):
+    month: str
+
+    @override
+    def __str__(self) -> str:
+        return f"Unable to ensure month; got {self.month!r}"
 
 
 def format_datetime_local_and_utc(datetime: dt.datetime, /) -> str:
     """Format a local datetime locally & in UTC."""
     check_zoned_datetime(datetime)
-    time_zone = ensure_time_zone(ensure_not_none(datetime.tzinfo))
+    time_zone = ensure_time_zone(
+        ensure_not_none(datetime.tzinfo, desc="datetime.tzinfo")
+    )
     if time_zone is UTC:
         return datetime.strftime("%Y-%m-%d %H:%M:%S (%a, UTC)")
     as_utc = datetime.astimezone(UTC)
@@ -198,7 +217,7 @@ NOW_UTC = get_now(time_zone=UTC)
 
 def get_now_hk() -> dt.datetime:
     """Get the current time in Hong Kong."""
-    return dt.datetime.now(tz=HONG_KONG)
+    return dt.datetime.now(tz=HongKong)
 
 
 NOW_HK = get_now_hk()
@@ -206,7 +225,7 @@ NOW_HK = get_now_hk()
 
 def get_now_tokyo() -> dt.datetime:
     """Get the current time in Tokyo."""
-    return dt.datetime.now(tz=TOKYO)
+    return dt.datetime.now(tz=Tokyo)
 
 
 NOW_TOKYO = get_now_tokyo()
@@ -253,11 +272,6 @@ def get_years(*, n: int = 1) -> dt.timedelta:
 YEAR = get_years(n=1)
 
 
-def isinstance_date_not_datetime(obj: Any, /) -> TypeGuard[dt.date]:
-    """Check if an object is a date, and not a datetime."""
-    return isinstance(obj, dt.date) and not isinstance(obj, dt.datetime)
-
-
 def is_equal_mod_tz(x: dt.datetime, y: dt.datetime, /) -> bool:
     """Check if x == y, modulo timezone."""
     x_aware, y_aware = x.tzinfo is not None, y.tzinfo is not None
@@ -268,11 +282,21 @@ def is_equal_mod_tz(x: dt.datetime, y: dt.datetime, /) -> bool:
     return x == y
 
 
+def is_instance_date_not_datetime(obj: Any, /) -> TypeGuard[dt.date]:
+    """Check if an object is a date, and not a datetime."""
+    return isinstance(obj, dt.date) and not isinstance(obj, dt.datetime)
+
+
 def is_local_datetime(obj: Any, /) -> TypeGuard[dt.datetime]:
     """Check if an object is a local datetime."""
     return isinstance(obj, dt.datetime) and (obj.tzinfo is None)
 
 
+def is_subclass_date_not_datetime(obj: type[Any], /) -> TypeGuard[type[dt.date]]:
+    """Check if a class is a date, and not a datetime."""
+    return issubclass(obj, dt.date) and not issubclass(obj, dt.datetime)
+
+
 _FRIDAY = 5
 
 
@@ -290,11 +314,11 @@ def is_zoned_datetime(obj: Any, /) -> TypeGuard[dt.datetime]:
 def maybe_sub_pct_y(text: str, /) -> str:
     """Substitute the `%Y' token with '%4Y' if necessary."""
     match SYSTEM:
-        case System.windows:  # skipif-os-ne-windows
+        case "windows":  # skipif-not-windows
             return text
-        case System.mac:  # skipif-os-ne-macos
+        case "mac":  # skipif-not-macos
             return text
-        case System.linux:  # skipif-os-ne-linux
+        case "linux":  # skipif-not-linux
             return sub("%Y", "%4Y", text)
         case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
             assert_never(never)
@@ -342,7 +366,7 @@ def milliseconds_since_epoch(
     return milliseconds + remainder / _MICROSECONDS_PER_MILLISECOND
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class MillisecondsSinceEpochError(Exception):
     datetime: dt.datetime
     remainder: int
@@ -362,7 +386,7 @@ def milliseconds_to_timedelta(milliseconds: int, /) -> dt.timedelta:
     return microseconds_to_timedelta(_MICROSECONDS_PER_MILLISECOND * milliseconds)
 
 
-@dataclass(order=True, frozen=True)
+@dataclass(order=True, unsafe_hash=True, slots=True)
 class Month:
     """Represents a month in time."""
 
@@ -391,10 +415,18 @@ class Month:
         year = self.year + years
         return replace(self, year=year, month=month)
 
-    def __sub__(self, other: Any, /) -> Self:
-        if not isinstance(other, int):  # pragma: no cover
-            return NotImplemented
-        return self + (-other)
+    @overload
+    def __sub__(self, other: Self, /) -> int: ...
+    @overload
+    def __sub__(self, other: int, /) -> Self: ...
+    def __sub__(self, other: Self | int, /) -> Self | int:
+        if isinstance(other, int):  # pragma: no cover
+            return self + (-other)
+        if isinstance(other, type(self)):
+            self_as_int = 12 * self.year + self.month
+            other_as_int = 12 * other.year + other.month
+            return self_as_int - other_as_int
+        return NotImplemented  # pragma: no cover
 
     @classmethod
     def from_date(cls, date: dt.date, /) -> Self:
@@ -405,7 +437,7 @@ class Month:
         return dt.date(self.year, self.month, day)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class MonthError(Exception):
     year: int
     month: int
@@ -431,7 +463,7 @@ def parse_month(month: str, /) -> Month:
     raise ParseMonthError(month=month)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class ParseMonthError(Exception):
     month: str
 
@@ -443,7 +475,7 @@ class ParseMonthError(Exception):
 _TPeriod = TypeVar("_TPeriod", dt.date, dt.datetime)
 
 
-@dataclass(slots=True)
+@dataclass(unsafe_hash=True, slots=True)
 class Period(Generic[_TPeriod]):
     """A period of time."""
 
@@ -451,9 +483,9 @@ class Period(Generic[_TPeriod]):
     end: _TPeriod
 
     def __post_init__(self) -> None:
-        if isinstance_date_not_datetime(self.start) is not isinstance_date_not_datetime(
-            self.end
-        ):
+        if is_instance_date_not_datetime(
+            self.start
+        ) is not is_instance_date_not_datetime(self.end):
             raise _PeriodDateAndDatetimeMixedError(start=self.start, end=self.end)
         for date in [self.start, self.end]:
             if isinstance(date, dt.datetime):
@@ -472,27 +504,27 @@ class Period(Generic[_TPeriod]):
         return self.end - self.start
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class PeriodError(Generic[_TPeriod], Exception):
     start: _TPeriod
     end: _TPeriod
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _PeriodDateAndDatetimeMixedError(PeriodError[_TPeriod]):
     @override
     def __str__(self) -> str:
         return f"Invalid period; got date and datetime mix ({self.start}, {self.end})"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _PeriodNaiveDatetimeError(PeriodError[_TPeriod]):
     @override
     def __str__(self) -> str:
         return f"Invalid period; got naive datetime(s) ({self.start}, {self.end})"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _PeriodInvalidError(PeriodError[_TPeriod]):
     @override
     def __str__(self) -> str:
@@ -567,7 +599,7 @@ def timedelta_to_milliseconds(
     return milliseconds + remainder / _MICROSECONDS_PER_MILLISECOND
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class TimedeltaToMillisecondsError(Exception):
     timedelta: dt.timedelta
     remainder: int
@@ -606,7 +638,7 @@ def yield_days(
     raise YieldDaysError(start=start, end=end, days=days)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class YieldDaysError(Exception):
     start: dt.date | None
     end: dt.date | None
@@ -648,7 +680,7 @@ def yield_weekdays(
     raise YieldWeekdaysError(start=start, end=end, days=days)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class YieldWeekdaysError(Exception):
     start: dt.date | None
     end: dt.date | None
@@ -668,6 +700,7 @@ __all__ = [
     "HALF_YEAR",
     "HOUR",
     "MAX_MONTH",
+    "MILLISECOND",
     "MINUTE",
     "MIN_MONTH",
     "MONTH",
@@ -684,6 +717,7 @@ __all__ = [
     "AddWeekdaysError",
     "CheckDateNotDatetimeError",
     "CheckZonedDatetimeError",
+    "EnsureMonthError",
     "MillisecondsSinceEpochError",
     "Month",
     "MonthError",
@@ -714,10 +748,11 @@ __all__ = [
     "get_today_hk",
     "get_today_tokyo",
     "get_years",
+    "is_instance_date_not_datetime",
     "is_local_datetime",
+    "is_subclass_date_not_datetime",
     "is_weekday",
     "is_zoned_datetime",
-    "isinstance_date_not_datetime",
     "maybe_sub_pct_y",
     "microseconds_since_epoch",
     "microseconds_since_epoch_to_datetime",
