@@ -15,9 +15,7 @@ from utilities.iterables import (
 )
 
 if TYPE_CHECKING:
-    from collections.abc import Hashable, Mapping
-
-    from utilities.types import IterableStrs
+    from collections.abc import Hashable, Iterable, Mapping
 
 
 def snake_case(text: str, /) -> str:
@@ -28,7 +26,7 @@ def snake_case(text: str, /) -> str:
     return text.lower()
 
 
-def snake_case_mappings(text: IterableStrs, /) -> dict[str, str]:
+def snake_case_mappings(text: Iterable[str], /) -> dict[str, str]:
     """Map a set of text into their snake cases."""
     keys = list(text)
     try:
@@ -48,20 +46,20 @@ def snake_case_mappings(text: IterableStrs, /) -> dict[str, str]:
     return mapping
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class SnakeCaseMappingsError(Exception):
     text: list[str]
     counts: Mapping[Hashable, int]
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _SnakeCaseMappingsDuplicateKeysError(SnakeCaseMappingsError):
     @override
     def __str__(self) -> str:
         return f"Strings {self.text} must not contain duplicates; got {self.counts}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _SnakeCaseMappingsDuplicateValuesError(SnakeCaseMappingsError):
     @override
     def __str__(self) -> str:
