@@ -2,16 +2,23 @@ from __future__ import annotations
 
 import builtins
 import datetime as dt
-from collections.abc import Collection, Hashable, Iterable, Iterator, Mapping
-from contextlib import contextmanager, suppress
+from collections.abc import AsyncIterator, Collection, Hashable, Iterable, Iterator
+from contextlib import (
+    AbstractAsyncContextManager,
+    asynccontextmanager,
+    contextmanager,
+    suppress,
+)
+from datetime import timezone
 from enum import Enum, auto
 from math import ceil, floor, inf, isfinite, nan
 from os import environ
 from pathlib import Path
 from re import search
-from string import ascii_letters, printable
+from string import ascii_letters, digits, printable
 from subprocess import run
 from typing import TYPE_CHECKING, Any, Protocol, TypeVar, assert_never, cast, overload
+from zoneinfo import ZoneInfo
 
 from hypothesis import HealthCheck, Phase, Verbosity, assume, settings
 from hypothesis.errors import InvalidArgument
@@ -30,15 +37,24 @@ from hypothesis.strategies import (
     lists,
     none,
     sampled_from,
+    sets,
     text,
     timedeltas,
     uuids,
 )
 from hypothesis.utils.conventions import not_set
-from typing_extensions import override
 
-from utilities.datetime import MAX_MONTH, MIN_MONTH, Month, date_to_month
-from utilities.math import MAX_LONG, MIN_LONG
+from utilities.datetime import MAX_MONTH, MIN_MONTH, Month, date_to_month, get_now
+from utilities.math import (
+    MAX_INT32,
+    MAX_INT64,
+    MAX_UINT32,
+    MAX_UINT64,
+    MIN_INT32,
+    MIN_INT64,
+    MIN_UINT32,
+    MIN_UINT64,
+)
 from utilities.pathlib import temp_cwd
 from utilities.platform import IS_WINDOWS
 from utilities.tempfile import TEMP_DIR, TemporaryDirectory
@@ -46,48 +62,19 @@ from utilities.text import ensure_str
 from utilities.zoneinfo import UTC
 
 if TYPE_CHECKING:
-    from uuid import UUID
-
-    import redis
     from hypothesis.database import ExampleDatabase
-    from pandas import Timestamp
-    from redis.commands.timeseries import TimeSeries
-    from semver import Version
-    from sqlalchemy import Engine, MetaData
+    from numpy.random import RandomState
     from sqlalchemy.ext.asyncio import AsyncEngine
 
-    from utilities.math import FloatFinPos, IntNonNeg
-    from utilities.numpy import NDArrayA, NDArrayB, NDArrayF, NDArrayI, NDArrayO
-    from utilities.pandas import IndexA, IndexI, IndexS
-    from utilities.xarray import DataArrayB, DataArrayF, DataArrayI, DataArrayO
+    from utilities.numpy import NDArrayB, NDArrayF, NDArrayI, NDArrayO
+    from utilities.redis import _TestRedis
+    from utilities.sqlalchemy import Dialect, TableOrORMInstOrClass
+    from utilities.types import Duration, Number
 
 
 _T = TypeVar("_T")
 MaybeSearchStrategy = _T | SearchStrategy[_T]
 Shape = int | tuple[int, ...]
-_INDEX_LENGTHS = integers(0, 10)
-
-
-async def aiosqlite_engines(
-    data: DataObject, /, *, metadata: MetaData | None = None, base: Any = None
-) -> AsyncEngine:
-    from utilities.sqlalchemy import create_engine
-
-    temp_path = data.draw(temp_paths())
-    path = Path(temp_path, "db.sqlite")
-    engine = create_engine("sqlite+aiosqlite", database=str(path), async_=True)
-    if metadata is not None:
-        async with engine.begin() as conn:  # pragma: no cover
-            await conn.run_sync(metadata.create_all)
-    if base is not None:
-        async with engine.begin() as conn:  # pragma: no cover
-            await conn.run_sync(base.metadata.create_all)
-
-    class EngineWithPath(type(engine)): ...
-
-    engine_with_path = EngineWithPath(engine.sync_engine)
-    cast(Any, engine_with_path).temp_path = temp_path  # keep `temp_path` alive
-    return engine_with_path
 
 
 @contextmanager
@@ -111,46 +98,6 @@ def assume_does_not_raise(
                 raise
 
 
-@composite
-def bool_data_arrays(
-    _draw: DrawFn,
-    indexes: MaybeSearchStrategy[Mapping[str, IndexA]] | None = None,
-    /,
-    *,
-    fill: SearchStrategy[Any] | None = None,
-    unique: MaybeSearchStrategy[bool] = False,
-    name: MaybeSearchStrategy[str | None] = None,
-    **indexes_kwargs: MaybeSearchStrategy[IndexA],
-) -> DataArrayB:
-    """Strategy for generating data arrays of booleans."""
-    from xarray import DataArray
-
-    draw = lift_draw(_draw)
-    indexes_ = draw(_merge_into_dict_of_indexes(indexes, **indexes_kwargs))
-    shape = tuple(map(len, indexes_.values()))
-    values = draw(bool_arrays(shape=shape, fill=fill, unique=unique))
-    return DataArray(data=values, coords=indexes_, dims=list(indexes_), name=draw(name))
-
-
-@composite
-def datetimes_utc(
-    _draw: DrawFn,
-    /,
-    *,
-    min_value: MaybeSearchStrategy[dt.datetime] = dt.datetime.min,
-    max_value: MaybeSearchStrategy[dt.datetime] = dt.datetime.max,
-) -> dt.datetime:
-    """Strategy for generating datetimes with the UTC timezone."""
-    draw = lift_draw(_draw)
-    return draw(
-        datetimes(
-            min_value=draw(min_value).replace(tzinfo=None),
-            max_value=draw(max_value).replace(tzinfo=None),
-            timezones=just(UTC),
-        )
-    )
-
-
 @composite
 def bool_arrays(
     _draw: DrawFn,
@@ -172,116 +119,39 @@ def bool_arrays(
 
 
 @composite
-def concatenated_arrays(
-    _draw: DrawFn,
-    strategy: SearchStrategy[NDArrayA],
-    size: MaybeSearchStrategy[IntNonNeg],
-    fallback: Shape,
-    /,
-    *,
-    dtype: Any = float,
-) -> NDArrayA:
-    """Strategy for generating arrays from lower-dimensional strategies."""
-    from numpy import concatenate, expand_dims, zeros
-
-    from utilities.numpy import (
-        EmptyNumpyConcatenateError,
-        redirect_empty_numpy_concatenate,
-    )
-
-    draw = lift_draw(_draw)
-    size_ = draw(size)
-    arrays = draw(lists_fixed_length(strategy, size_))
-    expanded = [expand_dims(array, axis=0) for array in arrays]
-    try:
-        with redirect_empty_numpy_concatenate():
-            return concatenate(expanded)
-    except EmptyNumpyConcatenateError:
-        shape = (size_, fallback) if isinstance(fallback, int) else (size_, *fallback)
-        return zeros(shape, dtype=dtype)
-
-
-@composite
-def dates_pd(
+def durations(
     _draw: DrawFn,
     /,
     *,
-    min_value: MaybeSearchStrategy[dt.date] | None = None,
-    max_value: MaybeSearchStrategy[dt.date] | None = None,
-) -> dt.date:
-    """Strategy for generating dates which can become Timestamps."""
-    from utilities.pandas import TIMESTAMP_MAX_AS_DATE, TIMESTAMP_MIN_AS_DATE
-
-    min_value_use = TIMESTAMP_MIN_AS_DATE if min_value is None else min_value
-    max_value_use = TIMESTAMP_MAX_AS_DATE if max_value is None else max_value
+    min_number: MaybeSearchStrategy[Number] | None = None,
+    max_number: MaybeSearchStrategy[Number] | None = None,
+    min_timedelta: MaybeSearchStrategy[dt.timedelta] = dt.timedelta.min,
+    max_timedelta: MaybeSearchStrategy[dt.timedelta] = dt.timedelta.max,
+    two_way: bool = False,
+) -> Duration:
+    """Strategy for generating durations."""
     draw = lift_draw(_draw)
-    return draw(dates(min_value=draw(min_value_use), max_value=draw(max_value_use)))
-
-
-@composite
-def datetimes_pd(
-    _draw: DrawFn,
-    /,
-    *,
-    min_value: MaybeSearchStrategy[dt.datetime] | None = None,
-    max_value: MaybeSearchStrategy[dt.datetime] | None = None,
-) -> dt.datetime:
-    """Strategy for generating datetimes which can become Timestamps."""
-    from utilities.pandas import TIMESTAMP_MAX_AS_DATETIME, TIMESTAMP_MIN_AS_DATETIME
-
-    draw = lift_draw(_draw)
-    min_value_use = TIMESTAMP_MIN_AS_DATETIME if min_value is None else min_value
-    max_value_use = TIMESTAMP_MAX_AS_DATETIME if max_value is None else max_value
-    datetime = draw(
-        datetimes(
-            min_value=draw(min_value_use).replace(tzinfo=None),
-            max_value=draw(max_value_use).replace(tzinfo=None),
+    min_number_, max_number_ = draw(min_number), draw(max_number)
+    min_timedelta_, max_timedelta_ = draw(min_timedelta), draw(max_timedelta)
+    if isinstance(min_number_, float) or isinstance(max_number_, float):
+        st_numbers = floats(
+            min_value=min_number_,
+            max_value=max_number_,
+            allow_nan=False,
+            allow_infinity=False,
         )
-    )
-    return datetime.replace(tzinfo=UTC)
-
-
-@composite
-def draw_text(
-    _draw: DrawFn,
-    alphabet: MaybeSearchStrategy[str],
-    /,
-    *,
-    min_size: MaybeSearchStrategy[int] = 0,
-    max_size: MaybeSearchStrategy[int | None] = None,
-    disallow_na: MaybeSearchStrategy[bool] = False,
-) -> str:
-    """Draw from a text-generating strategy."""
-    draw = lift_draw(_draw)
-    drawn = draw(text(alphabet, min_size=draw(min_size), max_size=draw(max_size)))
-    if draw(disallow_na):
-        _ = assume(drawn != "NA")
-    return drawn
-
-
-@composite
-def dicts_of_indexes(
-    _draw: DrawFn,
-    /,
-    *,
-    min_dims: int = 1,
-    max_dims: int | None = None,
-    min_side: int = 1,
-    max_side: int | None = None,
-) -> dict[str, IndexI]:
-    """Strategy for generating dictionaries of indexes."""
-    from hypothesis.extra.numpy import array_shapes
-
-    draw = lift_draw(_draw)
-    shape = draw(
-        array_shapes(
-            min_dims=min_dims, max_dims=max_dims, min_side=min_side, max_side=max_side
+    else:
+        st_numbers = integers(
+            min_value=min_number_ if isinstance(min_number_, int) else None,
+            max_value=max_number_ if isinstance(max_number_, int) else None,
         )
-    )
-    ndims = len(shape)
-    dims = draw(lists_fixed_length(text_ascii(), ndims, unique=True))
-    indexes = (draw(int_indexes(n=length)) for length in shape)
-    return dict(zip(dims, indexes, strict=True))
+    if two_way:
+        from utilities.whenever import MAX_TWO_WAY_TIMEDELTA, MIN_TWO_WAY_TIMEDELTA
+
+        min_timedelta_ = max(min_timedelta_, MIN_TWO_WAY_TIMEDELTA)
+        max_timedelta_ = min(max_timedelta_, MAX_TWO_WAY_TIMEDELTA)
+    st_timedeltas = timedeltas(min_value=min_timedelta_, max_value=max_timedelta_)
+    return _draw(st_numbers | st_timedeltas)
 
 
 @composite
@@ -320,47 +190,6 @@ def float_arrays(
     return draw(strategy)
 
 
-@composite
-def float_data_arrays(
-    _draw: DrawFn,
-    indexes: MaybeSearchStrategy[Mapping[str, IndexA]] | None = None,
-    /,
-    *,
-    min_value: MaybeSearchStrategy[float | None] = None,
-    max_value: MaybeSearchStrategy[float | None] = None,
-    allow_nan: MaybeSearchStrategy[bool] = False,
-    allow_inf: MaybeSearchStrategy[bool] = False,
-    allow_pos_inf: MaybeSearchStrategy[bool] = False,
-    allow_neg_inf: MaybeSearchStrategy[bool] = False,
-    integral: MaybeSearchStrategy[bool] = False,
-    fill: SearchStrategy[Any] | None = None,
-    unique: MaybeSearchStrategy[bool] = False,
-    name: MaybeSearchStrategy[str | None] = None,
-    **indexes_kwargs: MaybeSearchStrategy[IndexA],
-) -> DataArrayF:
-    """Strategy for generating data arrays of floats."""
-    from xarray import DataArray
-
-    draw = lift_draw(_draw)
-    indexes_ = draw(_merge_into_dict_of_indexes(indexes, **indexes_kwargs))
-    shape = tuple(map(len, indexes_.values()))
-    values = draw(
-        float_arrays(
-            shape=shape,
-            min_value=min_value,
-            max_value=max_value,
-            allow_nan=allow_nan,
-            allow_inf=allow_inf,
-            allow_pos_inf=allow_pos_inf,
-            allow_neg_inf=allow_neg_inf,
-            integral=integral,
-            fill=fill,
-            unique=unique,
-        )
-    )
-    return DataArray(data=values, coords=indexes_, dims=list(indexes_), name=draw(name))
-
-
 @composite
 def floats_extra(
     _draw: DrawFn,
@@ -426,38 +255,6 @@ def git_repos(
     return path
 
 
-@composite
-def indexes(
-    _draw: DrawFn,
-    /,
-    *,
-    elements: SearchStrategy[Any] | None = None,
-    dtype: Any = None,
-    n: MaybeSearchStrategy[int] = _INDEX_LENGTHS,
-    unique: MaybeSearchStrategy[bool] = True,
-    name: MaybeSearchStrategy[Hashable] = None,
-    sort: MaybeSearchStrategy[bool] = False,
-) -> IndexA:
-    """Strategy for generating Indexes."""
-    from hypothesis.extra.pandas import indexes as _indexes
-
-    draw = lift_draw(_draw)
-    n_ = draw(n)
-    index = draw(
-        _indexes(
-            elements=elements,
-            dtype=dtype,
-            min_size=n_,
-            max_size=n_,
-            unique=draw(unique),
-        )
-    )
-    index = index.rename(draw(name))
-    if draw(sort):
-        return index.sort_values()
-    return index
-
-
 def hashables() -> SearchStrategy[Hashable]:
     """Strategy for generating hashable elements."""
     return booleans() | integers() | none() | text_ascii()
@@ -469,22 +266,18 @@ def int_arrays(
     /,
     *,
     shape: MaybeSearchStrategy[Shape] | None = None,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
+    min_value: MaybeSearchStrategy[int] = MIN_INT64,
+    max_value: MaybeSearchStrategy[int] = MAX_INT64,
     fill: SearchStrategy[Any] | None = None,
     unique: MaybeSearchStrategy[bool] = False,
 ) -> NDArrayI:
     """Strategy for generating arrays of ints."""
     from hypothesis.extra.numpy import array_shapes, arrays
-    from numpy import iinfo, int64
+    from numpy import int64
 
     draw = lift_draw(_draw)
     shape_use = array_shapes() if shape is None else shape
-    info = iinfo(int64)
-    min_value_, max_value_ = draw(min_value), draw(max_value)
-    min_value_use = info.min if min_value_ is None else min_value_
-    max_value_use = info.max if max_value_ is None else max_value_
-    elements = integers(min_value=min_value_use, max_value=max_value_use)
+    elements = int64s(min_value=min_value, max_value=max_value)
     strategy: SearchStrategy[NDArrayI] = arrays(
         int64, draw(shape_use), elements=elements, fill=fill, unique=draw(unique)
     )
@@ -492,86 +285,34 @@ def int_arrays(
 
 
 @composite
-def int_data_arrays(
+def int32s(
     _draw: DrawFn,
-    indexes: MaybeSearchStrategy[Mapping[str, IndexA]] | None = None,
     /,
     *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-    fill: SearchStrategy[Any] | None = None,
-    unique: MaybeSearchStrategy[bool] = False,
-    name: MaybeSearchStrategy[str | None] = None,
-    **indexes_kwargs: MaybeSearchStrategy[IndexA],
-) -> DataArrayI:
-    """Strategy for generating data arrays of ints."""
-    from xarray import DataArray
-
-    draw = lift_draw(_draw)
-    indexes_ = draw(_merge_into_dict_of_indexes(indexes, **indexes_kwargs))
-    shape = tuple(map(len, indexes_.values()))
-    values = draw(
-        int_arrays(
-            shape=shape,
-            min_value=min_value,
-            max_value=max_value,
-            fill=fill,
-            unique=unique,
-        )
-    )
-    return DataArray(data=values, coords=indexes_, dims=list(indexes_), name=draw(name))
-
-
-def int_indexes(
-    *,
-    n: MaybeSearchStrategy[int] = _INDEX_LENGTHS,
-    unique: MaybeSearchStrategy[bool] = True,
-    name: MaybeSearchStrategy[Hashable] = None,
-    sort: MaybeSearchStrategy[bool] = False,
-) -> SearchStrategy[IndexI]:
-    """Strategy for generating integer Indexes."""
-    from numpy import int64
-
-    return indexes(
-        elements=int64s(), dtype=int64, n=n, unique=unique, name=name, sort=sort
-    )
-
-
-def int32s(
-    *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-) -> SearchStrategy[int]:
+    min_value: MaybeSearchStrategy[int] = MIN_INT32,
+    max_value: MaybeSearchStrategy[int] = MAX_INT32,
+) -> int:
     """Strategy for generating int32s."""
-    from numpy import int32
-
-    return _fixed_width_ints(int32, min_value=min_value, max_value=max_value)
-
-
-def int64s(
-    *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-) -> SearchStrategy[int]:
-    """Strategy for generating int64s."""
-    from numpy import int64
-
-    return _fixed_width_ints(int64, min_value=min_value, max_value=max_value)
+    draw = lift_draw(_draw)
+    min_value_, max_value_ = draw(min_value), draw(max_value)
+    min_value_ = max(min_value_, MIN_INT32)
+    max_value_ = min(max_value_, MAX_INT32)
+    return draw(integers(min_value_, max_value_))
 
 
 @composite
-def longs(
+def int64s(
     _draw: DrawFn,
     /,
     *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
+    min_value: MaybeSearchStrategy[int] = MIN_INT64,
+    max_value: MaybeSearchStrategy[int] = MAX_INT64,
 ) -> int:
-    """Strategy for generating longs (long integers)."""
+    """Strategy for generating int64s."""
     draw = lift_draw(_draw)
-    min_value_, max_value_ = (draw(mv) for mv in (min_value, max_value))
-    min_value_ = MIN_LONG if min_value_ is None else max(MIN_LONG, min_value_)
-    max_value_ = MAX_LONG if max_value_ is None else min(MAX_LONG, max_value_)
+    min_value_, max_value_ = draw(min_value), draw(max_value)
+    min_value_ = max(min_value_, MIN_INT64)
+    max_value_ = min(max_value_, MAX_INT64)
     return draw(integers(min_value_, max_value_))
 
 
@@ -587,13 +328,22 @@ class _MaybeDrawFn(Protocol):
         raise NotImplementedError(obj)  # pragma: no cover
 
 
+def lift_data(data: DataObject, /) -> _MaybeDrawFn:
+    """Lift the `data` object to handle non-`SearchStrategy` types."""
+
+    def func(obj: MaybeSearchStrategy[_MDF], /) -> _MDF:
+        return data.draw(obj) if isinstance(obj, SearchStrategy) else obj
+
+    return cast(_MaybeDrawFn, func)
+
+
 def lift_draw(draw: DrawFn, /) -> _MaybeDrawFn:
     """Lift the `draw` function to handle non-`SearchStrategy` types."""
 
     def func(obj: MaybeSearchStrategy[_MDF], /) -> _MDF:
         return draw(obj) if isinstance(obj, SearchStrategy) else obj
 
-    return cast(Any, func)
+    return cast(_MaybeDrawFn, func)
 
 
 @composite
@@ -634,42 +384,26 @@ def months(
 
 
 @composite
-def namespace_mixins(_draw: DrawFn, /) -> type:
-    """Strategy for generating task namespace mixins."""
+def random_states(
+    _draw: DrawFn, /, *, seed: MaybeSearchStrategy[int | None] = None
+) -> RandomState:
+    """Strategy for generating `numpy` random states."""
+    from numpy.random import RandomState
+
     draw = lift_draw(_draw)
-    path = draw(temp_paths())
-
-    class NamespaceMixin:
-        task_namespace = path.name
-
-    return NamespaceMixin
+    seed_ = draw(seed)
+    seed_use = draw(integers(0, MAX_UINT32)) if seed_ is None else seed_
+    return RandomState(seed=seed_use)
 
 
 @composite
-def redis_clients(_draw: DrawFn, /) -> tuple[redis.Redis, UUID]:
-    """Strategy for generating redis clients."""
-    import redis  # skipif-ci-and-not-linux
-    from redis.exceptions import ResponseError  # skipif-ci-and-not-linux
-
-    uuid = _draw(uuids())  # skipif-ci-and-not-linux
-
-    class RedisWithCleanup(redis.Redis):  # skipif-ci-and-not-linux
-        @override
-        def __del__(self) -> Any:
-            keys = self.keys(pattern=f"{uuid}_*")
-            with suppress(ResponseError):
-                _ = self.delete(*cast(Iterable[Any], keys))
-            return super().__del__()
-
-    client = RedisWithCleanup(db=15, decode_responses=True)  # skipif-ci-and-not-linux
-    return client, uuid  # skipif-ci-and-not-linux
-
-
-@composite
-def redis_time_series(_draw: DrawFn, /) -> tuple[TimeSeries, UUID]:
-    """Strategy for generating redis time series."""
-    client, uuid = _draw(redis_clients())  # skipif-ci-and-not-linux
-    return client.ts(), uuid  # skipif-ci-and-not-linux
+def sets_fixed_length(
+    _draw: DrawFn, strategy: SearchStrategy[_T], size: MaybeSearchStrategy[int], /
+) -> set[_T]:
+    """Strategy for generating lists of a fixed length."""
+    draw = lift_draw(_draw)
+    size_ = draw(size)
+    return draw(sets(strategy, min_size=size_, max_size=size_))
 
 
 def setup_hypothesis_profiles(
@@ -727,7 +461,7 @@ def setup_hypothesis_profiles(
 
 
 def settings_with_reduced_examples(
-    frac: FloatFinPos = 0.1,
+    frac: float = 0.1,
     /,
     *,
     derandomize: bool = not_set,  # pyright: ignore[reportArgumentType]
@@ -762,39 +496,62 @@ def settings_with_reduced_examples(
 @composite
 def slices(
     _draw: DrawFn,
-    iter_len: int,
+    iter_len: MaybeSearchStrategy[int],
     /,
     *,
     slice_len: MaybeSearchStrategy[int | None] = None,
 ) -> slice:
     """Strategy for generating continuous slices from an iterable."""
     draw = lift_draw(_draw)
+    iter_len_ = draw(iter_len)
     if (slice_len_ := draw(slice_len)) is None:
-        slice_len_ = draw(integers(0, iter_len))
-    elif not 0 <= slice_len_ <= iter_len:
-        msg = f"Slice length {slice_len_} exceeds iterable length {iter_len}"
+        slice_len_ = draw(integers(0, iter_len_))
+    elif not 0 <= slice_len_ <= iter_len_:
+        msg = f"Slice length {slice_len_} exceeds iterable length {iter_len_}"
         raise InvalidArgument(msg)
-    start = draw(integers(0, iter_len - slice_len_))
+    start = draw(integers(0, iter_len_ - slice_len_))
     stop = start + slice_len_
     return slice(start, stop)
 
 
-@composite
-def sqlite_engines(
-    _draw: DrawFn, /, *, metadata: MetaData | None = None, base: Any = None
-) -> Engine:
-    """Strategy for generating SQLite engines."""
-    from utilities.sqlalchemy import create_engine
+_STRATEGY_DIALECTS: list[Dialect] = ["sqlite", "postgresql"]
+_SQLALCHEMY_ENGINE_DIALECTS = sampled_from(_STRATEGY_DIALECTS)
 
-    temp_path = _draw(temp_paths())
-    path = Path(temp_path, "db.sqlite")
-    engine = create_engine("sqlite", database=str(path))
-    cast(Any, engine).temp_path = temp_path  # keep `temp_path` alive
-    if metadata is not None:
-        metadata.create_all(engine)
-    if base is not None:
-        base.metadata.create_all(engine)
-    return engine
+
+async def sqlalchemy_engines(
+    _data: DataObject,
+    /,
+    *tables_or_orms: TableOrORMInstOrClass,
+    dialect: MaybeSearchStrategy[Dialect] = _SQLALCHEMY_ENGINE_DIALECTS,
+) -> AsyncEngine:
+    """Strategy for generating sqlalchemy engines."""
+    from utilities.sqlalchemy import create_async_engine
+
+    draw = lift_data(_data)
+    dialect_: Dialect = draw(dialect)
+    if "CI" in environ:  # pragma: no cover
+        _ = assume(dialect_ == "sqlite")
+    match dialect_:
+        case "sqlite":
+            temp_path = draw(temp_paths())
+            path = Path(temp_path, "db.sqlite")
+            engine = create_async_engine("sqlite+aiosqlite", database=str(path))
+
+            class EngineWithPath(type(engine)): ...
+
+            engine_with_path = EngineWithPath(engine.sync_engine)
+            cast(Any, engine_with_path).temp_path = temp_path  # keep `temp_path` alive
+            return engine_with_path
+        case "postgresql":  # skipif-ci-and-not-linux
+            from utilities.sqlalchemy import ensure_tables_dropped
+
+            engine = create_async_engine(
+                "postgresql+asyncpg", host="localhost", port=5432, database="testing"
+            )
+            await ensure_tables_dropped(engine, *tables_or_orms)
+            return engine
+        case _:  # pragma: no cover
+            raise NotImplementedError(dialect)
 
 
 @composite
@@ -823,64 +580,6 @@ def str_arrays(
     return draw(strategy)
 
 
-@composite
-def str_data_arrays(
-    _draw: DrawFn,
-    indexes: MaybeSearchStrategy[Mapping[str, IndexA]] | None = None,
-    /,
-    *,
-    min_size: MaybeSearchStrategy[int] = 0,
-    max_size: MaybeSearchStrategy[int | None] = None,
-    allow_none: MaybeSearchStrategy[bool] = False,
-    fill: SearchStrategy[Any] | None = None,
-    unique: MaybeSearchStrategy[bool] = False,
-    name: MaybeSearchStrategy[str | None] = None,
-    **indexes_kwargs: MaybeSearchStrategy[IndexA],
-) -> DataArrayO:
-    """Strategy for generating data arrays of strings."""
-    from xarray import DataArray
-
-    draw = lift_draw(_draw)
-    indexes_ = draw(_merge_into_dict_of_indexes(indexes, **indexes_kwargs))
-    shape = tuple(map(len, indexes_.values()))
-    values = draw(
-        str_arrays(
-            shape=shape,
-            min_size=min_size,
-            max_size=max_size,
-            allow_none=allow_none,
-            fill=fill,
-            unique=unique,
-        )
-    )
-    return DataArray(data=values, coords=indexes_, dims=list(indexes_), name=draw(name))
-
-
-@composite
-def str_indexes(
-    _draw: DrawFn,
-    /,
-    *,
-    min_size: MaybeSearchStrategy[int] = 0,
-    max_size: MaybeSearchStrategy[int | None] = None,
-    n: MaybeSearchStrategy[int] = _INDEX_LENGTHS,
-    unique: MaybeSearchStrategy[bool] = True,
-    name: MaybeSearchStrategy[Hashable] = None,
-    sort: MaybeSearchStrategy[bool] = False,
-) -> IndexS:
-    """Strategy for generating string Indexes."""
-    from utilities.pandas import string
-
-    draw = lift_draw(_draw)
-    elements = text_ascii(min_size=min_size, max_size=max_size)
-    index = draw(
-        indexes(
-            elements=elements, dtype=object, n=n, unique=unique, name=name, sort=sort
-        )
-    )
-    return index.astype(string)
-
-
 _TEMP_DIR_HYPOTHESIS = Path(TEMP_DIR, "hypothesis")
 
 
@@ -914,7 +613,7 @@ def text_ascii(
     disallow_na: MaybeSearchStrategy[bool] = False,
 ) -> SearchStrategy[str]:
     """Strategy for generating ASCII text."""
-    return draw_text(
+    return _draw_text(
         characters(whitelist_categories=[], whitelist_characters=ascii_letters),
         min_size=min_size,
         max_size=max_size,
@@ -929,7 +628,7 @@ def text_clean(
     disallow_na: MaybeSearchStrategy[bool] = False,
 ) -> SearchStrategy[str]:
     """Strategy for generating clean text."""
-    return draw_text(
+    return _draw_text(
         characters(blacklist_categories=["Z", "C"]),
         min_size=min_size,
         max_size=max_size,
@@ -937,6 +636,21 @@ def text_clean(
     )
 
 
+def text_digits(
+    *,
+    min_size: MaybeSearchStrategy[int] = 0,
+    max_size: MaybeSearchStrategy[int | None] = None,
+    disallow_na: MaybeSearchStrategy[bool] = False,
+) -> SearchStrategy[str]:
+    """Strategy for generating ASCII text."""
+    return _draw_text(
+        characters(whitelist_categories=[], whitelist_characters=digits),
+        min_size=min_size,
+        max_size=max_size,
+        disallow_na=disallow_na,
+    )
+
+
 def text_printable(
     *,
     min_size: MaybeSearchStrategy[int] = 0,
@@ -944,7 +658,7 @@ def text_printable(
     disallow_na: MaybeSearchStrategy[bool] = False,
 ) -> SearchStrategy[str]:
     """Strategy for generating printable text."""
-    return draw_text(
+    return _draw_text(
         characters(whitelist_categories=[], whitelist_characters=printable),
         min_size=min_size,
         max_size=max_size,
@@ -957,203 +671,179 @@ def timedeltas_2w(
     _draw: DrawFn,
     /,
     *,
-    min_value: MaybeSearchStrategy[dt.timedelta] | None = None,
-    max_value: MaybeSearchStrategy[dt.timedelta] | None = None,
+    min_value: MaybeSearchStrategy[dt.timedelta] = dt.timedelta.min,
+    max_value: MaybeSearchStrategy[dt.timedelta] = dt.timedelta.max,
 ) -> dt.timedelta:
     """Strategy for generating timedeltas which can be se/deserialized."""
     from utilities.whenever import MAX_TWO_WAY_TIMEDELTA, MIN_TWO_WAY_TIMEDELTA
 
     draw = lift_draw(_draw)
-    min_value_use = MIN_TWO_WAY_TIMEDELTA if min_value is None else min_value
-    max_value_use = MAX_TWO_WAY_TIMEDELTA if max_value is None else max_value
-    return draw(
-        timedeltas(min_value=draw(min_value_use), max_value=draw(max_value_use))
+    min_value_ = max(draw(min_value), MIN_TWO_WAY_TIMEDELTA)
+    max_value_ = min(draw(max_value), MAX_TWO_WAY_TIMEDELTA)
+    return draw(timedeltas(min_value=min_value_, max_value=max_value_))
+
+
+@composite
+def uint32s(
+    _draw: DrawFn,
+    /,
+    *,
+    min_value: MaybeSearchStrategy[int] = MIN_UINT32,
+    max_value: MaybeSearchStrategy[int] = MAX_UINT32,
+) -> int:
+    """Strategy for generating uint32s."""
+    draw = lift_draw(_draw)
+    min_value_, max_value_ = draw(min_value), draw(max_value)
+    min_value_ = max(min_value_, MIN_UINT32)
+    max_value_ = min(max_value_, MAX_UINT32)
+    return draw(integers(min_value_, max_value_))
+
+
+@composite
+def uint64s(
+    _draw: DrawFn,
+    /,
+    *,
+    min_value: MaybeSearchStrategy[int] = MIN_UINT64,
+    max_value: MaybeSearchStrategy[int] = MAX_UINT64,
+) -> int:
+    """Strategy for generating uint64s."""
+    draw = lift_draw(_draw)
+    min_value_, max_value_ = draw(min_value), draw(max_value)
+    min_value_ = max(min_value_, MIN_UINT64)
+    max_value_ = min(max_value_, MAX_UINT64)
+    return draw(integers(min_value_, max_value_))
+
+
+def yield_test_redis(data: DataObject, /) -> AbstractAsyncContextManager[_TestRedis]:
+    """Strategy for generating test redis clients."""
+    from redis.exceptions import ResponseError  # skipif-ci-and-not-linux
+    from redis.typing import KeyT  # skipif-ci-and-not-linux
+
+    from utilities.redis import _TestRedis, yield_redis  #  skipif-ci-and-not-linux
+
+    draw = lift_data(data)  # skipif-ci-and-not-linux
+    now = get_now(time_zone="local")  # skipif-ci-and-not-linux
+    uuid = draw(uuids())  # skipif-ci-and-not-linux
+    key = f"{now}_{uuid}"  # skipif-ci-and-not-linux
+
+    @asynccontextmanager
+    async def func() -> AsyncIterator[_TestRedis]:  # skipif-ci-and-not-linux
+        async with yield_redis(db=15) as redis:  # skipif-ci-and-not-linux
+            keys = cast(list[KeyT], await redis.keys(pattern=f"{key}_*"))
+            with suppress(ResponseError):
+                _ = await redis.delete(*keys)
+            yield _TestRedis(redis=redis, timestamp=now, uuid=uuid, key=key)
+            keys = cast(list[KeyT], await redis.keys(pattern=f"{key}_*"))
+            with suppress(ResponseError):
+                _ = await redis.delete(*keys)
+
+    return func()  # skipif-ci-and-not-linux
+
+
+_ZONED_DATETIMES_LEFT_MOST = ZoneInfo("Asia/Manila")
+_ZONED_DATETIMES_RIGHT_MOST = ZoneInfo("Pacific/Kiritimati")
+
+
+@composite
+def zoned_datetimes(
+    _draw: DrawFn,
+    /,
+    *,
+    min_value: MaybeSearchStrategy[dt.datetime] = dt.datetime.min,
+    max_value: MaybeSearchStrategy[dt.datetime] = dt.datetime.max,
+    time_zone: MaybeSearchStrategy[ZoneInfo | timezone] = UTC,
+    valid: bool = False,
+) -> dt.datetime:
+    """Strategy for generating zoned datetimes."""
+    from utilities.whenever import (
+        CheckValidZonedDateimeError,
+        check_valid_zoned_datetime,
     )
 
-
-@composite
-def timestamps(
-    _draw: DrawFn,
-    /,
-    *,
-    min_value: MaybeSearchStrategy[dt.datetime] | None = None,
-    max_value: MaybeSearchStrategy[dt.datetime] | None = None,
-    allow_nanoseconds: MaybeSearchStrategy[bool] = False,
-) -> Timestamp:
-    """Strategy for generating Timestamps."""
-    from pandas import Timedelta, Timestamp
-
-    from utilities.pandas import TIMESTAMP_MAX_AS_DATETIME, TIMESTAMP_MIN_AS_DATETIME
-
     draw = lift_draw(_draw)
-    min_value_ = TIMESTAMP_MIN_AS_DATETIME if min_value is None else draw(min_value)
-    max_value_ = TIMESTAMP_MAX_AS_DATETIME if max_value is None else draw(max_value)
-    datetime = draw(datetimes_pd(min_value=min_value_, max_value=max_value_))
-    timestamp = cast(Timestamp, Timestamp(datetime))
-    if draw(allow_nanoseconds):
-        nanoseconds = draw(integers(-999, 999))
-        timedelta = Timedelta(nanoseconds=nanoseconds)
-        timestamp = cast(Timestamp, timestamp + timedelta)
-        _ = assume(min_value_ <= timestamp.floor("us"))
-        _ = assume(timestamp.ceil("us") <= max_value_)
-    return timestamp
-
-
-def uint32s(
-    *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-) -> SearchStrategy[int]:
-    """Strategy for generating uint32s."""
-    from numpy import uint32
-
-    return _fixed_width_ints(uint32, min_value=min_value, max_value=max_value)
-
-
-def uint64s(
-    *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-) -> SearchStrategy[int]:
-    """Strategy for generating uint64s."""
-    from numpy import uint64
-
-    return _fixed_width_ints(uint64, min_value=min_value, max_value=max_value)
+    min_value_, max_value_, time_zone_ = (
+        draw(min_value),
+        draw(max_value),
+        draw(time_zone),
+    )
+    if min_value_.tzinfo is not None:
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            min_value_ = min_value_.astimezone(time_zone_)
+        min_value_ = min_value_.replace(tzinfo=None)
+    if max_value_.tzinfo is not None:
+        with assume_does_not_raise(OverflowError, match="date value out of range"):
+            max_value_ = max_value_.astimezone(time_zone_)
+        max_value_ = max_value_.replace(tzinfo=None)
+    strategy = datetimes(
+        min_value=min_value_, max_value=max_value_, timezones=just(time_zone_)
+    )
+    datetime = draw(strategy)
+    with assume_does_not_raise(OverflowError, match="date value out of range"):
+        _ = datetime.astimezone(_ZONED_DATETIMES_LEFT_MOST)  # for dt.datetime.min
+    with assume_does_not_raise(OverflowError, match="date value out of range"):
+        _ = datetime.astimezone(  # for dt.datetime.max
+            _ZONED_DATETIMES_RIGHT_MOST
+        )
+    result = datetime.astimezone(time_zone_)
+    if valid:
+        with assume_does_not_raise(  # skipif-ci-and-windows
+            CheckValidZonedDateimeError
+        ):
+            check_valid_zoned_datetime(result)
+    return result
 
 
 @composite
-def versions(
+def _draw_text(
     _draw: DrawFn,
+    alphabet: MaybeSearchStrategy[str],
     /,
     *,
-    min_version: MaybeSearchStrategy[Version | None] = None,
-    max_version: MaybeSearchStrategy[Version | None] = None,
-) -> Version:
-    """Strategy for generating `Version`s."""
-    from semver import Version
-
+    min_size: MaybeSearchStrategy[int] = 0,
+    max_size: MaybeSearchStrategy[int | None] = None,
+    disallow_na: MaybeSearchStrategy[bool] = False,
+) -> str:
+    """Draw from a text-generating strategy."""
     draw = lift_draw(_draw)
-    min_version_, max_version_ = (draw(mv) for mv in (min_version, max_version))
-    if isinstance(min_version_, Version) and isinstance(max_version_, Version):
-        if min_version_ > max_version_:
-            msg = f"{min_version_=}, {max_version_=}"
-            raise InvalidArgument(msg)
-        major = draw(integers(min_version_.major, max_version_.major))
-        minor, patch = draw(lists_fixed_length(integers(min_value=0), 2))
-        version = Version(major=major, minor=minor, patch=patch)
-        _ = assume(min_version_ <= version <= max_version_)
-        return version
-    if isinstance(min_version_, Version) and (max_version_ is None):
-        major = draw(integers(min_value=min_version_.major))
-        if major > min_version_.major:
-            minor, patch = draw(lists_fixed_length(integers(min_value=0), 2))
-        else:
-            minor = draw(integers(min_version_.minor))
-            if minor > min_version_.minor:
-                patch = draw(integers(min_value=0))  # pragma: no cover
-            else:
-                patch = draw(integers(min_value=min_version_.patch))
-    elif (min_version_ is None) and isinstance(max_version_, Version):
-        major = draw(integers(0, max_version_.major))
-        if major < max_version_.major:
-            minor, patch = draw(lists_fixed_length(integers(min_value=0), 2))
-        else:
-            minor = draw(integers(0, max_version_.minor))
-            if minor < max_version_.minor:
-                patch = draw(integers(min_value=0))  # pragma: no cover
-            else:
-                patch = draw(integers(0, max_version_.patch))
-    elif (min_version_ is None) and (max_version_ is None):
-        major, minor, patch = draw(lists_fixed_length(integers(min_value=0), 3))
-    else:
-        msg = "Invalid case"  # pragma: no cover
-        raise RuntimeError(msg)  # pragma: no cover
-    return Version(major=major, minor=minor, patch=patch)
-
-
-@composite
-def _fixed_width_ints(
-    _draw: DrawFn,
-    dtype: Any,
-    /,
-    *,
-    min_value: MaybeSearchStrategy[int | None] = None,
-    max_value: MaybeSearchStrategy[int | None] = None,
-) -> int:
-    """Strategy for generating int64s."""
-    from numpy import iinfo
-
-    draw = lift_draw(_draw)
-    min_value_, max_value_ = (draw(mv) for mv in (min_value, max_value))
-    info = iinfo(dtype)
-    min_value_ = info.min if min_value_ is None else max(info.min, min_value_)
-    max_value = info.max if max_value_ is None else min(info.max, max_value_)
-    return draw(integers(min_value_, max_value))
-
-
-@composite
-def _merge_into_dict_of_indexes(
-    _draw: DrawFn,
-    indexes: MaybeSearchStrategy[Mapping[str, IndexA]] | None = None,
-    /,
-    **indexes_kwargs: MaybeSearchStrategy[IndexA],
-) -> dict[str, IndexA]:
-    """Merge positional & kwargs of indexes into a dictionary."""
-    draw = lift_draw(_draw)
-    if (indexes is None) and (len(indexes_kwargs) == 0):
-        return draw(dicts_of_indexes())
-    indexes_out: dict[str, IndexA] = {}
-    if indexes is not None:
-        indexes_out |= dict(draw(indexes))
-    indexes_out |= {k: draw(v) for k, v in indexes_kwargs.items()}
-    return indexes_out
+    drawn = draw(text(alphabet, min_size=draw(min_size), max_size=draw(max_size)))
+    if draw(disallow_na):
+        _ = assume(drawn != "NA")
+    return drawn
 
 
 __all__ = [
     "MaybeSearchStrategy",
     "Shape",
-    "aiosqlite_engines",
     "assume_does_not_raise",
     "bool_arrays",
-    "bool_data_arrays",
-    "concatenated_arrays",
-    "dates_pd",
-    "datetimes_pd",
-    "datetimes_utc",
-    "dicts_of_indexes",
-    "draw_text",
+    "durations",
     "float_arrays",
-    "float_data_arrays",
     "floats_extra",
     "git_repos",
     "hashables",
-    "indexes",
     "int32s",
     "int64s",
     "int_arrays",
-    "int_data_arrays",
-    "int_indexes",
+    "lift_data",
     "lift_draw",
     "lists_fixed_length",
-    "longs",
     "months",
-    "namespace_mixins",
-    "redis_clients",
-    "redis_time_series",
+    "random_states",
+    "sets_fixed_length",
     "setup_hypothesis_profiles",
     "slices",
-    "sqlite_engines",
+    "sqlalchemy_engines",
     "str_arrays",
-    "str_data_arrays",
-    "str_indexes",
     "temp_dirs",
     "temp_paths",
     "text_ascii",
     "text_clean",
+    "text_digits",
     "text_printable",
     "timedeltas_2w",
-    "timestamps",
     "uint32s",
     "uint64s",
-    "versions",
+    "yield_test_redis",
+    "zoned_datetimes",
 ]
