@@ -1,6 +1,7 @@
 from __future__ import annotations
 
-import re
+import datetime as dt
+import reprlib
 from collections import Counter
 from collections.abc import (
     Callable,
@@ -12,15 +13,20 @@ from collections.abc import (
     Sized,
 )
 from collections.abc import Set as AbstractSet
+from contextlib import suppress
 from dataclasses import dataclass
-from functools import partial
-from inspect import signature
-from itertools import accumulate, chain, groupby, islice, product
+from enum import Enum
+from functools import cmp_to_key, partial
+from itertools import accumulate, chain, groupby, islice, pairwise, product
+from math import isnan
+from types import NoneType
 from typing import (
+    TYPE_CHECKING,
     Any,
     Generic,
     Literal,
     Never,
+    Self,
     TypeGuard,
     TypeVar,
     assert_never,
@@ -30,6 +36,8 @@ from typing import (
 
 from typing_extensions import override
 
+from utilities.errors import ImpossibleCaseError
+from utilities.functions import ensure_not_none
 from utilities.math import (
     _CheckIntegerEqualError,
     _CheckIntegerEqualOrApproxError,
@@ -37,8 +45,13 @@ from utilities.math import (
     _CheckIntegerMinError,
     check_integer,
 )
+from utilities.sentinel import sentinel
 from utilities.text import ensure_str
 from utilities.types import ensure_hashable
+from utilities.zoneinfo import UTC
+
+if TYPE_CHECKING:
+    from utilities.sentinel import Sentinel
 
 _K = TypeVar("_K")
 _T = TypeVar("_T")
@@ -50,7 +63,21 @@ _T3 = TypeVar("_T3")
 _T4 = TypeVar("_T4")
 _T5 = TypeVar("_T5")
 _THashable = TypeVar("_THashable", bound=Hashable)
+_UHashable = TypeVar("_UHashable", bound=Hashable)
 MaybeIterable = _T | Iterable[_T]
+IterableHashable = tuple[_THashable, ...] | frozenset[_THashable]
+MaybeIterableHashable = _THashable | IterableHashable[_THashable]
+
+
+def always_iterable(obj: MaybeIterable[_T], /) -> Iterable[_T]:
+    """Typed version of `always_iterable`."""
+    obj = cast(Any, obj)
+    if isinstance(obj, str | bytes):
+        return cast(list[_T], [obj])
+    try:
+        return iter(cast(Iterable[_T], obj))
+    except TypeError:
+        return cast(list[_T], [obj])
 
 
 def check_bijection(mapping: Mapping[Any, Hashable], /) -> None:
@@ -61,16 +88,14 @@ def check_bijection(mapping: Mapping[Any, Hashable], /) -> None:
         raise CheckBijectionError(mapping=mapping, counts=error.counts) from None
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckBijectionError(Exception, Generic[_THashable]):
     mapping: Mapping[Any, _THashable]
     counts: Mapping[_THashable, int]
 
     @override
     def __str__(self) -> str:
-        return "Mapping {} must be a bijection; got duplicates {}.".format(
-            self.mapping, ", ".join(f"({k}, n={v})" for k, v in self.counts.items())
-        )
+        return f"Mapping {reprlib.repr(self.mapping)} must be a bijection; got duplicates {reprlib.repr(self.counts)}"
 
 
 def check_duplicates(iterable: Iterable[Hashable], /) -> None:
@@ -80,16 +105,14 @@ def check_duplicates(iterable: Iterable[Hashable], /) -> None:
         raise CheckDuplicatesError(iterable=iterable, counts=counts)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckDuplicatesError(Exception, Generic[_THashable]):
     iterable: Iterable[_THashable]
     counts: Mapping[_THashable, int]
 
     @override
     def __str__(self) -> str:
-        return (
-            f"Iterable {self.iterable} must not contain duplicates; got {self.counts}."
-        )
+        return f"Iterable {reprlib.repr(self.iterable)} must not contain duplicates; got {reprlib.repr(self.counts)}"
 
 
 def check_iterables_equal(left: Iterable[Any], right: Iterable[Any], /) -> None:
@@ -122,7 +145,7 @@ def check_iterables_equal(left: Iterable[Any], right: Iterable[Any], /) -> None:
 _CheckIterablesEqualState = Literal["left_longer", "right_longer"]
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckIterablesEqualError(Exception, Generic[_T]):
     left: list[_T]
     right: list[_T]
@@ -138,12 +161,12 @@ class CheckIterablesEqualError(Exception, Generic[_T]):
                 desc = f"{first} and {second}"
             case _ as never:  # pragma: no cover
                 assert_never(cast(Never, never))
-        return f"Iterables {self.left} and {self.right} must be equal; {desc}."
+        return f"Iterables {reprlib.repr(self.left)} and {reprlib.repr(self.right)} must be equal; {desc}"
 
     def _yield_parts(self) -> Iterator[str]:
         if len(self.errors) >= 1:
-            error_descs = (f"({lv}, {rv}, i={i})" for i, lv, rv in self.errors)
-            yield "differing items were {}".format(", ".join(error_descs))
+            errors = [(f"{i=}", lv, rv) for i, lv, rv in self.errors]
+            yield f"differing items were {reprlib.repr(errors)}"
         match self.state:
             case "left_longer":
                 yield "left was longer"
@@ -180,21 +203,21 @@ def check_length(
         raise _CheckLengthMaxError(obj=obj, max_=error.max_) from None
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckLengthError(Exception):
     obj: Sized
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckLengthEqualError(CheckLengthError):
     equal: int
 
     @override
     def __str__(self) -> str:
-        return f"Object {self.obj} must have length {self.equal}; got {len(self.obj)}."
+        return f"Object {reprlib.repr(self.obj)} must have length {self.equal}; got {len(self.obj)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckLengthEqualOrApproxError(CheckLengthError):
     equal_or_approx: int | tuple[int, float]
 
@@ -205,25 +228,25 @@ class _CheckLengthEqualOrApproxError(CheckLengthError):
                 desc = f"approximate length {target} (error {error:%})"
             case target:
                 desc = f"length {target}"
-        return f"Object {self.obj} must have {desc}; got {len(self.obj)}."
+        return f"Object {reprlib.repr(self.obj)} must have {desc}; got {len(self.obj)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckLengthMinError(CheckLengthError):
     min_: int
 
     @override
     def __str__(self) -> str:
-        return f"Object {self.obj} must have minimum length {self.min_}; got {len(self.obj)}."
+        return f"Object {reprlib.repr(self.obj)} must have minimum length {self.min_}; got {len(self.obj)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckLengthMaxError(CheckLengthError):
     max_: int
 
     @override
     def __str__(self) -> str:
-        return f"Object {self.obj} must have maximum length {self.max_}; got {len(self.obj)}."
+        return f"Object {reprlib.repr(self.obj)} must have maximum length {self.max_}; got {len(self.obj)}"
 
 
 def check_lengths_equal(left: Sized, right: Sized, /) -> None:
@@ -232,14 +255,14 @@ def check_lengths_equal(left: Sized, right: Sized, /) -> None:
         raise CheckLengthsEqualError(left=left, right=right)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckLengthsEqualError(Exception):
     left: Sized
     right: Sized
 
     @override
     def __str__(self) -> str:
-        return f"Sized objects {self.left} and {self.right} must have the same length; got {len(self.left)} and {len(self.right)}."
+        return f"Sized objects {reprlib.repr(self.left)} and {reprlib.repr(self.right)} must have the same length; got {len(self.left)} and {len(self.right)}"
 
 
 def check_mappings_equal(left: Mapping[Any, Any], right: Mapping[Any, Any], /) -> None:
@@ -266,7 +289,7 @@ def check_mappings_equal(left: Mapping[Any, Any], right: Mapping[Any, Any], /) -
         )
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckMappingsEqualError(Exception, Generic[_K, _V]):
     left: Mapping[_K, _V]
     right: Mapping[_K, _V]
@@ -285,16 +308,16 @@ class CheckMappingsEqualError(Exception, Generic[_K, _V]):
                 desc = f"{first}, {second} and {third}"
             case _ as never:  # pragma: no cover
                 assert_never(cast(Never, never))
-        return f"Mappings {self.left} and {self.right} must be equal; {desc}."
+        return f"Mappings {reprlib.repr(self.left)} and {reprlib.repr(self.right)} must be equal; {desc}"
 
     def _yield_parts(self) -> Iterator[str]:
         if len(self.left_extra) >= 1:
-            yield f"left had extra keys {self.left_extra}"
+            yield f"left had extra keys {reprlib.repr(self.left_extra)}"
         if len(self.right_extra) >= 1:
-            yield f"right had extra keys {self.right_extra}"
+            yield f"right had extra keys {reprlib.repr(self.right_extra)}"
         if len(self.errors) >= 1:
-            error_descs = (f"({lv}, {rv}, k={k})" for k, lv, rv in self.errors)
-            yield "differing values were {}".format(", ".join(error_descs))
+            errors = [(f"{k=}", lv, rv) for k, lv, rv in self.errors]
+            yield f"differing values were {reprlib.repr(errors)}"
 
 
 def check_sets_equal(left: Iterable[Any], right: Iterable[Any], /) -> None:
@@ -312,7 +335,7 @@ def check_sets_equal(left: Iterable[Any], right: Iterable[Any], /) -> None:
         )
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckSetsEqualError(Exception, Generic[_T]):
     left: AbstractSet[_T]
     right: AbstractSet[_T]
@@ -328,13 +351,13 @@ class CheckSetsEqualError(Exception, Generic[_T]):
                 desc = f"{first} and {second}"
             case _ as never:  # pragma: no cover
                 assert_never(cast(Never, never))
-        return f"Sets {self.left} and {self.right} must be equal; {desc}."
+        return f"Sets {reprlib.repr(self.left)} and {reprlib.repr(self.right)} must be equal; {desc}"
 
     def _yield_parts(self) -> Iterator[str]:
         if len(self.left_extra) >= 1:
-            yield f"left had extra items {self.left_extra}"
+            yield f"left had extra items {reprlib.repr(self.left_extra)}"
         if len(self.right_extra) >= 1:
-            yield f"right had extra items {self.right_extra}"
+            yield f"right had extra items {reprlib.repr(self.right_extra)}"
 
 
 def check_submapping(left: Mapping[Any, Any], right: Mapping[Any, Any], /) -> None:
@@ -355,7 +378,7 @@ def check_submapping(left: Mapping[Any, Any], right: Mapping[Any, Any], /) -> No
         raise CheckSubMappingError(left=left, right=right, extra=extra, errors=errors)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckSubMappingError(Exception, Generic[_K, _V]):
     left: Mapping[_K, _V]
     right: Mapping[_K, _V]
@@ -371,14 +394,14 @@ class CheckSubMappingError(Exception, Generic[_K, _V]):
                 desc = f"{first} and {second}"
             case _ as never:  # pragma: no cover
                 assert_never(cast(Never, never))
-        return f"Mapping {self.left} must be a submapping of {self.right}; {desc}."
+        return f"Mapping {reprlib.repr(self.left)} must be a submapping of {reprlib.repr(self.right)}; {desc}"
 
     def _yield_parts(self) -> Iterator[str]:
         if len(self.extra) >= 1:
-            yield f"left had extra keys {self.extra}"
+            yield f"left had extra keys {reprlib.repr(self.extra)}"
         if len(self.errors) >= 1:
-            error_descs = (f"({lv}, {rv}, k={k})" for k, lv, rv in self.errors)
-            yield "differing values were {}".format(", ".join(error_descs))
+            errors = [(f"{k=}", lv, rv) for k, lv, rv in self.errors]
+            yield f"differing values were {reprlib.repr(errors)}"
 
 
 def check_subset(left: Iterable[Any], right: Iterable[Any], /) -> None:
@@ -390,7 +413,7 @@ def check_subset(left: Iterable[Any], right: Iterable[Any], /) -> None:
         raise CheckSubSetError(left=left_as_set, right=right_as_set, extra=extra)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckSubSetError(Exception, Generic[_T]):
     left: AbstractSet[_T]
     right: AbstractSet[_T]
@@ -398,7 +421,7 @@ class CheckSubSetError(Exception, Generic[_T]):
 
     @override
     def __str__(self) -> str:
-        return f"Set {self.left} must be a subset of {self.right}; left had extra items {self.extra}."
+        return f"Set {reprlib.repr(self.left)} must be a subset of {reprlib.repr(self.right)}; left had extra items {reprlib.repr(self.extra)}"
 
 
 def check_supermapping(left: Mapping[Any, Any], right: Mapping[Any, Any], /) -> None:
@@ -419,7 +442,7 @@ def check_supermapping(left: Mapping[Any, Any], right: Mapping[Any, Any], /) ->
         raise CheckSuperMappingError(left=left, right=right, extra=extra, errors=errors)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckSuperMappingError(Exception, Generic[_K, _V]):
     left: Mapping[_K, _V]
     right: Mapping[_K, _V]
@@ -435,14 +458,14 @@ class CheckSuperMappingError(Exception, Generic[_K, _V]):
                 desc = f"{first} and {second}"
             case _ as never:  # pragma: no cover
                 assert_never(cast(Never, never))
-        return f"Mapping {self.left} must be a supermapping of {self.right}; {desc}."
+        return f"Mapping {reprlib.repr(self.left)} must be a supermapping of {reprlib.repr(self.right)}; {desc}"
 
     def _yield_parts(self) -> Iterator[str]:
         if len(self.extra) >= 1:
-            yield f"right had extra keys {self.extra}"
+            yield f"right had extra keys {reprlib.repr(self.extra)}"
         if len(self.errors) >= 1:
-            error_descs = (f"({lv}, {rv}, k={k})" for k, lv, rv in self.errors)
-            yield "differing values were {}".format(", ".join(error_descs))
+            errors = [(f"{k=}", lv, rv) for k, lv, rv in self.errors]
+            yield f"differing values were {reprlib.repr(errors)}"
 
 
 def check_superset(left: Iterable[Any], right: Iterable[Any], /) -> None:
@@ -454,7 +477,7 @@ def check_superset(left: Iterable[Any], right: Iterable[Any], /) -> None:
         raise CheckSuperSetError(left=left_as_set, right=right_as_set, extra=extra)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckSuperSetError(Exception, Generic[_T]):
     left: AbstractSet[_T]
     right: AbstractSet[_T]
@@ -462,7 +485,7 @@ class CheckSuperSetError(Exception, Generic[_T]):
 
     @override
     def __str__(self) -> str:
-        return f"Set {self.left} must be a superset of {self.right}; right had extra items {self.extra}."
+        return f"Set {reprlib.repr(self.left)} must be a superset of {reprlib.repr(self.right)}; right had extra items {reprlib.repr(self.extra)}."
 
 
 def chunked(iterable: Iterable[_T], n: int, /) -> Iterator[Sequence[_T]]:
@@ -470,29 +493,54 @@ def chunked(iterable: Iterable[_T], n: int, /) -> Iterator[Sequence[_T]]:
     return iter(partial(take, n, iter(iterable)), [])
 
 
-_DESCRIBE_MAPPING_REGEX = re.compile(r"^_")
+class Collection(frozenset[_THashable]):
+    """A collection of hashable, sortable items."""
 
+    def __new__(cls, *item_or_items: MaybeIterable[_THashable]) -> Self:
+        items = list(chain(*map(always_iterable, item_or_items)))
+        cls.check_items(items)
+        return super().__new__(cls, items)
 
-def describe_mapping(
-    mapping: Mapping[str, Any],
-    /,
-    *,
-    func: Callable[..., Any] | None = None,
-    include_underscore: bool = False,
-    include_none: bool = False,
-) -> str:
-    """Describe a mapping."""
-    if not include_underscore:
-        mapping = {
-            k: v for k, v in mapping.items() if not _DESCRIBE_MAPPING_REGEX.search(k)
-        }
-    if not include_none:
-        mapping = {k: v for k, v in mapping.items() if v is not None}
-    if func is not None:
-        params = set(signature(func).parameters)
-        mapping = {k: v for k, v in mapping.items() if k in params}
-    items = (f"{k}={v}" for k, v in mapping.items())
-    return ", ".join(items)
+    def __init__(self, *item_or_items: MaybeIterable[_THashable]) -> None:
+        super().__init__()
+        _ = item_or_items
+
+    @override
+    def __and__(self, other: MaybeIterable[_THashable], /) -> Self:
+        if isinstance(other, type(self)):
+            return type(self)(super().__and__(other))
+        return self.__and__(type(self)(other))
+
+    @override
+    def __or__(self, other: MaybeIterable[_THashable], /) -> Self:  # pyright: ignore[reportIncompatibleMethodOverride]
+        if isinstance(other, type(self)):
+            return type(self)(super().__or__(other))
+        return self.__or__(type(self)(other))
+
+    @override
+    def __sub__(self, other: MaybeIterable[_THashable], /) -> Self:
+        if isinstance(other, type(self)):
+            return type(self)(super().__sub__(other))
+        return self.__sub__(type(self)(other))
+
+    @classmethod
+    def check_items(cls, items: Iterable[_THashable], /) -> None:
+        _ = items
+
+    def filter(self, func: Callable[[_THashable], bool], /) -> Self:
+        return type(self)(filter(func, self))
+
+    def map(
+        self, func: Callable[[_THashable], _UHashable], /
+    ) -> Collection[_UHashable]:
+        values = cast(Any, map(func, self))
+        return cast(Any, type(self)(values))
+
+    def partition(self, func: Callable[[_THashable], bool], /) -> tuple[Self, Self]:
+        from more_itertools import partition
+
+        is_false, is_true = partition(func, self)
+        return type(self)(is_false), type(self)(is_true)
 
 
 def ensure_hashables(
@@ -511,13 +559,13 @@ def ensure_iterable(obj: Any, /) -> Iterable[Any]:
     raise EnsureIterableError(obj=obj)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class EnsureIterableError(Exception):
     obj: Any
 
     @override
     def __str__(self) -> str:
-        return f"Object {self.obj} must be iterable."
+        return f"Object {reprlib.repr(self.obj)} must be iterable"
 
 
 def ensure_iterable_not_str(obj: Any, /) -> Iterable[Any]:
@@ -527,13 +575,13 @@ def ensure_iterable_not_str(obj: Any, /) -> Iterable[Any]:
     raise EnsureIterableNotStrError(obj=obj)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class EnsureIterableNotStrError(Exception):
     obj: Any
 
     @override
     def __str__(self) -> str:
-        return f"Object {self.obj} must be iterable, but not a string."
+        return f"Object {reprlib.repr(self.obj)} must be iterable, but not a string"
 
 
 def expanding_window(iterable: Iterable[_T], /) -> islice[list[_T]]:
@@ -545,6 +593,52 @@ def expanding_window(iterable: Iterable[_T], /) -> islice[list[_T]]:
     return islice(accumulate(iterable, func=func, initial=[]), 1, None)
 
 
+def hashable_to_iterable(obj: _THashable | None, /) -> tuple[_THashable, ...] | None:
+    """Lift a hashable singleton to an iterable of hashables."""
+    return None if obj is None else (obj,)
+
+
+@overload
+def filter_include_and_exclude(
+    iterable: Iterable[_T],
+    /,
+    *,
+    include: MaybeIterable[_U] | None = None,
+    exclude: MaybeIterable[_U] | None = None,
+    key: Callable[[_T], _U],
+) -> Iterable[_T]: ...
+@overload
+def filter_include_and_exclude(
+    iterable: Iterable[_T],
+    /,
+    *,
+    include: MaybeIterable[_T] | None = None,
+    exclude: MaybeIterable[_T] | None = None,
+    key: Callable[[_T], Any] | None = None,
+) -> Iterable[_T]: ...
+def filter_include_and_exclude(
+    iterable: Iterable[_T],
+    /,
+    *,
+    include: MaybeIterable[_U] | None = None,
+    exclude: MaybeIterable[_U] | None = None,
+    key: Callable[[_T], _U] | None = None,
+) -> Iterable[_T]:
+    """Filter an iterable based on an inclusion/exclusion pair."""
+    include, exclude = resolve_include_and_exclude(include=include, exclude=exclude)
+    if include is not None:
+        if key is None:
+            iterable = (x for x in iterable if x in include)
+        else:
+            iterable = (x for x in iterable if key(x) in include)
+    if exclude is not None:
+        if key is None:
+            iterable = (x for x in iterable if x not in exclude)
+        else:
+            iterable = (x for x in iterable if key(x) not in exclude)
+    return iterable
+
+
 @overload
 def groupby_lists(
     iterable: Iterable[_T], /, *, key: None = None
@@ -574,6 +668,11 @@ def is_iterable(obj: Any, /) -> TypeGuard[Iterable[Any]]:
     return True
 
 
+def is_iterable_not_enum(obj: Any, /) -> TypeGuard[Iterable[Any]]:
+    """Check if an object is iterable, but not an Enum."""
+    return is_iterable(obj) and not (isinstance(obj, type) and issubclass(obj, Enum))
+
+
 def is_iterable_not_str(obj: Any, /) -> TypeGuard[Iterable[Any]]:
     """Check if an object is iterable, but not a string."""
     return is_iterable(obj) and not isinstance(obj, str)
@@ -593,26 +692,26 @@ def one(iterable: Iterable[_T], /) -> _T:
     raise OneNonUniqueError(iterable=iterable, first=first, second=second)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class OneError(Exception, Generic[_T]):
     iterable: Iterable[_T]
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class OneEmptyError(OneError[_T]):
     @override
     def __str__(self) -> str:
-        return f"Iterable {self.iterable} must not be empty."
+        return f"Iterable {reprlib.repr(self.iterable)} must not be empty"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class OneNonUniqueError(OneError[_T]):
     first: _T
     second: _T
 
     @override
     def __str__(self) -> str:
-        return f"Iterable {self.iterable} must contain exactly one item; got {self.first}, {self.second} and perhaps more."
+        return f"Iterable {reprlib.repr(self.iterable)} must contain exactly one item; got {self.first}, {self.second} and perhaps more"
 
 
 def one_str(
@@ -645,44 +744,47 @@ def one_str(
         raise _OneStrCaseInsensitiveEmptyError(iterable=iterable, text=text) from None
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class OneStrError(Exception):
     iterable: Iterable[str]
     text: str
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _OneStrDuplicatesError(OneStrError):
     counts: Mapping[str, int]
 
     @override
     def __str__(self) -> str:
-        return (
-            f"Iterable {self.iterable} must not contain duplicates; got {self.counts}."
-        )
+        return f"Iterable {reprlib.repr(self.iterable)} must not contain duplicates; got {reprlib.repr(self.counts)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _OneStrCaseSensitiveEmptyError(OneStrError):
     @override
     def __str__(self) -> str:
-        return f"Iterable {self.iterable} does not contain {self.text!r}."
+        return f"Iterable {reprlib.repr(self.iterable)} does not contain {reprlib.repr(self.text)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _OneStrCaseInsensitiveBijectionError(OneStrError):
     counts: Mapping[str, int]
 
     @override
     def __str__(self) -> str:
-        return f"Iterable {self.iterable} must not contain duplicates (case insensitive); got {self.counts}."
+        return f"Iterable {reprlib.repr(self.iterable)} must not contain duplicates (case insensitive); got {reprlib.repr(self.counts)}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _OneStrCaseInsensitiveEmptyError(OneStrError):
     @override
     def __str__(self) -> str:
-        return f"Iterable {self.iterable} does not contain {self.text!r} (case insensitive)."
+        return f"Iterable {reprlib.repr(self.iterable)} does not contain {reprlib.repr(self.text)} (case insensitive)"
+
+
+def pairwise_tail(iterable: Iterable[_T], /) -> Iterator[tuple[_T, _T | Sentinel]]:
+    """Return pairwise elements, with the last paired with the sentinel."""
+    return pairwise(chain(iterable, [sentinel]))
 
 
 def product_dicts(mapping: Mapping[_K, Iterable[_V]], /) -> Iterator[Mapping[_K, _V]]:
@@ -692,6 +794,140 @@ def product_dicts(mapping: Mapping[_K, Iterable[_V]], /) -> Iterator[Mapping[_K,
         yield cast(Mapping[_K, _V], dict(zip(keys, values, strict=True)))
 
 
+def resolve_include_and_exclude(
+    *,
+    include: MaybeIterable[_T] | None = None,
+    exclude: MaybeIterable[_T] | None = None,
+) -> tuple[set[_T] | None, set[_T] | None]:
+    """Resolve an inclusion/exclusion pair."""
+    include_use = include if include is None else set(always_iterable(include))
+    exclude_use = exclude if exclude is None else set(always_iterable(exclude))
+    if (
+        (include_use is not None)
+        and (exclude_use is not None)
+        and (len(include_use & exclude_use) >= 1)
+    ):
+        raise ResolveIncludeAndExcludeError(include=include_use, exclude=exclude_use)
+    return include_use, exclude_use
+
+
+@dataclass(kw_only=True, slots=True)
+class ResolveIncludeAndExcludeError(Exception, Generic[_T]):
+    include: Iterable[_T]
+    exclude: Iterable[_T]
+
+    @override
+    def __str__(self) -> str:
+        include = list(self.include)
+        exclude = list(self.exclude)
+        overlap = set(include) & set(exclude)
+        return f"Iterables {reprlib.repr(include)} and {reprlib.repr(exclude)} must not overlap; got {reprlib.repr(overlap)}"
+
+
+def sort_iterable(iterable: Iterable[_T], /) -> list[_T]:
+    """Sort an iterable across types."""
+    return sorted(iterable, key=cmp_to_key(_sort_iterable_cmp))
+
+
+def _sort_iterable_cmp(x: Any, y: Any, /) -> Literal[-1, 0, 1]:
+    """Compare two quantities."""
+    if type(x) is not type(y):
+        x_qualname = type(x).__qualname__
+        y_qualname = type(y).__qualname__
+        if x_qualname < y_qualname:
+            return -1
+        if x_qualname > y_qualname:
+            return 1
+        raise ImpossibleCaseError(  # pragma: no cover
+            case=[f"{x_qualname=}", f"{y_qualname=}"]
+        )
+
+    # singletons
+    if x is None:
+        y = cast(NoneType, y)
+        return 0
+    if isinstance(x, dt.datetime):
+        y = cast(dt.datetime, y)
+        return _sort_iterable_cmp_datetimes(x, y)
+    if isinstance(x, float):
+        y = cast(float, y)
+        return _sort_iterable_cmp_floats(x, y)
+    if isinstance(x, str):  # else Sequence
+        y = cast(str, y)
+        return cast(Literal[-1, 0, 1], (x > y) - (x < y))
+
+    # collections
+    if isinstance(x, Sized):
+        y = cast(Sized, y)
+        if (result := _sort_iterable_cmp(len(x), len(y))) != 0:
+            return result
+    if isinstance(x, Mapping):
+        y = cast(Mapping[Any, Any], y)
+        return _sort_iterable_cmp(x.items(), y.items())
+    if isinstance(x, AbstractSet):
+        y = cast(AbstractSet[Any], y)
+        return _sort_iterable_cmp(sort_iterable(x), sort_iterable(y))
+    if isinstance(x, Sequence):
+        y = cast(Sequence[Any], y)
+        it: Iterable[Literal[-1, 0, 1]] = (
+            _sort_iterable_cmp(x_i, y_i) for x_i, y_i in zip(x, y, strict=True)
+        )
+        with suppress(StopIteration):
+            return next(r for r in it if r != 0)
+
+    try:
+        return cast(Literal[-1, 0, 1], (x > y) - (x < y))
+    except TypeError:
+        raise SortIterableError(x=x, y=y) from None
+
+
+@dataclass(kw_only=True, slots=True)
+class SortIterableError(Exception):
+    x: Any
+    y: Any
+
+    @override
+    def __str__(self) -> str:
+        return f"Unable to sort {reprlib.repr(self.x)} and {reprlib.repr(self.y)}"
+
+
+def _sort_iterable_cmp_datetimes(
+    x: dt.datetime, y: dt.datetime, /
+) -> Literal[-1, 0, 1]:
+    """Compare two datetimes."""
+    if (x.tzinfo is None) and (y.tzinfo is None):
+        return cast(Literal[-1, 0, 1], (x > y) - (x < y))
+    if (x.tzinfo is None) and (y.tzinfo is not None):
+        return -1
+    if (x.tzinfo is not None) and (y.tzinfo is None):
+        return 1
+    if (x.tzinfo is not None) and (y.tzinfo is not None):
+        x_utc = x.astimezone(tz=UTC)
+        y_utc = y.astimezone(tz=UTC)
+        result = cast(Literal[-1, 0, 1], (x_utc > y_utc) - (x_utc < y_utc))
+        if result != 0:
+            return result
+        x_time_zone = ensure_not_none(x.tzinfo.tzname(x))
+        y_time_zone = ensure_not_none(y.tzinfo.tzname(y))
+        return cast(
+            Literal[-1, 0, 1], (x_time_zone > y_time_zone) - (x_time_zone < y_time_zone)
+        )
+    raise ImpossibleCaseError(case=[f"{x=}", f"{y=}"])  # pragma: no cover
+
+
+def _sort_iterable_cmp_floats(x: float, y: float, /) -> Literal[-1, 0, 1]:
+    """Compare two floats."""
+    if isnan(x) and isnan(y):
+        return 0
+    if isnan(x) and (not isnan(y)):
+        return 1
+    if (not isnan(x)) and isnan(y):
+        return -1
+    if (not isnan(x)) and (not isnan(y)):
+        return cast(Literal[-1, 0, 1], (x > y) - (x < y))
+    raise ImpossibleCaseError(case=[f"{x=}", f"{y=}"])  # pragma: no cover
+
+
 def take(n: int, iterable: Iterable[_T], /) -> Sequence[_T]:
     """Return first n items of the iterable as a list."""
     return list(islice(iterable, n))
@@ -735,10 +971,16 @@ __all__ = [
     "CheckSuperSetError",
     "EnsureIterableError",
     "EnsureIterableNotStrError",
+    "IterableHashable",
     "MaybeIterable",
+    "MaybeIterableHashable",
     "OneEmptyError",
     "OneError",
     "OneNonUniqueError",
+    "OneStrError",
+    "ResolveIncludeAndExcludeError",
+    "SortIterableError",
+    "always_iterable",
     "check_bijection",
     "check_duplicates",
     "check_iterables_equal",
@@ -750,16 +992,22 @@ __all__ = [
     "check_supermapping",
     "check_superset",
     "chunked",
-    "describe_mapping",
     "ensure_hashables",
     "ensure_iterable",
     "ensure_iterable_not_str",
     "expanding_window",
+    "filter_include_and_exclude",
     "groupby_lists",
+    "hashable_to_iterable",
     "is_iterable",
+    "is_iterable_not_enum",
     "is_iterable_not_str",
     "one",
+    "one_str",
+    "pairwise_tail",
     "product_dicts",
+    "resolve_include_and_exclude",
+    "sort_iterable",
     "take",
     "transpose",
 ]
