@@ -1,30 +1,159 @@
 from __future__ import annotations
 
 from dataclasses import dataclass
-from math import isclose, isfinite, isnan, log10
-from typing import Annotated, Literal, overload
+from math import ceil, exp, floor, isclose, isfinite, isnan, log, log10, modf
+from typing import Literal, TypeAlias, assert_never, overload
 
 from typing_extensions import override
 
 from utilities.errors import ImpossibleCaseError
 
-MAX_LONG = 2**31 - 1
-MIN_LONG = -(2**31)
+MIN_INT8, MAX_INT8 = -(2 ** (8 - 1)), 2 ** (8 - 1) - 1
+MIN_INT16, MAX_INT16 = -(2 ** (16 - 1)), 2 ** (16 - 1) - 1
+MIN_INT32, MAX_INT32 = -(2 ** (32 - 1)), 2 ** (32 - 1) - 1
+MIN_INT64, MAX_INT64 = -(2 ** (64 - 1)), 2 ** (64 - 1) - 1
+MIN_UINT8, MAX_UINT8 = 0, 2**8 - 1
+MIN_UINT16, MAX_UINT16 = 0, 2**16 - 1
+MIN_UINT32, MAX_UINT32 = 0, 2**32 - 1
+MIN_UINT64, MAX_UINT64 = 0, 2**64 - 1
 
 # functions
 
 
-def is_equal(x: float, y: float, /) -> bool:
+@dataclass(kw_only=True, slots=True)
+class _EWMParameters:
+    """A set of EWM parameters."""
+
+    com: float
+    span: float
+    half_life: float
+    alpha: float
+
+
+def ewm_parameters(
+    *,
+    com: float | None = None,
+    span: float | None = None,
+    half_life: float | None = None,
+    alpha: float | None = None,
+) -> _EWMParameters:
+    """Compute a set of EWM parameters."""
+    if (com is not None) and (span is None) and (half_life is None) and (alpha is None):
+        if com <= 0:
+            raise _EWMParametersCOMError(com=com)
+        alpha = 1 / (1 + com)
+        return _EWMParameters(
+            com=com,
+            span=_ewm_parameters_alpha_to_span(alpha),
+            half_life=_ewm_parameters_alpha_to_half_life(alpha),
+            alpha=alpha,
+        )
+    if (com is None) and (span is not None) and (half_life is None) and (alpha is None):
+        if span <= 1:
+            raise _EWMParametersSpanError(span=span)
+        alpha = 2 / (span + 1)
+        return _EWMParameters(
+            com=_ewm_parameters_alpha_to_com(alpha),
+            span=span,
+            half_life=_ewm_parameters_alpha_to_half_life(alpha),
+            alpha=alpha,
+        )
+    if (com is None) and (span is None) and (half_life is not None) and (alpha is None):
+        if half_life <= 0:
+            raise _EWMParametersHalfLifeError(half_life=half_life)
+        alpha = 1 - exp(-log(2) / half_life)
+        return _EWMParameters(
+            com=_ewm_parameters_alpha_to_com(alpha),
+            span=_ewm_parameters_alpha_to_span(alpha),
+            half_life=half_life,
+            alpha=alpha,
+        )
+    if (com is None) and (span is None) and (half_life is None) and (alpha is not None):
+        if not (0 < alpha < 1):
+            raise _EWMParametersAlphaError(alpha=alpha)
+        return _EWMParameters(
+            com=_ewm_parameters_alpha_to_com(alpha),
+            span=_ewm_parameters_alpha_to_span(alpha),
+            half_life=_ewm_parameters_alpha_to_half_life(alpha),
+            alpha=alpha,
+        )
+    raise _EWMParametersArgumentsError(
+        com=com, span=span, half_life=half_life, alpha=alpha
+    )
+
+
+@dataclass(kw_only=True, slots=True)
+class EWMParametersError(Exception):
+    com: float | None = None
+    span: float | None = None
+    half_life: float | None = None
+    alpha: float | None = None
+
+
+@dataclass(kw_only=True, slots=True)
+class _EWMParametersCOMError(EWMParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Center of mass (γ) must be positive; got {self.com}"  # noqa: RUF001
+
+
+@dataclass(kw_only=True, slots=True)
+class _EWMParametersSpanError(EWMParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Span (θ) must be greater than 1; got {self.span}"
+
+
+class _EWMParametersHalfLifeError(EWMParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Half-life (λ) must be positive; got {self.half_life}"
+
+
+class _EWMParametersAlphaError(EWMParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Smoothing factor (α) must be between 0 and 1 (exclusive); got {self.alpha}"  # noqa: RUF001
+
+
+class _EWMParametersArgumentsError(EWMParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Exactly one of center of mass (γ), span (θ), half-life (λ) or smoothing factor (α) must be given; got γ={self.com}, θ={self.span}, λ={self.half_life} and α={self.alpha}"  # noqa: RUF001
+
+
+def _ewm_parameters_alpha_to_com(alpha: float, /) -> float:
+    return 1 / alpha - 1
+
+
+def _ewm_parameters_alpha_to_span(alpha: float, /) -> float:
+    return 2 / alpha - 1
+
+
+def _ewm_parameters_alpha_to_half_life(alpha: float, /) -> float:
+    return -log(2) / log(1 - alpha)
+
+
+def is_equal(
+    x: float, y: float, /, *, rel_tol: float | None = None, abs_tol: float | None = None
+) -> bool:
     """Check if x == y."""
-    return (x == y) or (isnan(x) and isnan(y))
+    if isinstance(x, int) and isinstance(y, int):
+        return x == y
+    return _is_close(x, y, rel_tol=rel_tol, abs_tol=abs_tol) or (isnan(x) and isnan(y))
 
 
 def is_equal_or_approx(
-    x: int | tuple[int, float], y: int | tuple[int, float], /
+    x: int | tuple[int, float],
+    y: int | tuple[int, float],
+    /,
+    *,
+    rel_tol: float | None = None,
+    abs_tol: float | None = None,
 ) -> bool:
     """Check if x == y, or approximately."""
     if isinstance(x, int) and isinstance(y, int):
-        return x == y
+        return is_equal(x, y, rel_tol=rel_tol, abs_tol=abs_tol)
     if isinstance(x, int) and isinstance(y, tuple):
         return isclose(x, y[0], rel_tol=y[1])
     if isinstance(x, tuple) and isinstance(y, int):
@@ -375,7 +504,7 @@ def _number_of_decimals_check_scale(frac: float, scale: int, /) -> bool:
     return isclose(scaled, round(scaled))
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class NumberOfDecimalsError(Exception):
     x: float
     max_decimals: int
@@ -395,50 +524,130 @@ def order_of_magnitude(x: float, /, *, round_: bool = False) -> float:
     return round(result) if round_ else result
 
 
-# annotations
+_RoundMode: TypeAlias = Literal[
+    "standard",
+    "floor",
+    "ceil",
+    "toward-zero",
+    "away-zero",
+    "standard-tie-floor",
+    "standard-tie-ceil",
+    "standard-tie-toward-zero",
+    "standard-tie-away-zero",
+]
 
 
-# int
-IntNeg = Annotated[int, is_negative]
-IntNonNeg = Annotated[int, is_non_negative]
-IntNonPos = Annotated[int, is_non_positive]
-IntNonZr = Annotated[int, is_non_zero]
-IntPos = Annotated[int, is_positive]
-IntZr = Annotated[int, is_zero]
+def round_(
+    x: float,
+    /,
+    *,
+    mode: _RoundMode = "standard",
+    rel_tol: float | None = None,
+    abs_tol: float | None = None,
+) -> int:
+    """Round a float to an integer."""
+    match mode:
+        case "standard":
+            return round(x)
+        case "floor":
+            return floor(x)
+        case "ceil":
+            return ceil(x)
+        case "toward-zero":
+            return int(x)
+        case "away-zero":
+            match sign(x):
+                case 1:
+                    return ceil(x)
+                case 0:
+                    return 0
+                case -1:
+                    return floor(x)
+                case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+                    assert_never(never)
+        case "standard-tie-floor":
+            return _round_tie_standard(x, "floor", rel_tol=rel_tol, abs_tol=abs_tol)
+        case "standard-tie-ceil":
+            return _round_tie_standard(x, "ceil", rel_tol=rel_tol, abs_tol=abs_tol)
+        case "standard-tie-toward-zero":
+            return _round_tie_standard(
+                x, "toward-zero", rel_tol=rel_tol, abs_tol=abs_tol
+            )
+        case "standard-tie-away-zero":
+            return _round_tie_standard(x, "away-zero", rel_tol=rel_tol, abs_tol=abs_tol)
+        case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+            assert_never(never)
 
-# float
-FloatFin = Annotated[float, is_finite]
-FloatFinInt = Annotated[float, is_finite_and_integral]
-FloatFinIntNan = Annotated[float, is_finite_and_integral_or_nan]
-FloatFinNeg = Annotated[float, is_finite_and_negative]
-FloatFinNegNan = Annotated[float, is_finite_and_negative_or_nan]
-FloatFinNonNeg = Annotated[float, is_finite_and_non_negative]
-FloatFinNonNegNan = Annotated[float, is_finite_and_non_negative_or_nan]
-FloatFinNonPos = Annotated[float, is_finite_and_non_positive]
-FloatFinNonPosNan = Annotated[float, is_finite_and_non_positive_or_nan]
-FloatFinNonZr = Annotated[float, is_finite_and_non_zero]
-FloatFinNonZrNan = Annotated[float, is_finite_and_non_zero_or_nan]
-FloatFinPos = Annotated[float, is_finite_and_positive]
-FloatFinPosNan = Annotated[float, is_finite_and_positive_or_nan]
-FloatFinNan = Annotated[float, is_finite_or_nan]
-FloatInt = Annotated[float, is_integral]
-FloatIntNan = Annotated[float, is_integral_or_nan]
-FloatNeg = Annotated[float, is_negative]
-FloatNegNan = Annotated[float, is_negative_or_nan]
-FloatNonNeg = Annotated[float, is_non_negative]
-FloatNonNegNan = Annotated[float, is_non_negative_or_nan]
-FloatNonPos = Annotated[float, is_non_positive]
-FloatNonPosNan = Annotated[float, is_non_positive_or_nan]
-FloatNonZr = Annotated[float, is_non_zero]
-FloatNonZrNan = Annotated[float, is_non_zero_or_nan]
-FloatPos = Annotated[float, is_positive]
-FloatPosNan = Annotated[float, is_positive_or_nan]
-FloatZr = Annotated[float, is_zero]
-FloatZrFinNonMic = Annotated[float, is_zero_or_finite_and_non_micro]
-FloatZrFinNonMicNan = Annotated[float, is_zero_or_finite_and_non_micro_or_nan]
-FloatZrNan = Annotated[float, is_zero_or_nan]
-FloatZrNonMic = Annotated[float, is_zero_or_non_micro]
-FloatZrNonMicNan = Annotated[float, is_zero_or_non_micro_or_nan]
+
+def _round_tie_standard(
+    x: float,
+    mode: _RoundMode,
+    /,
+    *,
+    rel_tol: float | None = None,
+    abs_tol: float | None = None,
+) -> int:
+    """Round a float to an integer using the standard method."""
+    frac, _ = modf(x)
+    if _is_close(abs(frac), 0.5, rel_tol=rel_tol, abs_tol=abs_tol):
+        mode_use: _RoundMode = mode
+    else:
+        mode_use: _RoundMode = "standard"
+    return round_(x, mode=mode_use)
+
+
+def round_to_float(
+    x: float,
+    y: float,
+    /,
+    *,
+    mode: _RoundMode = "standard",
+    rel_tol: float | None = None,
+    abs_tol: float | None = None,
+) -> float:
+    """Round a float to the nearest multiple of another float."""
+    return y * round_(x / y, mode=mode, rel_tol=rel_tol, abs_tol=abs_tol)
+
+
+def safe_round(
+    x: float, /, *, rel_tol: float | None = None, abs_tol: float | None = None
+) -> int:
+    """Safely round a float."""
+    if is_finite_and_integral(x, rel_tol=rel_tol, abs_tol=abs_tol):
+        return round(x)
+    raise SafeRoundError(x=x, rel_tol=rel_tol, abs_tol=abs_tol)
+
+
+@dataclass(kw_only=True, slots=True)
+class SafeRoundError(Exception):
+    x: float
+    rel_tol: float | None = None
+    abs_tol: float | None = None
+
+    @override
+    def __str__(self) -> str:
+        return f"Unable to safely round {self.x} (rel_tol={self.rel_tol}, abs_tol={self.abs_tol})"
+
+
+def sign(
+    x: float, /, *, rel_tol: float | None = None, abs_tol: float | None = None
+) -> Literal[-1, 0, 1]:
+    """Get the sign of an integer/float."""
+    match x:
+        case int():
+            if x > 0:
+                return 1
+            if x < 0:
+                return -1
+            return 0
+        case float():
+            if is_positive(x, rel_tol=rel_tol, abs_tol=abs_tol):
+                return 1
+            if is_negative(x, rel_tol=rel_tol, abs_tol=abs_tol):
+                return -1
+            return 0
+        case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+            assert_never(never)
 
 
 # checks
@@ -464,12 +673,12 @@ def check_integer(
         raise _CheckIntegerMaxError(n=n, max_=max)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckIntegerError(Exception):
     n: int
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckIntegerEqualError(CheckIntegerError):
     equal: int
 
@@ -478,7 +687,7 @@ class _CheckIntegerEqualError(CheckIntegerError):
         return f"Integer must be equal to {self.equal}; got {self.n}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckIntegerEqualOrApproxError(CheckIntegerError):
     equal_or_approx: int | tuple[int, float]
 
@@ -492,7 +701,7 @@ class _CheckIntegerEqualOrApproxError(CheckIntegerError):
         return f"Integer must be {desc}; got {self.n}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckIntegerMinError(CheckIntegerError):
     min_: int
 
@@ -501,7 +710,7 @@ class _CheckIntegerMinError(CheckIntegerError):
         return f"Integer must be at least {self.min_}; got {self.n}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckIntegerMaxError(CheckIntegerError):
     max_: int
 
@@ -511,48 +720,27 @@ class _CheckIntegerMaxError(CheckIntegerError):
 
 
 __all__ = [
-    "MAX_LONG",
-    "MIN_LONG",
+    "MAX_INT8",
+    "MAX_INT16",
+    "MAX_INT32",
+    "MAX_INT64",
+    "MAX_UINT8",
+    "MAX_UINT16",
+    "MAX_UINT32",
+    "MAX_UINT64",
+    "MIN_INT8",
+    "MIN_INT16",
+    "MIN_INT32",
+    "MIN_INT64",
+    "MIN_UINT8",
+    "MIN_UINT16",
+    "MIN_UINT32",
+    "MIN_UINT64",
     "CheckIntegerError",
-    "FloatFin",
-    "FloatFinInt",
-    "FloatFinIntNan",
-    "FloatFinNan",
-    "FloatFinNeg",
-    "FloatFinNegNan",
-    "FloatFinNonNeg",
-    "FloatFinNonNegNan",
-    "FloatFinNonPos",
-    "FloatFinNonPosNan",
-    "FloatFinNonZr",
-    "FloatFinNonZrNan",
-    "FloatFinPos",
-    "FloatFinPosNan",
-    "FloatInt",
-    "FloatIntNan",
-    "FloatNeg",
-    "FloatNegNan",
-    "FloatNonNeg",
-    "FloatNonNegNan",
-    "FloatNonPos",
-    "FloatNonPosNan",
-    "FloatNonZr",
-    "FloatNonZrNan",
-    "FloatPos",
-    "FloatPosNan",
-    "FloatZr",
-    "FloatZrFinNonMic",
-    "FloatZrFinNonMicNan",
-    "FloatZrNan",
-    "FloatZrNonMic",
-    "FloatZrNonMicNan",
-    "IntNeg",
-    "IntNonNeg",
-    "IntNonPos",
-    "IntNonZr",
-    "IntPos",
-    "IntZr",
+    "EWMParametersError",
+    "SafeRoundError",
     "check_integer",
+    "ewm_parameters",
     "is_at_least",
     "is_at_least_or_nan",
     "is_at_most",
@@ -597,4 +785,7 @@ __all__ = [
     "is_zero_or_non_micro_or_nan",
     "number_of_decimals",
     "order_of_magnitude",
+    "round_",
+    "round_to_float",
+    "safe_round",
 ]
