@@ -9,6 +9,15 @@ if TYPE_CHECKING:
     from types import ModuleType
 
 
+def is_installed(module: str, /) -> bool:
+    """Check if a module is installed."""
+    try:
+        _ = import_module(module)
+    except ModuleNotFoundError:
+        return False
+    return True
+
+
 def yield_modules(
     module: ModuleType, /, *, recursive: bool = False
 ) -> Iterator[ModuleType]:
@@ -56,17 +65,20 @@ def yield_module_subclasses(
     *,
     recursive: bool = False,
     predicate: Callable[[type[Any]], bool] | None = None,
+    is_module: bool = False,
 ) -> Iterator[Any]:
     """Yield all the module subclasses under a package.
 
     Optionally, recurse into sub-packages.
     """
+    name = module.__name__
 
     def predicate_use(obj: type[Any], /) -> bool:
         return (
             issubclass(obj, cls)
             and not issubclass(cls, obj)
             and ((predicate is None) or predicate(obj))
+            and ((is_module and (obj.__module__ == name)) or not is_module)
         )
 
     return yield_module_contents(
@@ -74,4 +86,9 @@ def yield_module_subclasses(
     )
 
 
-__all__ = ["yield_module_contents", "yield_module_subclasses", "yield_modules"]
+__all__ = [
+    "is_installed",
+    "yield_module_contents",
+    "yield_module_subclasses",
+    "yield_modules",
+]
