@@ -1,11 +1,11 @@
 from __future__ import annotations
 
-from collections.abc import Callable, Iterable, Iterator, Sequence
+import builtins
+from collections.abc import Callable, Hashable, Iterable, Iterator, Mapping, Sequence
 from dataclasses import dataclass
 from itertools import islice
 from textwrap import indent
 from typing import (
-    TYPE_CHECKING,
     Any,
     Generic,
     Literal,
@@ -16,69 +16,39 @@ from typing import (
     overload,
 )
 
-from more_itertools import always_iterable as _always_iterable
-from more_itertools import partition, split_into
+from more_itertools import bucket, partition, split_into
 from more_itertools import peekable as _peekable
 from typing_extensions import override
 
+from utilities.functions import get_class_name
 from utilities.sentinel import Sentinel, sentinel
 
-if TYPE_CHECKING:
-    from utilities.iterables import MaybeIterable
-
 _T = TypeVar("_T")
+_THashable = TypeVar("_THashable", bound=Hashable)
 _U = TypeVar("_U")
 
 
-def always_iterable(
-    obj: MaybeIterable[_T],
-    /,
-    *,
-    base_type: type[Any] | tuple[type[Any], ...] | None = (str, bytes),
-) -> Iterator[_T]:
-    """Typed version of `always_iterable`."""
-    return _always_iterable(obj, base_type=base_type)
-
-
 @overload
-def filter_include_and_exclude(
-    iterable: Iterable[_T],
-    /,
-    *,
-    include: MaybeIterable[_U] | None = None,
-    exclude: MaybeIterable[_U] | None = None,
-    key: Callable[[_T], _U],
-) -> Iterable[_T]: ...
+def bucket_mapping(
+    iterable: Iterable[_T], func: Callable[[_T], _THashable], /, *, list: Literal[True]
+) -> Mapping[_THashable, Sequence[_T]]: ...
 @overload
-def filter_include_and_exclude(
+def bucket_mapping(
+    iterable: Iterable[_T], func: Callable[[_T], _THashable], /, *, list: bool = False
+) -> Mapping[_THashable, Iterator[_T]]: ...
+def bucket_mapping(
     iterable: Iterable[_T],
+    func: Callable[[_T], _THashable],
     /,
     *,
-    include: MaybeIterable[_T] | None = None,
-    exclude: MaybeIterable[_T] | None = None,
-    key: Callable[[_T], Any] | None = None,
-) -> Iterable[_T]: ...
-def filter_include_and_exclude(
-    iterable: Iterable[_T],
-    /,
-    *,
-    include: MaybeIterable[_U] | None = None,
-    exclude: MaybeIterable[_U] | None = None,
-    key: Callable[[_T], _U] | None = None,
-) -> Iterable[_T]:
-    """Filter an iterable based on an inclusion/exclusion pair."""
-    include, exclude = resolve_include_and_exclude(include=include, exclude=exclude)
-    if include is not None:
-        if key is None:
-            iterable = (x for x in iterable if x in include)
-        else:
-            iterable = (x for x in iterable if key(x) in include)
-    if exclude is not None:
-        if key is None:
-            iterable = (x for x in iterable if x not in exclude)
-        else:
-            iterable = (x for x in iterable if key(x) not in exclude)
-    return iterable
+    list: bool = False,  # noqa: A002
+) -> Mapping[_THashable, Iterator[_T]] | Mapping[_THashable, Sequence[_T]]:
+    """Bucket the values of iterable into a mapping."""
+    b = bucket(iterable, func)
+    mapping = {key: b[key] for key in b}
+    if list:
+        return {key: builtins.list(value) for key, value in mapping.items()}
+    return mapping
 
 
 def partition_typeguard(
@@ -96,6 +66,11 @@ class peekable(_peekable, Generic[_T]):  # noqa: N801
     def __init__(self, iterable: Iterable[_T], /) -> None:
         super().__init__(iterable)
 
+    @override
+    def __iter__(self) -> Iterator[_T]:  # pyright: ignore[reportIncompatibleMethodOverride]
+        while bool(self):
+            yield next(self)
+
     @override
     def __next__(self) -> _T:
         return super().__next__()
@@ -119,37 +94,7 @@ class peekable(_peekable, Generic[_T]):  # noqa: N801
             yield next(self)
 
 
-def resolve_include_and_exclude(
-    *,
-    include: _T | Iterable[_T] | None = None,
-    exclude: _T | Iterable[_T] | None = None,
-) -> tuple[set[_T] | None, set[_T] | None]:
-    """Resolve an inclusion/exclusion pair."""
-    include_use = include if include is None else set(always_iterable(include))
-    exclude_use = exclude if exclude is None else set(always_iterable(exclude))
-    if (
-        (include_use is not None)
-        and (exclude_use is not None)
-        and (len(include_use & exclude_use) >= 1)
-    ):
-        raise ResolveIncludeAndExcludeError(include=include_use, exclude=exclude_use)
-    return include_use, exclude_use
-
-
-@dataclass(kw_only=True)
-class ResolveIncludeAndExcludeError(Exception, Generic[_T]):
-    include: Iterable[_T]
-    exclude: Iterable[_T]
-
-    @override
-    def __str__(self) -> str:
-        include = list(self.include)
-        exclude = list(self.exclude)
-        overlap = set(include) & set(exclude)
-        return f"Iterables {include} and {exclude} must not overlap; got {overlap}."
-
-
-@dataclass(frozen=True)
+@dataclass(kw_only=True, slots=True)
 class Split(Generic[_T]):
     """An iterable split into head/tail."""
 
@@ -158,13 +103,14 @@ class Split(Generic[_T]):
 
     @override
     def __repr__(self) -> str:
+        cls = get_class_name(self)
         spaces = 4 * " "
         head_first = indent("head=", spaces)
         head_rest = indent(repr(self.head), 2 * spaces)
         tail_first = indent("tail=", spaces)
         tail_rest = indent(repr(self.tail), 2 * spaces)
         joined = f"{head_first}\n{head_rest}\n{tail_first}\n{tail_rest}"
-        return f"{type(self).__name__}(\n{joined}\n)"
+        return f"{cls}(\n{joined}\n)"
 
 
 def yield_splits(
@@ -232,13 +178,4 @@ def _yield_splits3(
         yield cast(Split[Sequence[_T]], Split(head=list(head_win), tail=list(tail_win)))
 
 
-__all__ = [
-    "ResolveIncludeAndExcludeError",
-    "Split",
-    "always_iterable",
-    "filter_include_and_exclude",
-    "partition_typeguard",
-    "peekable",
-    "resolve_include_and_exclude",
-    "yield_splits",
-]
+__all__ = ["Split", "bucket_mapping", "partition_typeguard", "peekable", "yield_splits"]
