@@ -2,10 +2,10 @@ from __future__ import annotations
 
 import gzip
 from pickle import dump, load
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
-from utilities.atomicwrites import writer
-from utilities.types import PathLike
+if TYPE_CHECKING:
+    from utilities.types import PathLike
 
 
 def read_pickle(path: PathLike, /) -> Any:
@@ -16,6 +16,8 @@ def read_pickle(path: PathLike, /) -> Any:
 
 def write_pickle(obj: Any, path: PathLike, /, *, overwrite: bool = False) -> None:
     """Write an object to disk."""
+    from utilities.atomicwrites import writer
+
     with writer(path, overwrite=overwrite) as temp, gzip.open(temp, mode="wb") as gz:
         dump(obj, gz)
 
