@@ -1,26 +1,28 @@
 from __future__ import annotations
 
 import datetime as dt
+import reprlib
 from collections.abc import Callable, Iterable, Iterator, Mapping, Sequence
 from collections.abc import Set as AbstractSet
-from contextlib import contextmanager, suppress
-from dataclasses import dataclass
+from contextlib import suppress
+from dataclasses import _MISSING_TYPE, asdict, dataclass, fields
 from datetime import timezone
 from enum import Enum
-from functools import reduce
+from functools import partial, reduce
 from itertools import chain
 from typing import (
     TYPE_CHECKING,
     Any,
+    Generic,
     Literal,
     Never,
     TypeGuard,
     TypeVar,
     assert_never,
     cast,
-    get_type_hints,
     overload,
 )
+from zoneinfo import ZoneInfo
 
 from polars import (
     Boolean,
@@ -36,7 +38,9 @@ from polars import (
     Utf8,
     all_horizontal,
     col,
+    concat,
     lit,
+    struct,
     when,
 )
 from polars._typing import (
@@ -53,41 +57,110 @@ from polars.exceptions import ColumnNotFoundError, OutOfBoundsError
 from polars.testing import assert_frame_equal
 from typing_extensions import override
 
-from utilities.dataclasses import Dataclass, is_dataclass_class
-from utilities.errors import redirect_error
+from utilities.dataclasses import Dataclass, asdict_without_defaults, is_dataclass_class
 from utilities.iterables import (
     CheckIterablesEqualError,
     CheckMappingsEqualError,
+    CheckSubSetError,
     CheckSuperMappingError,
+    CheckSuperSetError,
     MaybeIterable,
     check_iterables_equal,
     check_mappings_equal,
+    check_subset,
     check_supermapping,
+    check_superset,
     is_iterable_not_str,
     one,
 )
-from utilities.math import CheckIntegerError, check_integer
+from utilities.math import (
+    CheckIntegerError,
+    _EWMParameters,
+    check_integer,
+    ewm_parameters,
+)
+from utilities.types import StrMapping, ensure_datetime
 from utilities.typing import (
     get_args,
+    get_type_hints,
     is_frozenset_type,
     is_list_type,
     is_literal_type,
     is_optional_type,
     is_set_type,
 )
-from utilities.zoneinfo import UTC, get_time_zone_name
+from utilities.zoneinfo import UTC, ensure_time_zone, get_time_zone_name
 
 if TYPE_CHECKING:
     from collections.abc import Callable, Iterable, Iterator, Sequence
     from collections.abc import Set as AbstractSet
-    from zoneinfo import ZoneInfo
-
-    from utilities.types import IterableStrs
 
 
+_T = TypeVar("_T")
+_TDataclass = TypeVar("_TDataclass", bound=Dataclass)
+DatetimeHongKong = Datetime(time_zone="Asia/Hong_Kong")
+DatetimeTokyo = Datetime(time_zone="Asia/Tokyo")
+DatetimeUSCentral = Datetime(time_zone="US/Central")
+DatetimeUSEastern = Datetime(time_zone="US/Eastern")
 DatetimeUTC = Datetime(time_zone="UTC")
 
 
+def append_dataclass(df: DataFrame, obj: Dataclass, /) -> DataFrame:
+    """Append a dataclass object to a DataFrame."""
+    non_null_fields = {k: v for k, v in asdict(obj).items() if v is not None}
+    try:
+        check_subset(non_null_fields, df.columns)
+    except CheckSubSetError as error:
+        raise AppendDataClassError(
+            left=error.left, right=error.right, extra=error.extra
+        ) from None
+    row_cols = set(df.columns) & set(non_null_fields)
+    row = dataclass_to_row(obj).select(*row_cols)
+    return concat([df, row], how="diagonal")
+
+
+@dataclass(kw_only=True, slots=True)
+class AppendDataClassError(Exception, Generic[_T]):
+    left: AbstractSet[_T]
+    right: AbstractSet[_T]
+    extra: AbstractSet[_T]
+
+    @override
+    def __str__(self) -> str:
+        return f"Dataclass fields {reprlib.repr(self.left)} must be a subset of DataFrame columns {reprlib.repr(self.right)}; dataclass had extra items {reprlib.repr(self.extra)}"
+
+
+def are_frames_equal(
+    left: DataFrame,
+    right: DataFrame,
+    /,
+    *,
+    check_row_order: bool = True,
+    check_column_order: bool = True,
+    check_dtypes: bool = True,
+    check_exact: bool = False,
+    rtol: float = 1e-5,
+    atol: float = 1e-8,
+    categorical_as_str: bool = False,
+) -> bool:
+    """Check if two DataFrames are equal."""
+    try:
+        assert_frame_equal(
+            left,
+            right,
+            check_row_order=check_row_order,
+            check_column_order=check_column_order,
+            check_dtypes=check_dtypes,
+            check_exact=check_exact,
+            rtol=rtol,
+            atol=atol,
+            categorical_as_str=categorical_as_str,
+        )
+    except AssertionError:
+        return False
+    return True
+
+
 @overload
 def ceil_datetime(column: Expr | str, every: Expr | str, /) -> Expr: ...
 @overload
@@ -110,7 +183,7 @@ def check_polars_dataframe(
     df: DataFrame,
     /,
     *,
-    columns: IterableStrs | None = None,
+    columns: Iterable[str] | None = None,
     dtypes: Iterable[PolarsDataType] | None = None,
     height: int | tuple[int, float] | None = None,
     min_height: int | None = None,
@@ -150,21 +223,22 @@ def check_polars_dataframe(
         _check_polars_dataframe_width(df, width)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class CheckPolarsDataFrameError(Exception):
     df: DataFrame
 
 
-def _check_polars_dataframe_columns(df: DataFrame, columns: IterableStrs, /) -> None:
+def _check_polars_dataframe_columns(df: DataFrame, columns: Iterable[str], /) -> None:
+    columns = list(columns)
     try:
         check_iterables_equal(df.columns, columns)
     except CheckIterablesEqualError as error:
         raise _CheckPolarsDataFrameColumnsError(df=df, columns=columns) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameColumnsError(CheckPolarsDataFrameError):
-    columns: IterableStrs
+    columns: Sequence[str]
 
     @override
     def __str__(self) -> str:
@@ -180,7 +254,7 @@ def _check_polars_dataframe_dtypes(
         raise _CheckPolarsDataFrameDTypesError(df=df, dtypes=dtypes) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameDTypesError(CheckPolarsDataFrameError):
     dtypes: Iterable[PolarsDataType]
 
@@ -203,7 +277,7 @@ def _check_polars_dataframe_height(
         raise _CheckPolarsDataFrameHeightError(df=df) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameHeightError(CheckPolarsDataFrameError):
     @override
     def __str__(self) -> str:
@@ -229,7 +303,7 @@ def _check_polars_dataframe_predicates(
         )
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFramePredicatesError(CheckPolarsDataFrameError):
     predicates: Mapping[str, Callable[[Any], bool]]
     missing: AbstractSet[str]
@@ -264,7 +338,7 @@ def _check_polars_dataframe_schema_list(df: DataFrame, schema: SchemaDict, /) ->
         raise _CheckPolarsDataFrameSchemaListError(df=df, schema=schema) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameSchemaListError(CheckPolarsDataFrameError):
     schema: SchemaDict
 
@@ -280,7 +354,7 @@ def _check_polars_dataframe_schema_set(df: DataFrame, schema: SchemaDict, /) ->
         raise _CheckPolarsDataFrameSchemaSetError(df=df, schema=schema) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameSchemaSetError(CheckPolarsDataFrameError):
     schema: SchemaDict
 
@@ -296,7 +370,7 @@ def _check_polars_dataframe_schema_subset(df: DataFrame, schema: SchemaDict, /)
         raise _CheckPolarsDataFrameSchemaSubsetError(df=df, schema=schema) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameSchemaSubsetError(CheckPolarsDataFrameError):
     schema: SchemaDict
 
@@ -310,7 +384,7 @@ def _check_polars_dataframe_shape(df: DataFrame, shape: tuple[int, int], /) -> N
         raise _CheckPolarsDataFrameShapeError(df=df, shape=shape) from None
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameShapeError(CheckPolarsDataFrameError):
     shape: tuple[int, int]
 
@@ -334,7 +408,7 @@ def _check_polars_dataframe_sorted(
         raise _CheckPolarsDataFrameSortedError(df=df, by=by_use) from error
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameSortedError(CheckPolarsDataFrameError):
     by: IntoExpr | list[IntoExpr]
 
@@ -353,7 +427,7 @@ def _check_polars_dataframe_unique(
         raise _CheckPolarsDataFrameUniqueError(df=df, by=by_use)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameUniqueError(CheckPolarsDataFrameError):
     by: IntoExpr | list[IntoExpr]
 
@@ -367,7 +441,7 @@ def _check_polars_dataframe_width(df: DataFrame, width: int, /) -> None:
         raise _CheckPolarsDataFrameWidthError(df=df, width=width)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _CheckPolarsDataFrameWidthError(CheckPolarsDataFrameError):
     width: int
 
@@ -378,37 +452,6 @@ class _CheckPolarsDataFrameWidthError(CheckPolarsDataFrameError):
         )
 
 
-def check_zoned_dtype_or_series(dtype_or_series: DataType | Series, /) -> None:
-    """Check if a dtype/series is a zoned datetime."""
-    if isinstance(dtype_or_series, DataType):
-        dtype = dtype_or_series
-    else:
-        dtype = dtype_or_series.dtype
-    if not isinstance(dtype, Datetime):
-        raise _CheckZonedDTypeOrSeriesNotDatetimeError(dtype=dtype)
-    if dtype.time_zone is None:
-        raise _CheckZonedDTypeOrSeriesNotZonedError(dtype=dtype)
-
-
-@dataclass(kw_only=True)
-class CheckZonedDTypeOrSeriesError(Exception):
-    dtype: DataType
-
-
-@dataclass(kw_only=True)
-class _CheckZonedDTypeOrSeriesNotDatetimeError(CheckZonedDTypeOrSeriesError):
-    @override
-    def __str__(self) -> str:
-        return f"Data type must be Datetime; got {self.dtype}"
-
-
-@dataclass(kw_only=True)
-class _CheckZonedDTypeOrSeriesNotZonedError(CheckZonedDTypeOrSeriesError):
-    @override
-    def __str__(self) -> str:
-        return f"Data type must be zoned; got {self.dtype}"
-
-
 def collect_series(expr: Expr, /) -> Series:
     """Collect a column expression into a Series."""
     data = DataFrame().with_columns(expr)
@@ -424,7 +467,7 @@ def columns_to_dict(df: DataFrame, key: str, value: str, /) -> dict[Any, Any]:
     return dict(zip(col_key, col_value, strict=True))
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class ColumnsToDictError(Exception):
     df: DataFrame
     key: str
@@ -434,6 +477,69 @@ class ColumnsToDictError(Exception):
         return f"DataFrame must be unique on {self.key!r}\n\n{self.df}"
 
 
+@overload
+def convert_time_zone(obj: Series, /, *, time_zone: ZoneInfo | str = ...) -> Series: ...
+@overload
+def convert_time_zone(
+    obj: DataFrame, /, *, time_zone: ZoneInfo | str = ...
+) -> DataFrame: ...
+def convert_time_zone(
+    obj: Series | DataFrame, /, *, time_zone: ZoneInfo | str = UTC
+) -> Series | DataFrame:
+    """Convert the time zone(s) of a Series or Dataframe."""
+    match obj:
+        case Series() as series:
+            return _convert_time_zone_series(series, time_zone=time_zone)
+        case DataFrame() as df:
+            return df.select(
+                *(
+                    _convert_time_zone_series(df[c], time_zone=time_zone)
+                    for c in df.columns
+                )
+            )
+        case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+            assert_never(never)
+
+
+def _convert_time_zone_series(
+    sr: Series, /, *, time_zone: ZoneInfo | str = UTC
+) -> Series:
+    if isinstance(sr.dtype, Struct):
+        df = sr.struct.unnest()
+        df = convert_time_zone(df, time_zone=time_zone).select(
+            struct("*").alias(sr.name)
+        )
+        return df[sr.name]
+    if isinstance(sr.dtype, Datetime):
+        return sr.dt.convert_time_zone(get_time_zone_name(time_zone))
+    return sr
+
+
+def dataclass_to_row(obj: Dataclass, /) -> DataFrame:
+    """Convert a dataclass into a 1-row DataFrame."""
+    try:
+        df = DataFrame([obj], orient="row")
+    except NameError:
+        df = DataFrame([asdict_without_defaults(obj, recursive=True)], orient="row")
+    return reduce(partial(_dataclass_to_row_reducer, obj=obj), df.columns, df)
+
+
+def _dataclass_to_row_reducer(
+    df: DataFrame, column: str, /, *, obj: Dataclass
+) -> DataFrame:
+    dtype = df[column].dtype
+    if isinstance(dtype, Datetime):
+        datetime = ensure_datetime(getattr(obj, column), nullable=True)
+        if (datetime is None) or (datetime.tzinfo is None):
+            return df
+        time_zone = ensure_time_zone(datetime.tzinfo)
+        return df.with_columns(col(column).cast(zoned_datetime(time_zone=time_zone)))
+    if isinstance(dtype, Struct):
+        inner = dataclass_to_row(getattr(obj, column))
+        return df.with_columns(**{column: inner.select(all=struct("*"))["all"]})
+    return df
+
+
 def drop_null_struct_series(series: Series, /) -> Series:
     """Drop nulls in a struct-dtype Series as per the <= 1.1 definition."""
     try:
@@ -443,7 +549,7 @@ def drop_null_struct_series(series: Series, /) -> Series:
     return series.filter(is_not_null)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class DropNullStructSeriesError(Exception):
     series: Series
 
@@ -479,6 +585,40 @@ def floor_datetime(column: IntoExprColumn, every: Expr | str, /) -> Expr | Serie
     return DataFrame().with_columns(floor.alias(column.name))[column.name]
 
 
+def get_data_type_or_series_time_zone(
+    dtype_or_series: DataType | Series, /
+) -> ZoneInfo:
+    """Get the time zone of a dtype/series."""
+    if isinstance(dtype_or_series, DataType):
+        dtype = dtype_or_series
+    else:
+        dtype = dtype_or_series.dtype
+    if not isinstance(dtype, Datetime):
+        raise _GetDataTypeOrSeriesTimeZoneNotDatetimeError(dtype=dtype)
+    if dtype.time_zone is None:
+        raise _GetDataTypeOrSeriesTimeZoneNotZonedError(dtype=dtype)
+    return ZoneInfo(dtype.time_zone)
+
+
+@dataclass(kw_only=True, slots=True)
+class GetDataTypeOrSeriesTimeZoneError(Exception):
+    dtype: DataType
+
+
+@dataclass(kw_only=True, slots=True)
+class _GetDataTypeOrSeriesTimeZoneNotDatetimeError(GetDataTypeOrSeriesTimeZoneError):
+    @override
+    def __str__(self) -> str:
+        return f"Data type must be Datetime; got {self.dtype}"
+
+
+@dataclass(kw_only=True, slots=True)
+class _GetDataTypeOrSeriesTimeZoneNotZonedError(GetDataTypeOrSeriesTimeZoneError):
+    @override
+    def __str__(self) -> str:
+        return f"Data type must be zoned; got {self.dtype}"
+
+
 def is_not_null_struct_series(series: Series, /) -> Series:
     """Check if a struct-dtype Series is not null as per the <= 1.1 definition."""
     try:
@@ -487,7 +627,7 @@ def is_not_null_struct_series(series: Series, /) -> Series:
         raise IsNotNullStructSeriesError(series=error.series) from None
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class IsNotNullStructSeriesError(Exception):
     series: Series
 
@@ -510,7 +650,7 @@ def is_null_struct_series(series: Series, /) -> Series:
 
 
 def _is_null_struct_series_one(
-    dtype: Struct, /, *, root: Sequence[str] = ()
+    dtype: Struct, /, *, root: Iterable[str] = ()
 ) -> Iterator[Sequence[str]]:
     for field in dtype.fields:
         name = field.name
@@ -522,7 +662,7 @@ def _is_null_struct_series_one(
             yield path
 
 
-def _is_null_struct_to_expr(path: Sequence[str], /) -> Expr:
+def _is_null_struct_to_expr(path: Iterable[str], /) -> Expr:
     head, *tail = path
     return reduce(_is_null_struct_to_expr_reducer, tail, col(head)).is_null()
 
@@ -531,7 +671,7 @@ def _is_null_struct_to_expr_reducer(expr: Expr, path: str, /) -> Expr:
     return expr.struct[path]
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class IsNullStructSeriesError(Exception):
     series: Series
 
@@ -543,14 +683,15 @@ class IsNullStructSeriesError(Exception):
 def join(
     df: DataFrame,
     *dfs: DataFrame,
-    on: str | Expr | Sequence[str | Expr],
+    on: MaybeIterable[str | Expr],
     how: JoinStrategy = "inner",
     validate: JoinValidation = "m:m",
 ) -> DataFrame:
     """Join a set of DataFrames."""
+    on_use = on if isinstance(on, str | Expr) else list(on)
 
     def inner(left: DataFrame, right: DataFrame, /) -> DataFrame:
-        return left.join(right, on=on, how=how, validate=validate)
+        return left.join(right, on=on_use, how=how, validate=validate)
 
     return reduce(inner, chain([df], dfs))
 
@@ -588,43 +729,207 @@ def nan_sum_cols(
     return reduce(func, all_exprs)
 
 
-@contextmanager
-def redirect_empty_polars_concat() -> Iterator[None]:
-    """Redirect to the `EmptyPolarsConcatError`."""
-    with redirect_error(
-        ValueError, EmptyPolarsConcatError, match="cannot concat empty list"
+@overload
+def replace_time_zone(
+    obj: Series, /, *, time_zone: ZoneInfo | str | None = ...
+) -> Series: ...
+@overload
+def replace_time_zone(
+    obj: DataFrame, /, *, time_zone: ZoneInfo | str | None = ...
+) -> DataFrame: ...
+def replace_time_zone(
+    obj: Series | DataFrame, /, *, time_zone: ZoneInfo | str | None = UTC
+) -> Series | DataFrame:
+    """Replace the time zone(s) of a Series or Dataframe."""
+    match obj:
+        case Series() as series:
+            return _replace_time_zone_series(series, time_zone=time_zone)
+        case DataFrame() as df:
+            return df.select(
+                *(
+                    _replace_time_zone_series(df[c], time_zone=time_zone)
+                    for c in df.columns
+                )
+            )
+        case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+            assert_never(never)
+
+
+def _replace_time_zone_series(
+    sr: Series, /, *, time_zone: ZoneInfo | str | None = UTC
+) -> Series:
+    if isinstance(sr.dtype, Struct):
+        df = sr.struct.unnest()
+        df = replace_time_zone(df, time_zone=time_zone).select(
+            struct("*").alias(sr.name)
+        )
+        return df[sr.name]
+    if isinstance(sr.dtype, Datetime):
+        time_zone_use = None if time_zone is None else get_time_zone_name(time_zone)
+        return sr.dt.replace_time_zone(time_zone_use)
+    return sr
+
+
+@overload
+def rolling_parameters(
+    *,
+    s_window: int,
+    e_com: None = None,
+    e_span: None = None,
+    e_half_life: None = None,
+    e_alpha: None = None,
+    min_periods: int | None = None,
+) -> RollingParametersSimple: ...
+@overload
+def rolling_parameters(
+    *,
+    s_window: None = None,
+    e_com: float | None = None,
+    e_span: float | None = None,
+    e_half_life: float | None = None,
+    e_alpha: float | None = None,
+    min_periods: int,
+) -> RollingParametersExponential: ...
+@overload
+def rolling_parameters(
+    *,
+    s_window: int | None = None,
+    e_com: float | None = None,
+    e_span: float | None = None,
+    e_half_life: float | None = None,
+    e_alpha: float | None = None,
+    min_periods: int | None = None,
+) -> RollingParametersSimple | RollingParametersExponential: ...
+def rolling_parameters(
+    *,
+    s_window: int | None = None,
+    e_com: float | None = None,
+    e_span: float | None = None,
+    e_half_life: float | None = None,
+    e_alpha: float | None = None,
+    min_periods: int | None = None,
+) -> RollingParametersSimple | RollingParametersExponential:
+    """Resolve a set of rolling parameters."""
+    if (
+        (s_window is not None)
+        and (e_com is None)
+        and (e_span is None)
+        and (e_half_life is None)
+        and (e_alpha is None)
     ):
-        yield
+        return RollingParametersSimple(window=s_window, min_periods=min_periods)
+    if (s_window is None) and (
+        (e_com is not None)
+        or (e_span is not None)
+        or (e_half_life is not None)
+        or (e_alpha is not None)
+    ):
+        if min_periods is None:
+            raise _RollingParametersMinPeriodsError(
+                e_com=e_com,
+                e_span=e_span,
+                e_half_life=e_half_life,
+                e_alpha=e_alpha,
+                min_periods=min_periods,
+            )
+        params = ewm_parameters(
+            com=e_com, span=e_span, half_life=e_half_life, alpha=e_alpha
+        )
+        return RollingParametersExponential(
+            com=params.com,
+            span=params.span,
+            half_life=params.half_life,
+            alpha=params.alpha,
+            min_periods=min_periods,
+        )
+    raise _RollingParametersArgumentsError(
+        s_window=s_window,
+        e_com=e_com,
+        e_span=e_span,
+        e_half_life=e_half_life,
+        e_alpha=e_alpha,
+    )
 
 
-class EmptyPolarsConcatError(Exception): ...
+@dataclass(kw_only=True, slots=True)
+class RollingParametersError(Exception):
+    s_window: int | None = None
+    e_com: float | None = None
+    e_span: float | None = None
+    e_half_life: float | None = None
+    e_alpha: float | None = None
+
+
+@dataclass(kw_only=True, slots=True)
+class _RollingParametersArgumentsError(RollingParametersError):
+    @override
+    def __str__(self) -> str:
+        return f"Exactly one of simple window, exponential center of mass (γ), exponential span (θ), exponential half-life (λ) or exponential smoothing factor (α) must be given; got s_window={self.s_window}, γ={self.e_com}, θ={self.e_span}, λ={self.e_half_life} and α={self.e_alpha}"  # noqa: RUF001
+
+
+@dataclass(kw_only=True, slots=True)
+class _RollingParametersMinPeriodsError(RollingParametersError):
+    min_periods: int | None = None
+
+    @override
+    def __str__(self) -> str:
+        return f"Exponential rolling requires 'min_periods' to be set; got {self.min_periods}"
+
+
+@dataclass(kw_only=True, slots=True)
+class RollingParametersSimple:
+    window: int
+    min_periods: int | None = None
+
+
+@dataclass(kw_only=True, slots=True)
+class RollingParametersExponential(_EWMParameters):
+    min_periods: int
 
 
 def set_first_row_as_columns(df: DataFrame, /) -> DataFrame:
     """Set the first row of a DataFrame as its columns."""
-    with redirect_error(OutOfBoundsError, SetFirstRowAsColumnsError(f"{df=}")):
+    try:
         row = df.row(0)
+    except OutOfBoundsError:
+        raise SetFirstRowAsColumnsError(df=df) from None
     mapping = dict(zip(df.columns, row, strict=True))
     return df[1:].rename(mapping)
 
 
-class SetFirstRowAsColumnsError(Exception): ...
+@dataclass(kw_only=True, slots=True)
+class SetFirstRowAsColumnsError(Exception):
+    df: DataFrame
+
+    @override
+    def __str__(self) -> str:
+        return f"DataFrame must have at least 1 row; got {self.df}"
 
 
-def struct_data_type(
-    cls: type[Dataclass], /, *, time_zone: ZoneInfo | str | None = None
+def struct_dtype(**kwargs: PolarsDataType) -> Struct:
+    """Construct a Struct data type from a set of keyword arguments."""
+    return Struct(kwargs)
+
+
+def struct_from_dataclass(
+    cls: type[Dataclass],
+    /,
+    *,
+    globalns: StrMapping | None = None,
+    localns: StrMapping | None = None,
+    time_zone: ZoneInfo | str | None = None,
 ) -> Struct:
     """Construct the Struct data type for a dataclass."""
     if not is_dataclass_class(cls):
-        raise _StructDataTypeNotADataclassError(cls=cls)
-    anns = get_type_hints(cls)
+        raise _StructFromDataClassNotADataclassError(cls=cls)
+    anns = get_type_hints(cls, globalns=globalns, localns=localns)
     data_types = {
-        k: _struct_data_type_one(v, time_zone=time_zone) for k, v in anns.items()
+        k: _struct_from_dataclass_one(v, time_zone=time_zone) for k, v in anns.items()
     }
     return Struct(data_types)
 
 
-def _struct_data_type_one(
+def _struct_from_dataclass_one(
     ann: Any, /, *, time_zone: ZoneInfo | str | None = None
 ) -> PolarsDataType:
     mapping = {bool: Boolean, dt.date: Date, float: Float64, int: Int64, str: Utf8}
@@ -632,27 +937,27 @@ def _struct_data_type_one(
         return mapping[ann]
     if ann is dt.datetime:
         if time_zone is None:
-            raise _StructDataTypeTimeZoneMissingError
+            raise _StructFromDataClassTimeZoneMissingError
         return zoned_datetime(time_zone=time_zone)
     if is_dataclass_class(ann):
-        return struct_data_type(ann, time_zone=time_zone)
+        return struct_from_dataclass(ann, time_zone=time_zone)
     if (isinstance(ann, type) and issubclass(ann, Enum)) or (
         is_literal_type(ann) and all(isinstance(a, str) for a in get_args(ann))
     ):
         return Utf8
     if is_optional_type(ann):
-        return _struct_data_type_one(one(get_args(ann)), time_zone=time_zone)
+        return _struct_from_dataclass_one(one(get_args(ann)), time_zone=time_zone)
     if is_frozenset_type(ann) or is_list_type(ann) or is_set_type(ann):
-        return List(_struct_data_type_one(one(get_args(ann)), time_zone=time_zone))
-    raise _StructDataTypeTypeError(ann=ann)
+        return List(_struct_from_dataclass_one(one(get_args(ann)), time_zone=time_zone))
+    raise _StructFromDataClassTypeError(ann=ann)
 
 
-@dataclass(kw_only=True)
-class StructDataTypeError(Exception): ...
+@dataclass(kw_only=True, slots=True)
+class StructFromDataClassError(Exception): ...
 
 
-@dataclass(kw_only=True)
-class _StructDataTypeNotADataclassError(StructDataTypeError):
+@dataclass(kw_only=True, slots=True)
+class _StructFromDataClassNotADataclassError(StructFromDataClassError):
     cls: type[Dataclass]
 
     @override
@@ -660,15 +965,15 @@ class _StructDataTypeNotADataclassError(StructDataTypeError):
         return f"Object must be a dataclass; got {self.cls}"
 
 
-@dataclass(kw_only=True)
-class _StructDataTypeTimeZoneMissingError(StructDataTypeError):
+@dataclass(kw_only=True, slots=True)
+class _StructFromDataClassTimeZoneMissingError(StructFromDataClassError):
     @override
     def __str__(self) -> str:
         return "Time-zone must be given"
 
 
-@dataclass(kw_only=True)
-class _StructDataTypeTypeError(StructDataTypeError):
+@dataclass(kw_only=True, slots=True)
+class _StructFromDataClassTypeError(StructFromDataClassError):
     ann: Any
 
     @override
@@ -676,6 +981,111 @@ class _StructDataTypeTypeError(StructDataTypeError):
         return f"Unsupported type: {self.ann}"
 
 
+def yield_rows_as_dataclasses(
+    df: DataFrame,
+    cls: type[_TDataclass],
+    /,
+    *,
+    check_types: Literal["none", "first", "all"] = "first",
+    globalns: StrMapping | None = None,
+    localns: StrMapping | None = None,
+) -> Iterator[_TDataclass]:
+    """Yield the rows of a DataFrame as dataclasses."""
+    from dacite import Config, from_dict
+    from dacite.exceptions import WrongTypeError
+
+    from utilities.dacite import yield_literal_forward_references
+
+    columns = df.columns
+    required: set[str] = set()
+    for field in fields(cls):
+        if isinstance(field.default, _MISSING_TYPE) and isinstance(
+            field.default_factory, _MISSING_TYPE
+        ):
+            required.add(field.name)
+    try:
+        check_superset(columns, required)
+    except CheckSuperSetError as error:
+        raise _YieldRowsAsDataClassesColumnsSuperSetError(
+            df=df, cls=cls, left=error.left, right=error.right, extra=error.extra
+        ) from None
+    rows = df.iter_rows(named=True)
+    match check_types:
+        case "none":
+            yield from _yield_rows_as_dataclasses_no_check_types(rows, cls)
+        case "first":
+            try:
+                first = next(rows)
+            except StopIteration:
+                return
+            fwd_refs = dict(
+                yield_literal_forward_references(
+                    cls, globalns=globalns, localns=localns
+                )
+            )
+            config = Config(forward_references=fwd_refs)
+            try:
+                yield from_dict(cls, first, config=config)
+            except WrongTypeError as error:
+                raise _YieldRowsAsDataClassesWrongTypeError(
+                    df=df, cls=cls, msg=str(error)
+                ) from None
+            yield from _yield_rows_as_dataclasses_no_check_types(rows, cls)
+        case "all":
+            fwd_refs = dict(
+                yield_literal_forward_references(
+                    cls, globalns=globalns, localns=localns
+                )
+            )
+            config = Config(forward_references=fwd_refs)
+            try:
+                for row in rows:
+                    yield from_dict(cls, row, config=config)
+            except WrongTypeError as error:
+                raise _YieldRowsAsDataClassesWrongTypeError(
+                    df=df, cls=cls, msg=str(error)
+                ) from None
+        case _ as never:  # pyright: ignore[reportUnnecessaryComparison]
+            assert_never(never)
+
+
+def _yield_rows_as_dataclasses_no_check_types(
+    rows: Iterator[dict[str, Any]], cls: type[_TDataclass], /
+) -> Iterator[_TDataclass]:
+    """Yield the rows of a DataFrame as dataclasses without type checking."""
+    from dacite import Config, from_dict
+
+    config = Config(check_types=False)
+    for row in rows:
+        yield from_dict(cls, row, config=config)
+
+
+@dataclass(kw_only=True, slots=True)
+class YieldRowsAsDataClassesError(Exception):
+    df: DataFrame
+    cls: type[Dataclass]
+
+
+@dataclass(kw_only=True, slots=True)
+class _YieldRowsAsDataClassesColumnsSuperSetError(YieldRowsAsDataClassesError):
+    left: AbstractSet[str]
+    right: AbstractSet[str]
+    extra: AbstractSet[str]
+
+    @override
+    def __str__(self) -> str:
+        return f"DataFrame columns {reprlib.repr(self.left)} must be a superset of dataclass fields {reprlib.repr(self.right)}; dataclass had extra fields {reprlib.repr(self.extra)}."
+
+
+@dataclass(kw_only=True, slots=True)
+class _YieldRowsAsDataClassesWrongTypeError(YieldRowsAsDataClassesError):
+    msg: str
+
+    @override
+    def __str__(self) -> str:
+        return self.msg
+
+
 @overload
 def yield_struct_series_elements(
     series: Series, /, *, strict: Literal[True]
@@ -715,34 +1125,32 @@ def _yield_struct_series_element_is_mapping_of_str(
     return isinstance(obj, Mapping) and all(isinstance(k, str) for k in obj)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class YieldStructSeriesElementsError(Exception):
     series: Series
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _YieldStructSeriesElementsDTypeError(YieldStructSeriesElementsError):
     @override
     def __str__(self) -> str:
         return f"Series must have Struct-dtype; got {self.series.dtype}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _YieldStructSeriesElementsNullElementsError(YieldStructSeriesElementsError):
     @override
     def __str__(self) -> str:
         return f"Series must not have nulls; got {self.series}"
 
 
-_TDataclass = TypeVar("_TDataclass", bound=Dataclass)
-
-
 @overload
 def yield_struct_series_dataclasses(
     series: Series,
     cls: type[_TDataclass],
     /,
     *,
+    forward_references: dict[str, Any] | None = ...,
     check_types: bool = ...,
     strict: Literal[True],
 ) -> Iterator[_TDataclass]: ...
@@ -752,6 +1160,7 @@ def yield_struct_series_dataclasses(
     cls: type[_TDataclass],
     /,
     *,
+    forward_references: dict[str, Any] | None = ...,
     check_types: bool = ...,
     strict: bool = False,
 ) -> Iterator[_TDataclass | None]: ...
@@ -760,13 +1169,16 @@ def yield_struct_series_dataclasses(
     cls: type[_TDataclass],
     /,
     *,
+    forward_references: dict[str, Any] | None = None,
     check_types: bool = True,
     strict: bool = False,
 ) -> Iterator[_TDataclass | None]:
     """Yield the elements of a struct-dtype Series as dataclasses."""
     from dacite import Config, from_dict
 
-    config = Config(check_types=check_types, strict=True)
+    config = Config(
+        forward_references=forward_references, check_types=check_types, strict=True
+    )
     for value in yield_struct_series_elements(series, strict=strict):
         yield None if value is None else from_dict(cls, value, config=config)
 
@@ -780,30 +1192,45 @@ def zoned_datetime(
 
 __all__ = [
     "CheckPolarsDataFrameError",
-    "CheckZonedDTypeOrSeriesError",
     "ColumnsToDictError",
+    "DatetimeHongKong",
+    "DatetimeTokyo",
+    "DatetimeUSCentral",
+    "DatetimeUSEastern",
     "DatetimeUTC",
     "DropNullStructSeriesError",
-    "EmptyPolarsConcatError",
+    "GetDataTypeOrSeriesTimeZoneError",
     "IsNullStructSeriesError",
+    "RollingParametersError",
+    "RollingParametersExponential",
+    "RollingParametersSimple",
     "SetFirstRowAsColumnsError",
+    "StructFromDataClassError",
+    "YieldRowsAsDataClassesError",
     "YieldStructSeriesElementsError",
+    "append_dataclass",
+    "are_frames_equal",
     "ceil_datetime",
     "check_polars_dataframe",
-    "check_zoned_dtype_or_series",
     "collect_series",
     "columns_to_dict",
+    "convert_time_zone",
+    "dataclass_to_row",
     "drop_null_struct_series",
     "ensure_expr_or_series",
     "floor_datetime",
+    "get_data_type_or_series_time_zone",
     "is_not_null_struct_series",
     "is_null_struct_series",
     "join",
     "nan_sum_agg",
     "nan_sum_cols",
-    "redirect_empty_polars_concat",
+    "replace_time_zone",
+    "rolling_parameters",
     "set_first_row_as_columns",
-    "struct_data_type",
+    "struct_dtype",
+    "struct_from_dataclass",
+    "yield_rows_as_dataclasses",
     "yield_struct_series_dataclasses",
     "yield_struct_series_elements",
     "zoned_datetime",
