@@ -1,7 +1,7 @@
 from __future__ import annotations
 
+import re
 from dataclasses import dataclass
-from re import compile
 
 from typing_extensions import override
 
@@ -13,7 +13,7 @@ def extract_group(pattern: str, text: str, /, *, flags: int = 0) -> str:
 
     The regex must have 1 capture group, and this must match exactly once.
     """
-    compiled = compile(pattern, flags=flags)
+    compiled = re.compile(pattern, flags=flags)
     match compiled.groups:
         case 0:
             raise _ExtractGroupNoCaptureGroupsError(pattern=pattern, text=text)
@@ -31,20 +31,20 @@ def extract_group(pattern: str, text: str, /, *, flags: int = 0) -> str:
             raise _ExtractGroupMultipleCaptureGroupsError(pattern=pattern, text=text)
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class ExtractGroupError(Exception):
     pattern: str
     text: str
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupMultipleCaptureGroupsError(ExtractGroupError):
     @override
     def __str__(self) -> str:
         return f"Pattern {self.pattern} must contain exactly one capture group; it had multiple"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupMultipleMatchesError(ExtractGroupError):
     matches: list[str]
 
@@ -53,7 +53,7 @@ class _ExtractGroupMultipleMatchesError(ExtractGroupError):
         return f"Pattern {self.pattern} must match against {self.text} exactly once; matches were {self.matches}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupNoCaptureGroupsError(ExtractGroupError):
     @override
     def __str__(self) -> str:
@@ -62,7 +62,7 @@ class _ExtractGroupNoCaptureGroupsError(ExtractGroupError):
         )
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupNoMatchesError(ExtractGroupError):
     @override
     def __str__(self) -> str:
@@ -75,7 +75,7 @@ def extract_groups(pattern: str, text: str, /, *, flags: int = 0) -> list[str]:
     The regex may have any number of capture groups, and they must collectively
     match exactly once.
     """
-    compiled = compile(pattern, flags=flags)
+    compiled = re.compile(pattern, flags=flags)
     if (n_groups := compiled.groups) == 0:
         raise _ExtractGroupsNoCaptureGroupsError(pattern=pattern, text=text)
     matches: list[str] = compiled.findall(text)
@@ -92,13 +92,13 @@ def extract_groups(pattern: str, text: str, /, *, flags: int = 0) -> list[str]:
             )
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class ExtractGroupsError(Exception):
     pattern: str
     text: str
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupsMultipleMatchesError(ExtractGroupsError):
     matches: list[str]
 
@@ -107,7 +107,7 @@ class _ExtractGroupsMultipleMatchesError(ExtractGroupsError):
         return f"Pattern {self.pattern} must match against {self.text} exactly once; matches were {self.matches}"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupsNoCaptureGroupsError(ExtractGroupsError):
     pattern: str
     text: str
@@ -117,7 +117,7 @@ class _ExtractGroupsNoCaptureGroupsError(ExtractGroupsError):
         return f"Pattern {self.pattern} must contain at least one capture group"
 
 
-@dataclass(kw_only=True)
+@dataclass(kw_only=True, slots=True)
 class _ExtractGroupsNoMatchesError(ExtractGroupsError):
     @override
     def __str__(self) -> str:
